Package: betaARMA
Title: Beta Autoregressive Moving Average Models
Version: 1.2.0
Date: 2026-04-22
Authors@R: c(
  person(given = "Everton",
         family = "da Costa",
         email = "everto.cost@gmail.com",
         role = c("aut", "cre"),
         comment = c(ORCID = "0000-0001-7580-2639")),
  person(given = "Francisco",
         family = "Cribari-Neto",
         email = "francisco.cribari@ufpe.br",
         role = c("ctb", "ths"),
         comment = c(ORCID = "0000-0002-5909-6698", "Theoretical foundations")),
  person(given = "Vinicius",
         family = "Scher",
         role = "ctb",
         comment = c(ORCID = "0000-0003-0406-0265"))
  )
Description: Fits Beta Autoregressive Moving Average (BARMA) models for time series data distributed in the standard unit interval (0, 1). The estimation is performed via the conditional maximum likelihood method using the Broyden-Fletcher-Goldfarb-Shanno (BFGS) quasi-Newton algorithm. A ridge penalization scheme is available to improve numerical stability of the estimation, as proposed by Cribari-Neto, Costa and Fonseca (2025) <doi:10.1214/25-BJPS645>. The package includes tools for model fitting, diagnostic checking, and forecasting, along with two hydro-environmental datasets from Brazil. Based on the work of Rocha and Cribari-Neto (2009) <doi:10.1007/s11749-008-0112-z> and the associated erratum Rocha and Cribari-Neto (2017) <doi:10.1007/s11749-017-0528-4>. The original code was developed by Fabio M. Bayer.
License: MIT + file LICENSE
URL: https://github.com/Everton-da-Costa/betaARMA
BugReports: https://github.com/Everton-da-Costa/betaARMA/issues
Encoding: UTF-8
RoxygenNote: 7.3.2
Language: en-US
Imports: forecast, ggplot2, rlang, gridExtra
Suggests: knitr, zoo, xtable, here, moments, rmarkdown, tseries, lbfgs,
        dplyr, scales, testthat (>= 3.0.0)
VignetteBuilder: knitr
Depends: R (>= 3.5)
LazyData: true
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-23 17:12:22 UTC; cost
Author: Everton da Costa [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7580-2639>),
  Francisco Cribari-Neto [ctb, ths] (ORCID:
    <https://orcid.org/0000-0002-5909-6698>, Theoretical foundations),
  Vinicius Scher [ctb] (ORCID: <https://orcid.org/0000-0003-0406-0265>)
Maintainer: Everton da Costa <everto.cost@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-23 22:30:02 UTC
Built: R 4.6.0; ; 2026-05-25 03:28:31 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(coef,barma)
S3method(fitted,barma)
S3method(forecast,barma)
S3method(plot,barma)
S3method(print,barma)
S3method(print,summary.barma)
S3method(residuals,barma)
S3method(summary,barma)
export(barma)
export(fim_barma)
export(loglik_barma)
export(make_link_structure)
export(score_vector_barma)
export(simu_barma)
export(start_values)
importFrom(forecast,forecast)
importFrom(forecast,ggAcf)
importFrom(forecast,ggPacf)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,stat_qq)
importFrom(ggplot2,stat_qq_line)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(grid,grid.draw)
importFrom(gridExtra,arrangeGrob)
importFrom(rlang,.data)
importFrom(stats,acf)
importFrom(stats,coef)
importFrom(stats,dbeta)
importFrom(stats,density)
importFrom(stats,fitted)
importFrom(stats,frequency)
importFrom(stats,is.ts)
importFrom(stats,lm.fit)
importFrom(stats,make.link)
importFrom(stats,optim)
importFrom(stats,pacf)
importFrom(stats,pbeta)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rbeta)
importFrom(stats,residuals)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,ts)
importFrom(stats,var)

# betaARMA: Beta Autoregressive Moving Average Models

[![CRAN
status](https://www.r-pkg.org/badges/version/betaARMA)](https://CRAN.R-project.org/package=betaARMA)
[![DOI](https://img.shields.io/badge/DOI-10.32614%2FCRAN.package.betaARMA-blue)](https://doi.org/10.32614/CRAN.package.betaARMA)
[![CRAN
downloads](https://cranlogs.r-pkg.org/badges/betaARMA)](https://CRAN.R-project.org/package=betaARMA)
[![CRAN all-time
downloads](https://cranlogs.r-pkg.org/badges/grand-total/betaARMA)](https://CRAN.R-project.org/package=betaARMA)
[![R-CMD-check](https://github.com/Everton-da-Costa/betaARMA/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Everton-da-Costa/betaARMA/actions/workflows/R-CMD-check.yaml)
[![R-hub](https://github.com/Everton-da-Costa/betaARMA/actions/workflows/rhub.yaml/badge.svg)](https://github.com/Everton-da-Costa/betaARMA/actions/workflows/rhub.yaml)
[![License:
MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)

The `betaARMA` package provides a comprehensive suite of tools for
fitting, diagnosing, and forecasting Beta Autoregressive Moving Average
models. These models are specifically tailored for time series data that
assume values in the standard unit interval (0, 1), such as rates,
proportions, and relative indices.

## Features

- **Estimation:** Supports both standard Conditional Maximum Likelihood
  Estimation (CMLE) and Penalized Conditional Maximum Likelihood
  Estimation (PCMLE) via ridge penalization to handle numerical
  instabilities and flat likelihood surfaces (Cribari-Neto, Costa, and
  Fonseca, 2025).
- **Diagnostics:** Highly optimized, fully vectorized implementations of
  the Ljung-Box and Monti portmanteau tests, correctly adjusted for the
  effective sample size and the number of estimated parameters.
- **Forecasting:** Built-in methods for out-of-sample predictions. True
  to the theoretical properties of the betaARMA model, all forecasts
  naturally remain strictly within the (0, 1) bounds.

------------------------------------------------------------------------

## What’s New in v1.2.0

- **Ridge Penalization:** New `penalty` argument in `barma()` enables
  Penalized CMLE (PCMLE) as proposed by Cribari-Neto, Costa, and Fonseca
  (2025). Addresses numerical instability caused by flat log-likelihood
  surfaces in challenging data structures.
- **Advanced Optimization Backends:** Users can now specify the
  optimization algorithm via the `optimization` argument in `barma()`.
  Supported methods include `stats::optim` (BFGS and L-BFGS-B with
  bounds) and `lbfgs::lbfgs`.
- **Enhanced Model Summaries:** The `summary()` method has been
  significantly upgraded. It now outputs significance levels (stars) for
  coefficients, explicitly details the optimization procedure and
  convergence status used, and reports the $\lambda$ penalty parameter
  when ridge penalization is applied.
- **Diagnostic Plots:** New `plot()` method for `barma` objects displays
  a comprehensive diagnostic grid.
- **New Vignette:** A new, comprehensive applied vignette is now
  included. It demonstrates the handling of numerical instability, the
  application of ridge penalization (PCMLE), and a full modeling
  workflow using monthly relative humidity data from Brasília, Brazil.

------------------------------------------------------------------------

## Table of Contents

- [Project Motivation](#-project-motivation)
- [Foundational Literature](#-foundational-literature)
- [Key Features](#-key-features)
- [Repository Structure](#-repository-structure)
- [Installation](#️-installation)
- [Getting Started](#-getting-started)
- [Citation](#-citation)
- [Contributing](#-contributing)
- [License](#-license)
- [Contact](#-contact)

------------------------------------------------------------------------

## Project Motivation

Modeling time series data bounded within the unit interval $(0, 1)$
presents unique challenges. Standard Gaussian methods (like ARIMA) are
often inappropriate because they do not respect the natural boundaries
of the data, potentially leading to fitted values or forecasts outside
the admissible range.

The **$\beta\text{ARMA}$ model** addresses this by assuming the
conditional distribution of the variable follows a Beta law. While the
theoretical foundations exist, there has been a need for a modern,
robust R package that:

1.  **Unifies the workflow** for $\beta$AR, $\beta$MA, and $\beta$ARMA
    specifications.
2.  **Ensures stability** through numerical optimization with analytic
    gradients, including ridge penalization for challenging data
    structures.
3.  **Provides a standard API** consistent with popular time series
    tools (like `forecast`).

This project aims to fill that gap, serving as a go-to resource for
hydrologists, economists, and data scientists working with bounded data.

------------------------------------------------------------------------

## Foundational Literature

The foundational methodology implemented in this package was published
in **TEST**, journal in the fields of Statistics and Probability.

[![SCImago Journal & Country
Rank](https://www.scimagojr.com/journal_img.php?id=14882)](https://www.scimagojr.com/journalsearch.php?q=14882&tip=sid)

**Key Publications:**

- **Rocha, A. V., & Cribari-Neto, F. (2009).** “Beta autoregressive
  moving average models.” *TEST*, 18(3), 529–545.
  [doi:10.1007/s11749-008-0112-z](https://doi.org/10.1007/s11749-008-0112-z)
- **Rocha, A. V., & Cribari-Neto, F. (2017).** “Erratum to: Beta
  autoregressive moving average models.” *TEST*, 26(2), 451–459.
  [doi:10.1007/s11749-017-0528-4](https://doi.org/10.1007/s11749-017-0528-4)
- **Cribari-Neto, F., Costa, E., & Fonseca, R. V. (2025).** “Numerical
  stability enhancements in beta autoregressive moving average model
  estimation.” *Brazilian Journal of Probability and Statistics*, 39(4),
  410–437. [doi:10.1214/25-BJPS645](https://doi.org/10.1214/25-BJPS645)
  — *Basis for the `penalty` argument in `barma()`.*
- **Costa, E., Cribari-Neto, F., & Scher, V. T. (2024).** “Test
  inferences and link function selection in dynamic beta modeling of
  seasonal hydro-environmental time series with temporary abnormal
  regimes.” *Journal of Hydrology*, 638, 131489.

------------------------------------------------------------------------

## Key Features

This package utilizes modern R development standards (S3 classes,
roxygen2 documentation) to provide a seamless user experience.

- **Unified Model Fitting:** A single core function, `barma()`, handles
  any combination of AR and MA lags, exogenous regressors (`xreg`), and
  ridge penalization (`penalty = TRUE`).
- **Estimation:** Implements Conditional Maximum Likelihood Estimation
  (CMLE) and Penalized CMLE (PCMLE). Users can select the numerical
  optimization backend (`BFGS`, `L-BFGS-B`, or `lbfgs`) utilizing
  **analytical gradients** (score vectors) for improved convergence and
  speed.
- **Correctness:** Fully implements the corrections to the Information
  Matrix and Score Vector detailed in the 2017 Erratum, for all
  supported link functions (`logit`, `cloglog`, `loglog`, `probit`).
- **Standard S3 Methods:** Works with standard R functions:
  - `summary()`: Detailed coefficients, significance levels (stars),
    optimization tracking, and penalization metrics.
  - `plot()`: Six-panel diagnostic grid (observed vs. fitted, residuals
    over time, ACF, PACF, Ljung-Box p-values, Monti p-values) via which
    = ‘all’, or a four-panel default view.
  - `forecast()`: Multi-step-ahead point forecasts.
  - `residuals()`: Extraction of Pearson, quantile, raw, or link-scale
    residuals.
  - `fitted()`: Extraction of fitted values.
- **Advanced Access:** Core mathematical functions (`loglik_barma()`,
  `score_vector_barma()`, `fim_barma()`) are exported for researchers
  who need direct access to gradients or model internals.

------------------------------------------------------------------------

## Repository Structure

The repository is structured as a standard R package for clarity and
reproducibility.

``` plaintext
.
├── R/                  # Source code for all R functions.
├── data/               # Processed data objects (e.g., brasilia_ts).
├── data-raw/           # Scripts and raw CSV files used to generate the datasets.
├── doc/                # Compiled HTML vignettes and extracted R code.
├── inst/               # Additional package files (CITATION, REFERENCES.bib).
├── man/                # R package documentation files (generated by roxygen2).
├── vignettes/          # R Markdown source files for vignettes.
├── DESCRIPTION         # Package metadata and dependencies.
├── NAMESPACE           # Manages the package's namespace (generated by roxygen2).
├── NEWS.md             # Package changelog.
├── LICENSE             # MIT License file.
├── betaARMA.Rproj      # RStudio project file.
└── README.md           # This file.
```

------------------------------------------------------------------------

## Installation

### CRAN (Recommended)

You can install the stable version of `betaARMA` directly from CRAN:

``` r
install.packages("betaARMA")
```

### Development Version

You can install the development version from GitHub using the remotes
package:

``` r
# install.packages("remotes")
remotes::install_github("Everton-da-Costa/betaARMA", dependencies = TRUE)
```

------------------------------------------------------------------------

## Usage and Vignettes

The package includes a detailed vignette demonstrating a complete
modeling workflow, including how to handle convergence failures using
the penalized estimator.

To explore the application of betaARMA modeling on empirical data, check
out the main vignette:

``` r
vignette("humidity_brasilia", package = "betaARMA")
```

### Application: Modeling Relative Humidity in Brasília

This vignette walks through a full analysis of monthly relative humidity
in Brasília, Brazil. It highlights a specific lag specification where
standard CMLE fails to converge. It demonstrates how applying the PCMLE
(ridge-penalized) estimator achieves convergence, allows for valid model
reduction, and ultimately produces a stable, well-specified model that
passes all residual diagnostic tests.

------------------------------------------------------------------------

## Getting Started

Here is a quick example of how to simulate data, fit a model, and
generate forecasts.

``` r
library(betaARMA)

# Example 1: Fit a BAR(1) model

# 1. Simulate data from a BAR(1) process
set.seed(2025)
y_sim_bar <- simu_barma(
  n = 250,
  alpha = 0.0,
  varphi = 0.6,
  phi = 25.0,
  link = "logit",
  freq = 12
)

# 2. Fit the model
fit_bar <- barma(y_sim_bar, ar = 1, link = "logit")

# 3. Inspect results
summary(fit_bar)
coef(fit_bar)

# 4. Diagnostic plots
plot(fit_bar)

# 5. Forecast the next 6 steps
pred_bar <- forecast(fit_bar, h = 6)
print(pred_bar)

# Example 2: Fit a BARMA(1,1) model using L-BFGS-B optimization

# 1. Simulate data from a BARMA(1,1) process
set.seed(2025)
y_sim_barma <- simu_barma(
    n = 250,
    alpha = 0.0,
    varphi = 0.6, 
    theta = 0.3, 
    phi = 25,
    link = "logit",
    freq = 12
)

# 2. Fit the model explicitly using L-BFGS-B
fit_barma <- barma(
    y_sim_barma, 
    ar = 1, 
    ma = 1, 
    link = "logit",
    optimization = list(method = "L-BFGS-B")
)

# 3. Inspect results
summary(fit_barma)
coef(fit_barma)

# 4. Forecast the next 6 steps
pred_barma <- forecast(fit_barma, h = 6)
print(pred_barma)

# Example 3: Ridge-penalized estimation (PCMLE)
# Useful when standard CMLE produces implausible estimates.

fit_penalized <- barma(y_sim_barma, ar = 1, ma = 1, link = "logit", penalty = TRUE)
summary(fit_penalized)
```

------------------------------------------------------------------------

## Citation

If you use this package in your research, please cite it as follows:

``` bibtex
@Manual{,
  title = {betaARMA: Beta Autoregressive Moving Average Models},
  author = {Everton da Costa, Francisco Cribari-Neto and Vinícius T. Scher},
  year = {2026},
  note = {R package version 1.2.0},
  doi = {10.32614/CRAN.package.betaARMA},
  url = {https://CRAN.R-project.org/package=betaARMA}
}
```

If you use the ridge penalization (`penalty = TRUE`), please also cite:

``` bibtex
@Article{Cribari_Costa_Fonseca2025,
  title   = {Numerical stability enhancements in beta autoregressive moving
             average model estimation},
  author  = {Francisco Cribari-Neto and Everton Costa and Rodney V. Fonseca},
  journal = {Brazilian Journal of Probability and Statistics},
  year    = {2025},
  volume  = {39},
  number  = {4},
  pages   = {410--437},
  doi     = {10.1214/25-BJPS645}
}
```

If your application involves hydro-environmental modeling, please
consider citing:

``` bibtex
@Article{Costa_Cribari_Scher2024,
  title     = {Test inferences and link function selection in dynamic beta 
               modeling of seasonal hydro-environmental time series with 
               temporary abnormal regimes},
  author    = {Costa, E. and Cribari-Neto, F. and Scher, V. T.},
  journal   = {Journal of Hydrology},
  year      = {2024},
  note      = {forthcoming}
}
```

------------------------------------------------------------------------

## Contributing

Contributions are welcome! If you find any issues or have suggestions
for improvements, please open an issue or submit a pull request.

## License

This project is licensed under the **MIT License**. See the `LICENSE`
file for details.

## Contact

For questions, suggestions, or issues related to the code, please
contact:

**Everton da Costa** 📧 <everto.cost@gmail.com>

------------------------------------------------------------------------

## Code of Conduct

Please note that the betaARMA project is released with a [Contributor
Code of
Conduct](https://contributor-covenant.org/version/2/1/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  fig.width = 7,
  fig.height = 3.5,
  fig.align = "center",
  warning   = FALSE,
  message   = FALSE
)

## ----libraries----------------------------------------------------------------
library(betaARMA)
library(forecast)
library(ggplot2)
library(gridExtra)
library(zoo)
library(scales)
library(moments)

## ----config_plot, echo=FALSE--------------------------------------------------
# Shared ggplot configuration
sample_size     <- length(brasilia_ts)
end_time_series <- time(brasilia_ts)[sample_size] + 1

ggplot_size_font <- 9

ggplot_theme <- theme_minimal() + 
  theme(
  legend.position = "bottom",
  title        = element_text(size = ggplot_size_font),
  axis.text    = element_text(size = ggplot_size_font),
  axis.title   = element_text(size = ggplot_size_font),
  legend.text  = element_text(size = ggplot_size_font),
  legend.title = element_text(size = ggplot_size_font)
)

ggplot_scale_y <- scale_y_continuous(
  breaks = seq(0.0, 1.00, 0.10)
)

ggplot_scale_x <- scale_x_continuous(
  breaks = seq(1999, end_time_series, 2),
  limits = c(1999, end_time_series)
)

## ----subsample----------------------------------------------------------------
y_subsample_ts <- window(
  brasilia_ts,
  start = c(2006, 02),
  end   = c(2019, 07)
)

y_train <- window(
  y_subsample_ts,
  end = c(2018, 07)
)

y_test <- window(
  y_subsample_ts,
  start = c(2018, 08)
)

## ----ggplot_subsample, echo=FALSE---------------------------------------------
len_y_ts        <- length(y_train)
start_subsample <- time(y_train)[1]
end_subsample   <- time(y_train)[len_y_ts]

ggplot_subsample <- ggplot(brasilia_df, aes(time, y)) +
  geom_point(size = 0.5) +
  geom_line(aes(group = 1)) +
  geom_vline(xintercept = start_subsample,
             linetype = "dashed", linewidth = 0.5, color = "red") +
  geom_vline(xintercept = end_subsample,
             linetype = "dashed", linewidth = 0.5, color = "red") +
  labs(x = "Year", y = "Relative humidity (proportion)") +
  ggplot_scale_x +
  ggplot_scale_y +
  ggplot_theme

## ----print_ggplot_subsample, echo=FALSE, fig.cap="Relative humidity in Brasília. Dashed red lines delimit the training period used in the instability analysis."----
print(ggplot_subsample)

## ----descriptive_subsample, echo=FALSE----------------------------------------
descriptive_sub_df <- data.frame(
  Min             = min(y_subsample_ts),
  Max             = max(y_subsample_ts),
  Median          = median(y_subsample_ts),
  Mean            = mean(y_subsample_ts),
  SD              = sd(y_subsample_ts),
  Skewness        = moments::skewness(y_subsample_ts),
  Excess_kurtosis = moments::kurtosis(y_subsample_ts)
)
descriptive_sub_df <- round(descriptive_sub_df, 2)

## ----print_descriptive_subsample, echo=FALSE----------------------------------
knitr::kable(
  descriptive_sub_df,
  caption = "Descriptive statistics of the monthly relative humidity in
  Brasília — subsample (February 2006 to July 2019)."
)

## ----regressors---------------------------------------------------------------
freq              <- frequency(y_train)
n_test            <- length(y_test)
trend_index_train <- seq_along(y_train)
trend_index_test  <- (max(trend_index_train) + 1):
                     (max(trend_index_train) + n_test)

hs_train <- sin(2 * pi * trend_index_train / freq)
hc_train <- cos(2 * pi * trend_index_train / freq)
hs_test  <- sin(2 * pi * trend_index_test  / freq)
hc_test  <- cos(2 * pi * trend_index_test  / freq)

X_train <- cbind(hs = hs_train, hc = hc_train)
X_test  <- cbind(hs = hs_test,  hc = hc_test)

## ----fit_cmle-----------------------------------------------------------------
ar_lags <- c(10, 18)
ma_lags <- c(1, 11, 13)

fit_cmle <- barma(
  y       = y_train,
  ar      = ar_lags,
  ma      = ma_lags,
  penalty = FALSE,
  xreg    = X_train
)

## ----summary_cmle-------------------------------------------------------------
summary(fit_cmle)

## ----fit_pcmle----------------------------------------------------------------
fit_pcmle <- barma(
  y       = y_train,
  ar      = ar_lags,
  ma      = ma_lags,
  penalty = TRUE,
  xreg    = X_train
)

## ----summary_pcmle------------------------------------------------------------
summary(fit_pcmle)

## ----diagnostics_pcmle, fig.height=7.0, fig.cap="Diagnostic plots for the PCMLE fit: AR=(10,18), MA=(1,11,13)."----
plot(fit_pcmle, which = "all")

## ----fit_cmle_reduced---------------------------------------------------------
ar_lags_reduced <- c(10, 18)
ma_lags_reduced <- c(1, 13)

fit_cmle_reduced <- barma(
  y       = y_train,
  ar      = ar_lags_reduced,
  ma      = ma_lags_reduced,
  penalty = FALSE,
  xreg    = X_train
)

## ----summary_cmle_reduced-----------------------------------------------------
summary(fit_cmle_reduced)

## ----diagnostics_all, fig.height=7.0, fig.cap="Diagnostic plots for the reduced CMLE model AR=(10,18), MA=(1,13)."----
plot(fit_cmle_reduced, which = "all")

## ----diagnostics_qq, fig.height=4.0, fig.width=4.5, fig.cap="Normal Q-Q plot of quantile residuals for the reduced CMLE model."----
plot(fit_cmle_reduced, which = "qq")

## ----forecast_setup-----------------------------------------------------------
# Define the translation dictionary
month_map <- c(
  "Jan" = "Jan", "Fev" = "Feb", "Mar" = "Mar", "Abr" = "Apr",
  "Mai" = "May", "Jun" = "Jun", "Jul" = "Jul", "Ago" = "Aug",
  "Set" = "Sep", "Out" = "Oct", "Nov" = "Nov", "Dez" = "Dec"
)

dates     <- as.Date(zoo::as.yearmon(time(y_test)))
pt_months <- tools::toTitleCase(format(dates, "%b"))
months    <- unname(month_map[pt_months])

forecasts_df <- data.frame(
  observed = as.numeric(y_test),
  forecast = forecast(fit_cmle_reduced, xreg = X_test, h = n_test)$mean,
  month    = months
)
forecasts_df$time <- seq_len(n_test)

## ----ggplot_forecast, echo=FALSE----------------------------------------------
ggplot_forecast <- ggplot(forecasts_df, aes(x = time)) +
  geom_line(aes(y = forecast, colour = "Forecast"), linewidth = 0.8) +
  geom_point(aes(y = observed, shape = "Observed"),
             color = "#00BFC4", size = 3) +
  scale_colour_manual(name   = " ",
                      values = c("Forecast" = "#C77CFF")) +
  scale_shape_manual(name   = " ",
                     values = c("Observed" = 16)) +
  scale_x_continuous(breaks = seq_len(n_test),
                     limits = c(1, n_test),
                     labels = forecasts_df$month) +
  scale_y_continuous(breaks = seq(0.3, 1.0, 0.1),
                     limits = c(0.3, 1.0),
                     labels = number_format(accuracy = 0.1)) +
  labs(x = "Time (months)", y = "Relative humidity (proportion)") +
  ggplot_theme

## ----print_forecast, echo=FALSE, fig.cap="Out-of-sample forecasts from the reduced CMLE model against the observed test data (teal dots)."----
print(ggplot_forecast)

## ----forecast_accuracy--------------------------------------------------------
mae  <- function(obs, pred) mean(abs(obs - pred))
rmse <- function(obs, pred) sqrt(mean((obs - pred)^2))

obs <- forecasts_df$observed

accuracy_table <- data.frame(
  Model = "betaARMA CMLE (reduced)",
  MAE   = round(mae(obs,  forecasts_df$forecast) * 100, 2),
  RMSE  = round(rmse(obs, forecasts_df$forecast) * 100, 2)
)

knitr::kable(
  accuracy_table,
  caption = paste0(
    "MAE and RMSE ($\\times 100$) over the ",
    n_test,
    "-month forecast horizon."
  )
)

## ----session_info, echo=FALSE-------------------------------------------------
cat("=================================================================\n")
cat("Session Information\n")
cat("=================================================================\n")
cat("This report was generated at:",
    format(Sys.time(), "%B %d, %Y at %I:%M %p"), "\n\n")
print(sessionInfo())

# Internal utility functions — not exported
# These helpers are shared across loglik_barma(), score_vector_barma(),
# and fim_barma() to avoid duplication of critical constants.

# .barma_ridge_lambda: data-driven L2 penalty scalar per Cribari-Neto,
# Costa and Fonseca (2025, BJPS, Section 2): lambda_n = 1/(n-a)^ridge_rate
.barma_ridge_lambda <- function(n_obs, max_lag, ridge_rate = 0.9) {
  1 / (n_obs - max_lag)^ridge_rate
}

#' Generate Initial Values for BARMA Model Estimation
#'
#' @description
#' This function calculates reasonable starting values for the parameters of
#' various Beta Autoregressive Moving Average (BARMA) models. The method is
#' based on the approach proposed by Ferrari & Cribari-Neto (2004) for
#' beta regression, adapted here for the time series context.
#'
#' @details
#' The function computes initial values by fitting a linear model
#' (`lm.fit`) to the link-transformed response variable `g(y)`.
#' This provides a computationally cheap and stable way to initialize the
#' main optimization algorithm.
#'
#' @author
#' Original code by Fabio M. Bayer (bayer@ufsm.br).
#' Substantially modified and improved by Everton da Costa
#' (everto.cost@gmail.com).
#'
#' @param y A numeric time series with values in the open interval (0, 1).
#' @param link A string specifying the link function for the mean.
#' @param ar A numeric vector of autoregressive (AR) lags. Defaults to
#'   \code{integer(0)}, which omits the AR component entirely.
#' @param ma A numeric vector of moving average (MA) lags. Defaults to
#'   \code{integer(0)}, which omits the MA component entirely.
#' @param xreg An optional numeric matrix or data frame of exogenous variables.
#'
#' @importFrom stats lm.fit fitted residuals var
#'
#' @return
#' A named numeric vector containing the initial values for the model
#' parameters (`alpha`, `varphi`, `theta`, `beta`, `phi`), ready to be
#' used by an optimization routine.
#'
#' @export
start_values <- function(y, link, ar = integer(0), ma = integer(0), xreg = NA) {
  
  # Link functions
  # ------------------------------------------------------------------------- #
  link_structure <- make_link_structure(link)
  linkfun <- link_structure$linkfun
  linkinv <- link_structure$linkinv
  mu.eta  <- link_structure$mu.eta
  
  y_transformed <- linkfun(y)
  n_obs <- length(y)
  
  # Resolve lag vectors to integer(0) if absent
  ar_lags <- if (length(ar) > 0) ar else integer(0)
  ma_lags <- if (length(ma) > 0) ma else integer(0)
  
  # Named flags for readability throughout the function
  has_ar   <- length(ar_lags) > 0
  has_ma   <- length(ma_lags) > 0
  has_xreg <- !is.null(xreg) && !all(is.na(xreg))
  
  ar_order <- if (has_ar) max(ar_lags, 0L) else 0L
  ma_order <- if (has_ma) max(ma_lags, 0L) else 0L
  
  n_ar_params <- length(ar_lags)
  n_ma_params <- length(ma_lags)
  
  max_lag <- max(ar_order, ma_order)
  
  if (has_xreg) {
    xreg <- as.matrix(xreg)
  }
  
  names_varphi <- if (has_ar) paste0("varphi", ar_lags) else character(0)
  names_theta  <- if (has_ma) paste0("theta", ma_lags) else character(0)
  names_beta   <- if (has_xreg) colnames(xreg) else character(0)
  
  n_eff <- n_obs - max_lag
  
  # ======================================================================== #
  # Initialize empty containers (Single Point of Truth prep)
  # ======================================================================== #
  alpha_start  <- numeric(0)
  varphi_start <- numeric(0)
  theta_start  <- numeric(0)
  beta_start   <- numeric(0)
  phi_start    <- numeric(0)
  
  # ======================================================================== #
  # 1. BARMA initial values (has_ar, has_ma, !has_xreg)
  # ======================================================================== #
  if (has_ar && has_ma && !has_xreg) {
    P <- matrix(NA, nrow = n_eff, ncol = n_ar_params)
    for (i in 1:n_eff) {
      P[i, ] <- y_transformed[i + max_lag - ar_lags]
    }
    x_inter <- matrix(1, nrow = n_eff, ncol = 1)
    
    fit_start <- lm.fit(
      x = cbind(x_inter, P),
      y = y_transformed[(max_lag + 1):n_obs]
    )
    mqo <- fit_start$coef
    
    alpha_start  <- mqo[1]
    varphi_start <- mqo[2:(n_ar_params + 1)]
    theta_start  <- rep(0, n_ma_params)
    phi_start    <- .get_phi_start(fit_start, n_eff, linkinv, mu.eta)
    
  }
  
  # ======================================================================== #
  # 2. BAR initial values (has_ar, !has_ma, !has_xreg)
  # ======================================================================== #
  if (has_ar && !has_ma && !has_xreg) {
    P <- matrix(NA, nrow = n_eff, ncol = n_ar_params)
    for (i in 1:n_eff) {
      P[i, ] <- y_transformed[i + max_lag - ar_lags]
    }
    x_inter <- matrix(1, nrow = n_eff, ncol = 1)
    
    fit_start <- lm.fit(
      x = cbind(x_inter, P),
      y = y_transformed[(max_lag + 1):n_obs]
    )
    mqo <- fit_start$coef
    
    alpha_start  <- mqo[1]
    varphi_start <- mqo[2:(n_ar_params + 1)]
    phi_start    <- .get_phi_start(fit_start, n_eff, linkinv, mu.eta)
    
  }
  # ======================================================================== #
  # 3. BMA initial values (!has_ar, has_ma, !has_xreg)
  # ======================================================================== #
  if (!has_ar && has_ma && !has_xreg) {
    mean_y <- mean(y)
    alpha_start <- mean(y_transformed)
    theta_start <- rep(0, n_ma_params)
    phi_start   <- (mean_y * (1 - mean_y)) / var(y)
    
  }
  
  # ======================================================================== #
  # 4. BARMAX initial values (has_ar, has_ma, has_xreg)
  # ======================================================================== #
  if (has_ar && has_ma && has_xreg) {
    P <- matrix(NA, nrow = n_eff, ncol = n_ar_params)
    for (i in 1:n_eff) {
      P[i, ] <- y_transformed[i + max_lag - ar_lags]
    }
    x_inter <- matrix(1, nrow = n_eff, ncol = 1)
    x_reg_mat <- xreg[(max_lag + 1):n_obs, , drop = FALSE]
    
    fit_start <- lm.fit(
      x = cbind(x_inter, P, x_reg_mat),
      y = y_transformed[(max_lag + 1):n_obs]
    )
    mqo <- fit_start$coef
    
    alpha_start  <- mqo[1]
    varphi_start <- mqo[2:(n_ar_params + 1)]
    theta_start  <- rep(0, n_ma_params)
    beta_start   <- mqo[(n_ar_params + 2):length(mqo)]
    phi_start    <- .get_phi_start(fit_start, n_eff, linkinv, mu.eta)
    
  }
  
  # ======================================================================== #
  # 5. BARX initial values (has_ar, !has_ma, has_xreg)
  # ======================================================================== #
  if (has_ar && !has_ma && has_xreg) {
    P <- matrix(NA, nrow = n_eff, ncol = n_ar_params)
    for (i in 1:n_eff) {
      P[i, ] <- y_transformed[i + max_lag - ar_lags]
    }
    x_inter <- matrix(1, nrow = n_eff, ncol = 1)
    x_reg_mat <- xreg[(max_lag + 1):n_obs, , drop = FALSE]
    
    fit_start <- lm.fit(
      x = cbind(x_inter, P, x_reg_mat),
      y = y_transformed[(max_lag + 1):n_obs]
    )
    mqo <- fit_start$coef
    
    alpha_start  <- mqo[1]
    varphi_start <- mqo[2:(n_ar_params + 1)]
    beta_start   <- mqo[(n_ar_params + 2):length(mqo)]
    phi_start    <- .get_phi_start(fit_start, n_eff, linkinv, mu.eta)
    
  }
  
  # ======================================================================== #
  # 6. BMAX initial values (!has_ar, has_ma, has_xreg)
  # ======================================================================== #
  if (!has_ar && has_ma && has_xreg) {
    x_inter <- matrix(1, nrow = n_eff, ncol = 1)
    x_reg_mat <- xreg[(max_lag + 1):n_obs, , drop = FALSE]
    
    fit_start <- lm.fit(
      x = cbind(x_inter, x_reg_mat),
      y = y_transformed[(max_lag + 1):n_obs]
    )
    mqo <- fit_start$coef
    
    alpha_start <- mqo[1]
    theta_start <- rep(0, n_ma_params)
    beta_start  <- mqo[2:length(mqo)]
    phi_start   <- .get_phi_start(fit_start, n_eff, linkinv, mu.eta)
    
  }
  
  # ======================================================================== #
  # 7. Beta Regression (!has_ar, !has_ma, has_xreg)
  # ======================================================================== #
  if (!has_ar && !has_ma && has_xreg) {
    x_inter <- matrix(1, nrow = n_obs, ncol = 1)
    
    fit_start <- lm.fit(
      x = cbind(x_inter, xreg),
      y = y_transformed
    )
    mqo <- fit_start$coef
    
    alpha_start <- mqo[1]
    beta_start  <- mqo[2:length(mqo)]
    phi_start   <- .get_phi_start(fit_start, n_obs, linkinv, mu.eta)
    
  }
  
  # ======================================================================== #
  # 8. Intercept-only Model (!has_ar, !has_ma, !has_xreg)
  # ======================================================================== #
  if (!has_ar && !has_ma && !has_xreg) {
    mean_y <- mean(y)
    alpha_start <- mean(y_transformed)
    phi_start <- (mean_y * (1 - mean_y)) / var(y)
    
  }
  
  # ======================================================================== #
  # FINAL ASSEMBLY
  # ======================================================================== #
  start_value <- c(
    alpha_start,
    varphi_start,
    theta_start,
    beta_start,
    phi_start
  )
  
  names(start_value) <- c(
    "alpha",
    if (length(varphi_start) > 0) names_varphi else character(0),
    if (length(theta_start) > 0)  names_theta  else character(0),
    if (length(beta_start) > 0)   names_beta   else character(0),
    "phi"
  )
  
  return(start_value)
}

#' Internal Helper to Calculate Initial Phi
#'
#' @description
#' Calculates the initial value for the precision parameter phi based on the
#' residuals of an initial 'lm.fit' object. The formula is adapted and
#' empirically improved from Ferrari & Cribari-Neto (2004) — specifically,
#' the "-1" term in the original precision estimator is omitted, as Monte
#' Carlo simulations showed better optimization performance without it.
#'
#' @param fit An 'lm.fit' object.
#' @param n_eff The effective number of observations used in the fit.
#' @param linkinv The inverse link function.
#' @param mu.eta The derivative of the mean w.r.t. eta (d(mu)/d(eta)).
#'
#' @return A single numeric value for phi_start.
#' @keywords internal
.get_phi_start <- function(fit, n_eff, linkinv, mu.eta) {
  
  mqo <- fit$coef
  k <- length(mqo)
  n1 <- n_eff
  
  y_hat_fit_start <- fitted(fit)
  mean_fit_start <- linkinv(y_hat_fit_start)
  
  linkfun_deriv_aux <- mu.eta(eta = y_hat_fit_start)
  linkfun_deriv <- 1 / linkfun_deriv_aux
  
  er <- residuals(fit)
  
  # ---
  sigma2 <- sum(er^2) / ((n1 - k) * linkfun_deriv^2)
  phi_start_aux <- sum(mean_fit_start * (1 - mean_fit_start) / sigma2)
  
  phi_start <- phi_start_aux / n1
  # ---
  
  return(phi_start)
}

#' @title Create Link Function Structure for BARMA Models
#'
#' @description
#' A helper function that constructs a list containing the link function,
#' its inverse, and the derivative of the mean function. It extends the
#' standard \code{make.link} by adding support for the \code{"loglog"} link.
#'
#' @details
#' This function is used by \code{\link{barma}}, \code{\link{loglik_barma}},
#' \code{\link{score_vector_barma}}, and \code{\link{fim_barma}} to handle
#' the \code{link} argument in a standardized way. It is also exported for
#' users who implement custom workflows.
#'
#' For the \code{"logit"}, \code{"probit"}, and \code{"cloglog"} links,
#' the function acts as a wrapper around the base R \code{make.link}.
#'
#' For the \code{"loglog"} link, which is not available in \code{make.link},
#' the necessary components are defined explicitly:
#' \itemize{
#'   \item Link function: \eqn{g(\mu) = -\log(-\log(\mu))}
#'   \item Inverse link: \eqn{g^{-1}(\eta) = \exp(-\exp(-\eta))}
#'   \item Derivative \eqn{\frac{d\mu}{d\eta}}:
#'     \eqn{\exp(-\exp(-\eta) - \eta)}
#' }
#'
#' If an unsupported link is provided, the function stops with an error
#' message listing the available options.
#'
#' @param link A character string specifying the link function.
#'   Defaults to \code{"logit"}. Accepted values are \code{"logit"},
#'   \code{"probit"}, \code{"cloglog"}, and \code{"loglog"}.
#'
#' @importFrom stats make.link
#'
#' @return A list with three components:
#' \item{linkfun}{The link function \eqn{g(\mu)}, transforming
#'   \eqn{\mu \in (0,1)} to \eqn{\eta \in (-\infty, \infty)}.}
#' \item{linkinv}{The inverse link function \eqn{g^{-1}(\eta)},
#'   transforming \eqn{\eta} back to \eqn{\mu}.}
#' \item{mu.eta}{The derivative \eqn{d\mu/d\eta} of the inverse
#'   link function.}
#'
#' @author
#' Original R code by Fabio M. Bayer
#' (Federal University of Santa Maria, \email{bayer@ufsm.br}).
#' Modified and improved by Everton da Costa
#' (Federal University of Pernambuco, \email{everto.cost@gmail.com}).
#'
#' @seealso \code{\link{barma}}, \code{\link[stats]{make.link}}
#'
#' @examples
#' # --- Create a logit link structure ---
#' logit_link <- make_link_structure(link = "logit")
#'
#' # Apply the link function
#' mu <- 0.5
#' eta <- logit_link$linkfun(mu)
#' print(eta) # Should be 0
#'
#' # Apply the inverse link function
#' mu_restored <- logit_link$linkinv(eta)
#' print(mu_restored) # Should be 0.5
#'
#' # --- Create a loglog link structure ---
#' loglog_link <- make_link_structure(link = "loglog")
#'
#' # Apply the loglog link function
#' mu_loglog <- 0.8
#' eta_loglog <- loglog_link$linkfun(mu_loglog)
#' print(eta_loglog)
#'
#' # Apply the inverse loglog link function
#' mu_restored_loglog <- loglog_link$linkinv(eta_loglog)
#' print(mu_restored_loglog) # Should be ~0.8
#'
#' @export
make_link_structure <- function(link = "logit") {
  
  # ======================================================================== #
  # 1. Validate Link Argument
  # ======================================================================== #
  
  if (!is.character(link) || length(link) != 1) {
    stop("Argument 'link' must be a single character string.")
  }
  
  # ======================================================================== #
  # 2. Select Link Structure
  # ======================================================================== #
  
  if (link == "logit") {
    stats <- make.link("logit")
  } else if (link == "probit") {
    stats <- make.link("probit")
  } else if (link == "cloglog") {
    stats <- make.link("cloglog")
  } else if (link == "loglog") {
    stats <- list(
      linkfun = function(mu)  -log(-log(mu)),
      linkinv = function(eta) exp(-exp(-eta)),
      mu.eta  = function(eta) exp(-exp(-eta) - eta)
    )
  } else {
    stop(
      "'", link, "' link not available. ",
      "Available links are: 'logit', 'probit', 'cloglog', 'loglog'."
    )
  }
  
  # ======================================================================== #
  # 3. Return Structure
  # ======================================================================== #
  
  return(list(
    linkfun = stats$linkfun,
    linkinv = stats$linkinv,
    mu.eta  = stats$mu.eta
  ))
  
}

#' Fisher Information Matrix for BARMA Models
#'
#' @title
#' Compute Fisher Information Matrix for Beta Autoregressive
#' Moving Average Models
#'
#' @description
#' Computes the expected Fisher Information Matrix (FIM) of a Beta
#' Autoregressive Moving Average (BARMA) model. This function also
#' efficiently returns auxiliary values like fitted values and residuals.
#' 
#' This function is designed for users who:
#' \itemize{
#'   \item Compute standard errors of parameter estimates
#'   \item Construct confidence intervals
#'   \item Perform hypothesis tests
#'   \item Verify theoretical properties
#'   \item Conduct simulation studies
#' }
#'
#' @details
#' The expected Fisher Information Matrix is computed analytically using
#' the trigamma-based expressions from Rocha and Cribari-Neto (2009).
#' Standard errors are obtained via \code{sqrt(diag(solve(FIM)))}.
#'
#' **Non-Diagonal Structure**: Unlike generalized linear models, the FIM
#' is not block-diagonal due to the coupling introduced by the ARMA
#' dynamics. This is documented in the Rocha & Cribari-Neto (2009) paper.
#'
#' **Important**: This function implements the corrections from the 2017
#' Erratum (Rocha & Cribari-Neto, 2017) for moving average components.
#' See References section for details.
#'
#' **Parameter Order**: Parameters should be supplied in the order:
#' alpha, varphi (AR), theta (MA), beta (regressors), phi.
#' This matches the parameter order used by \code{\link{barma}}.
#'
#' @param y
#' A numeric vector representing the time series data, in (0, 1).
#'
#' @param ar
#' A numeric vector specifying the autoregressive (AR) lags
#' (e.g., \code{c(1, 2)}). Defaults to \code{integer(0)}, which omits the
#' AR component entirely. Absence should be expressed by omitting this
#' argument or passing \code{integer(0)}.
#'
#' @param ma
#' A numeric vector specifying the moving average (MA) lags
#' (e.g., \code{1}). Defaults to \code{integer(0)}, which omits the
#' MA component entirely. Absence should be expressed by omitting this
#' argument or passing \code{integer(0)}.
#'
#' @param alpha
#' The intercept term (numeric scalar).
#'
#' @param varphi
#' A numeric vector of autoregressive (AR) parameters.
#' Absence should be expressed by omitting this argument or passing
#' \code{numeric(0)}.
#'
#' @param theta
#' A numeric vector of moving average (MA) parameters.
#' Absence should be expressed by omitting this argument or passing
#' \code{numeric(0)}.
#'
#' @param phi
#' The precision parameter of the BARMA model (must be positive and finite).
#' Larger values indicate less variance for a given mean.
#'
#' @param link
#' A character string specifying the link function:
#' \code{"logit"} (default), \code{"probit"}, or \code{"cloglog"}.
#'
#' @param xreg
#' A matrix or data frame of static regressors (optional).
#' Must have the same number of rows as length of \code{y}.
#'
#' @param beta
#' A numeric vector of regression coefficients for \code{xreg} (optional).
#' Length must match number of columns in \code{xreg}.
#' 
#' @param penalty
#' Logical. If \code{TRUE}, the ridge penalty of Cribari-Neto, Costa and
#' Fonseca (2025) is added to the Fisher Information Matrix, yielding the
#' penalized FIM \eqn{K_{\text{pen}}(\boldsymbol{\nu}) = K(\boldsymbol{\nu})
#' + 2(n-a)\Lambda}, where \eqn{\Lambda =
#' \text{diag}\{\lambda_n,\ldots,\lambda_n, 0,\ldots,0\}} with
#' \eqn{\lambda_n = 1/(n-a)^{0.9}} applied to \eqn{\alpha},
#' \eqn{\boldsymbol{\varphi}}, and \eqn{\boldsymbol{\theta}} only.
#' Defaults to \code{FALSE}.
#'
#' @return
#' A list containing:
#' \item{fisher_info_mat}{The Fisher Information Matrix (numeric matrix).
#'   Dimensions: \code{(k+p+q+2) x (k+p+q+2)} where k is number of regressors,
#'   p is AR order, q is MA order. The matrix is symmetric and should be
#'   positive definite at the MLE.}
#' \item{fitted_ts}{The fitted values (conditional mean) as a \code{ts} object,
#'   with the same time index as the input \code{y}. Values for the first
#'   \eqn{m = \max(p,q)} observations are \code{NA}.}
#' \item{muhat_effective}{The fitted conditional means (numeric vector)
#'   excluding the first \eqn{m} observations.}
#' \item{etahat_full}{The estimated linear predictor values (numeric vector,
#'   length = length(y)), with \code{NA} for the first \eqn{m} observations.}
#' \item{errorhat_full}{The estimated errors on the predictor scale
#'   (numeric vector, length = length(y)), zero-padded for the first
#'   \eqn{m} observations.}
#'
#' @references
#' Rocha, A.V., & Cribari-Neto, F. (2009). Beta autoregressive moving
#' average models. \emph{TEST}, 18(3), 529-545.
#' \doi{10.1007/s11749-008-0112-z}
#'
#' Rocha, A.V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
#' moving average models. \emph{TEST}, 26, 451-459.
#' \doi{10.1007/s11749-017-0528-4}
#' 
#' Cribari-Neto, F., Costa, E., & Fonseca, R.V. (2025). Numerical stability
#' enhancements in beta autoregressive moving average model estimation.
#' \emph{Brazilian Journal of Probability and Statistics}, 39(4), 410-437.
#' \doi{10.1214/25-BJPS645}
#' 
#' @seealso
#' \code{\link{barma}} for model fitting,
#' \code{\link{loglik_barma}} for log-likelihood computation,
#' \code{\link{score_vector_barma}} for score vector (gradient)
#'
#' @examples
#' \donttest{
#'   # Example 1: Fisher Information Matrix for a BAR(1) model (no MA component)
#'   set.seed(2025)
#'   y_sim_bar1 <- simu_barma(
#'     n      = 250,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     phi    = 25.0,
#'     link   = "logit",
#'     freq   = 12
#'   )
#'
#'   result_bar1 <- fim_barma(
#'     y      = y_sim_bar1,
#'     ar     = 1,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     theta  = numeric(0),
#'     phi    = 25.0,
#'     link   = "logit"
#'   )
#'
#'   # Check positive definiteness
#'   fim <- result_bar1$fisher_info_mat
#'   all(eigen(fim)$values > 0)
#'
#'   # Standard errors from inverse of FIM
#'   sqrt(diag(solve(fim)))
#'
#'   # Example 2: Fisher Information Matrix for a BARMA(1, 1) model
#'   set.seed(2025)
#'   y_sim_barma11 <- simu_barma(
#'     n      = 250,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     theta  = 0.3,
#'     phi    = 25.0,
#'     link   = "logit",
#'     freq   = 12
#'   )
#'
#'   result_barma11 <- fim_barma(
#'     y      = y_sim_barma11,
#'     ar     = 1,
#'     ma     = 1,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     theta  = 0.3,
#'     phi    = 25.0,
#'     link   = "logit"
#'   )
#'
#'   sqrt(diag(solve(result_barma11$fisher_info_mat)))
#'
#'   # Example 3: Fisher Information Matrix for a BMA(1) model (no AR component)
#'   set.seed(2025)
#'   y_sim_bma1 <- simu_barma(
#'     n      = 250,
#'     alpha  = 0.0,
#'     theta  = 0.3,
#'     phi    = 20.0,
#'     link   = "logit",
#'     freq   = 12
#'   )
#'
#'   result_bma1 <- fim_barma(
#'     y      = y_sim_bma1,
#'     ma     = 1,
#'     alpha  = 0.0,
#'     varphi = numeric(0),
#'     theta  = 0.3,
#'     phi    = 20.0,
#'     link   = "logit"
#'   )
#'
#'   sqrt(diag(solve(result_bma1$fisher_info_mat)))
#' }
#'
#' @export
fim_barma <- function(
    y,
    ar   = integer(0),
    ma   = integer(0),
    alpha,
    varphi = numeric(0),
    theta = numeric(0),
    phi,
    link = "logit",
    xreg = NULL,
    beta = NULL, 
    penalty = FALSE
) {
  
  # --------------------------------------------------------------------------
  # 1. SETUP LINK FUNCTIONS AND MODEL STRUCTURE
  # --------------------------------------------------------------------------
  
  # Get link function structures
  link_structure <- make_link_structure(link)
  linkfun <- link_structure$linkfun
  linkinv <- link_structure$linkinv
  mu_eta_fun <- link_structure$mu.eta
  
  # Transform response using link function
  y_transformed <- linkfun(y)
  
  # Resolve lag vectors to integer(0) if absent
  ar_lags <- ar
  ma_lags <- ma
  
  # Force varphi / theta to numeric(0) when the corresponding component
  # is absent
  varphi <- if (length(ar_lags) > 0) varphi else numeric(0)
  theta  <- if (length(ma_lags) > 0) theta  else numeric(0)
  
  has_xreg <- !is.null(xreg)
  
  n_ar_params <- length(varphi)
  n_ma_params <- length(theta)
  
  # Named flags for readability throughout the function
  has_ar <- n_ar_params > 0
  has_ma <- n_ma_params > 0
  
  # Setup Regressors
  if (has_xreg) {
    if (!is.matrix(xreg)) xreg <- as.matrix(xreg)
    n_beta_params <- ncol(xreg)
    xb <- as.vector(xreg %*% beta)
  } else {
    n_beta_params <- 0
    xb <- numeric(length(y))
  }
  
  # Determine maximum lag required to initialize the recursive structure
  ar_order <- if (has_ar) max(ar_lags) else 0L
  ma_order <- if (has_ma) max(ma_lags) else 0L
  max_lag  <- max(ar_order, ma_order)
  
  n_obs <- length(y)
  
  # --------------------------------------------------------------------------
  # 2. RECURSIVE CALCULATION OF PREDICTOR, ERROR, AND DERIVATIVES
  # --------------------------------------------------------------------------
  
  # Initialize containers for linear predictor and errors
  error   <- rep(0, n_obs)
  eta     <- rep(NA_real_, n_obs)
  
  # Initialize derivative matrices for chain rule calculations
  d_eta_d_alpha  <- rep(0, n_obs)
  d_eta_d_varphi <- matrix(0, nrow = n_obs, ncol = n_ar_params)
  d_eta_d_theta  <- matrix(0, nrow = n_obs, ncol = n_ma_params)
  d_eta_d_beta   <- matrix(0, nrow = n_obs, ncol = n_beta_params)
  
  # Recursively compute predictor, errors, and their derivatives
  for (t in (max_lag + 1):n_obs) {
    
    # Part A: Compute Linear Predictor and Error
    eta[t] <- alpha + xb[t] +
      drop(crossprod(varphi, y_transformed[t - ar_lags] - xb[t - ar_lags])) +
      drop(crossprod(theta,  error[t - ma_lags]))
    
    error[t] <- y_transformed[t] - eta[t]
    
    # Part B: Compute Recursive Derivatives
    
    # 1. Derivative w.r.t. alpha
    d_eta_d_alpha[t] <- 1 -
      drop(crossprod(theta, d_eta_d_alpha[t - ma_lags]))
    
    # 2. Derivative w.r.t. AR (varphi)
    if (has_ar) {
      d_eta_d_varphi[t, ] <- y_transformed[t - ar_lags] - xb[t - ar_lags] -
        drop(crossprod(theta, d_eta_d_varphi[t - ma_lags, , drop = FALSE]))
    }
    
    # 3. Derivative w.r.t. MA (theta)
    if (has_ma) {
      d_eta_d_theta[t, ] <- error[t - ma_lags] -
        drop(crossprod(theta, d_eta_d_theta[t - ma_lags, , drop = FALSE]))
    }
    
    # 4. Derivative w.r.t. beta (Regressors)
    if (has_xreg) {
      d_eta_d_beta[t, ] <- xreg[t, ] -
        drop(crossprod(varphi, xreg[t - ar_lags, , drop = FALSE])) -
        drop(crossprod(theta,  d_eta_d_beta[t - ma_lags, , drop = FALSE]))
    }
  }
  
  # --------------------------------------------------------------------------
  # 3. GET EFFECTIVE (NON-NA) OBSERVATIONS
  # --------------------------------------------------------------------------
  
  # Extract observations used in likelihood (excluding first m observations)
  idx_effective <- (max_lag + 1):n_obs
  eta_eff <- eta[idx_effective]
  mu_eff  <- linkinv(eta = eta_eff)
  
  # Numerical stability check
  if (any(mu_eff <= 0 | mu_eff >= 1 | !is.finite(mu_eff))) {
    warning("mu_effective out of bounds during FIM calculation.")
    n_params <- 1 + n_ar_params + n_ma_params + n_beta_params + 1
    return(list(
      fisher_info_mat = matrix(NA_real_, nrow = n_params, ncol = n_params),
      fitted_ts = ts(rep(NA_real_, n_obs), start = start(y), 
                     frequency = frequency(y)),
      muhat_effective = rep(NA_real_, length(idx_effective)),
      etahat_full = eta,
      errorhat_full = error
    ))
  }
  
  # Extract effective derivative vectors/matrices
  s_vec <- d_eta_d_alpha[idx_effective]
  P_mat <- d_eta_d_varphi[idx_effective, , drop = FALSE]
  R_mat <- d_eta_d_theta[idx_effective, , drop = FALSE]
  M_mat <- d_eta_d_beta[idx_effective, , drop = FALSE]
  
  # --------------------------------------------------------------------------
  # 4. CALCULATE FIM COMPONENT VECTORS (Efficiently)
  # --------------------------------------------------------------------------
  
  # Trigamma functions: derivatives of digamma
  trigamma_p <- trigamma(mu_eff * phi)
  trigamma_q <- trigamma((1 - mu_eff) * phi)
  
  # d(mu)/d(eta) - derivative of inverse link function
  mu_eta_val <- mu_eta_fun(eta = eta_eff)
  
  # Vector W
  # W = phi * {psi'(mu*phi) + psi'((1-mu)*phi)} * (mu_eta)^2
  w_t_vec <- phi * (trigamma_p + trigamma_q) * mu_eta_val^2
  
  # Vector c
  # c = phi * {psi'(mu*phi)*mu - psi'((1-mu)*phi)*(1-mu)}
  c_t_vec <- phi * (trigamma_p * mu_eff - trigamma_q * (1 - mu_eff))
  
  # Vector d
  # d = psi'(mu*phi)*mu^2 + psi'((1-mu)*phi)*(1-mu)^2 - psi'(phi)
  d_t_vec <- trigamma_p * mu_eff^2 +
    trigamma_q * (1 - mu_eff)^2 -
    trigamma(phi)
  
  # --------------------------------------------------------------------------
  # 5. CALCULATE FIM BLOCKS
  # --------------------------------------------------------------------------
  
  # Helper to compute the cross-product
  calc_block <- function(X, Y, w, scale = 1) {
    if (length(X) == 0 || length(Y) == 0) {
      return(matrix(numeric(0), 0, 0))
    }
    
    X <- as.matrix(X)
    Y <- as.matrix(Y)
    
    if (length(w) != nrow(Y)) {
      stop("The vector length must match number of rows in Y.")
    }
    
    result <- crossprod(X, Y * w) * scale
    return(result)
  }
  
  # Blocks scaling with phi
  K_aa <- calc_block(s_vec, s_vec, w_t_vec, scale = phi)
  
  K_ap <- if (has_ar) {
    calc_block(s_vec, P_mat, w_t_vec, scale = phi)
  } else {
    numeric(0)
  }
  
  K_at <- if (has_ma) {
    calc_block(s_vec, R_mat, w_t_vec, scale = phi)
  } else {
    numeric(0)
  }
  
  K_ab <- if (has_xreg) {
    calc_block(s_vec, M_mat, w_t_vec, scale = phi)
  } else {
    numeric(0)
  }
  
  K_pp <- if (has_ar) {
    calc_block(P_mat, P_mat, w_t_vec, scale = phi)
  } else {
    matrix(0, 0, 0)
  }
  
  K_pt <- if (has_ar && has_ma) {
    calc_block(P_mat, R_mat, w_t_vec, scale = phi)
  } else {
    matrix(0, 0, 0)
  }
  
  K_pb <- if (has_ar && has_xreg) {
    calc_block(P_mat, M_mat, w_t_vec, scale = phi)
  } else {
    matrix(0, 0, 0)
  }
  
  K_tt <- if (has_ma) {
    calc_block(R_mat, R_mat, w_t_vec, scale = phi)
  } else {
    matrix(0, 0, 0)
  }
  
  K_tb <- if (has_ma && has_xreg) {
    calc_block(R_mat, M_mat, w_t_vec, scale = phi)
  } else {
    matrix(0, 0, 0)
  }
  
  K_bb <- if (has_xreg) {
    calc_block(M_mat, M_mat, w_t_vec, scale = phi)
  } else {
    matrix(0, 0, 0)
  }
  
  # Blocks involving Phi
  K_aphi <- as.numeric(crossprod(s_vec, mu_eta_val * c_t_vec))
  
  K_pphi <- if (has_ar) {
    crossprod(P_mat, mu_eta_val * c_t_vec)
  } else {
    numeric(0)
  }
  
  K_tphi <- if (has_ma) {
    crossprod(R_mat, mu_eta_val * c_t_vec)
  } else {
    numeric(0)
  }
  
  K_bphi <- if (has_xreg) {
    crossprod(M_mat, mu_eta_val * c_t_vec)
  } else {
    numeric(0)
  }
  
  # Phi vs Phi Block
  K_phiphi <- sum(d_t_vec)
  
  # --------------------------------------------------------------------------
  # 6. ASSEMBLE AND RETURN THE FINAL FIM
  # --------------------------------------------------------------------------
  
  # Parameter order: alpha, varphi, theta, beta, Phi
  n_params <- 1 + n_ar_params + n_ma_params + n_beta_params + 1
  fim <- matrix(NA_real_, nrow = n_params, ncol = n_params)
  
  # Define parameter indices
  idx_a <- 1
  
  idx_p <- if (has_ar) 2:(1 + n_ar_params) else integer(0)
  last_idx <- if (has_ar) max(idx_p) else idx_a
  
  idx_t <- if (has_ma) {
    (last_idx + 1):(last_idx + n_ma_params)
  } else {
    integer(0)
  }
  if (has_ma) last_idx <- max(idx_t)
  
  idx_b <- if (has_xreg) {
    (last_idx + 1):(last_idx + n_beta_params)
  } else {
    integer(0)
  }
  if (has_xreg) last_idx <- max(idx_b)
  
  idx_phi <- n_params
  
  # Fill Matrix (Symmetric)
  fim[idx_a, idx_a]   <- K_aa
  
  if (has_ar) {
    fim[idx_a, idx_p] <- K_ap
    fim[idx_p, idx_a] <- t(K_ap)
  }
  if (has_ma) {
    fim[idx_a, idx_t] <- K_at
    fim[idx_t, idx_a] <- t(K_at)
  }
  if (has_xreg) {
    fim[idx_a, idx_b] <- K_ab
    fim[idx_b, idx_a] <- t(K_ab)
  }
  fim[idx_a, idx_phi] <- K_aphi
  fim[idx_phi, idx_a] <- t(K_aphi)
  
  if (has_ar) {
    fim[idx_p, idx_p] <- K_pp
    if (has_ma) {
      fim[idx_p, idx_t] <- K_pt
      fim[idx_t, idx_p] <- t(K_pt)
    }
    if (has_xreg) {
      fim[idx_p, idx_b] <- K_pb
      fim[idx_b, idx_p] <- t(K_pb)
    }
    fim[idx_p, idx_phi] <- K_pphi
    fim[idx_phi, idx_p] <- t(K_pphi)
  }
  
  if (has_ma) {
    fim[idx_t, idx_t] <- K_tt
    if (has_xreg) {
      fim[idx_t, idx_b] <- K_tb
      fim[idx_b, idx_t] <- t(K_tb)
    }
    fim[idx_t, idx_phi] <- K_tphi
    fim[idx_phi, idx_t] <- t(K_tphi)
  }
  
  if (has_xreg) {
    fim[idx_b, idx_b] <- K_bb
    fim[idx_b, idx_phi] <- K_bphi
    fim[idx_phi, idx_b] <- t(K_bphi)
  }
  
  fim[idx_phi, idx_phi] <- K_phiphi
  
  # --------------------------------------------------------------------------
  # 7. PENALIZED FISHER INFORMATION MATRIX
  # --------------------------------------------------------------------------
  if (penalty) {
    
    # Penalized Fisher information matrix (Cribari-Neto, Costa and Fonseca,
    # 2025, Section 2):
    #   K_pen(nu) = K(nu) + 2 * (n - a) * Lambda
    # where Lambda = diag{lambda_n, ..., lambda_n, 0, ..., 0}, with lambda_n
    # computed by .barma_ridge_lambda() and applied to (alpha, varphi, theta).
    # phi and beta are excluded from penalization (see paper, Section 2).
    lambda <- .barma_ridge_lambda(n_obs, max_lag)
    
    # Indicator vector for Lambda: 1 for penalized parameters, 0 otherwise.
    # diag(penalty_raw) builds the diagonal matrix Lambda.
    # Order must match fim: (alpha, varphi, theta, beta, phi)
    penalty_raw <- c(
      1,                      # alpha
      rep(1, n_ar_params),    # varphi
      rep(1, n_ma_params),    # theta
      rep(0, n_beta_params),  # beta: not penalized
      0                       # phi:  not penalized
    )
    
    penalty_diag <- diag(penalty_raw)
    
    penalty_term <- 2 * (n_obs - max_lag) * lambda * penalty_diag

    fim <- fim + penalty_term
    
  }
  
  # --------------------------------------------------------------------------
  # 8. PREPARE FITTED VALUES AND OUTPUT
  # --------------------------------------------------------------------------
  
  # Create fitted values as a time series object
  fitted_ts <- ts(
    c(rep(NA, max_lag), mu_eff),
    start = start(y),
    frequency = frequency(y)
  )
  
  # Name the matrix rows and columns
  names_varphi <- if (has_ar) paste0("varphi", ar_lags) else character(0)
  names_theta  <- if (has_ma) paste0("theta", ma_lags) else character(0)
  names_beta <- if (has_xreg) {
    if (!is.null(colnames(xreg))) colnames(xreg) else 
      paste0("beta", seq_len(n_beta_params))
  } else character(0)
  
  names_fim <- c("alpha", names_varphi, names_theta, names_beta, "phi")
  colnames(fim) <- names_fim
  rownames(fim) <- names_fim
  
  # Prepare output list with all required components
  output_list <- list(
    fisher_info_mat = fim,           # Fisher Information Matrix
    fitted_ts = fitted_ts,           # Fitted conditional means (ts object)
    muhat_effective = mu_eff,        # Effective fitted means (vector)
    etahat_full = eta,               # Full linear predictor (NA-padded)
    errorhat_full = error            # Full error/residuals (0-padded)
  )
  
  return(output_list)
}

#' Extract Fitted Values from a barma Model
#'
#' @description
#' S3 method for extracting the fitted mean values (mu-hat) from a fitted 
#' model object of class `"barma"`.
#'
#' @details
#' The fitted values are returned as a time series (`ts`) object, 
#' matching the time properties of the original input `y`. The first
#' `max_lag` observations are `NA`, as they cannot be fitted.
#'
#' @param object A fitted model object of class `"barma"`.
#' @param ... Additional arguments (currently ignored).
#'
#' @return
#' A `ts` object of the fitted mean values.
#'
#' @importFrom stats fitted
#' @export
#' @method fitted barma
fitted.barma <- function(object, ...) {
  return(object$fitted)
}

#' Forecast a barma Model
#'
#' @description
#' S3 method for producing forecasts from a fitted `barma` model object.
#'
#' @details
#' This function computes dynamic, multi-step-ahead point forecasts.
#' It implements a "Regression with ARMA errors" logic, where the AR components
#' are applied to the deviations from the regression line:
#' \eqn{AR(g(y_{t-k}) - x_{t-k}^\top \beta)}.
#'
#' @param object A fitted model object of class `barma`. Must contain
#'   `object$xreg` if regressors were used.
#' @param h The number of steps to forecast ahead (forecast horizon).
#'   Default is 6.
#' @param xreg A matrix of future regressor values for the forecast horizon.
#'   Should have h rows and the same number of columns as xreg used in model
#'   fitting.
#' @param ... Additional arguments (currently ignored).
#'
#' @return
#' A `ts` object containing the point forecasts for h steps ahead.
#'
#' @importFrom forecast forecast
#' @export
#' @method forecast barma
forecast.barma <- function(object, h = 6, xreg = NULL, ...) {
  
  # --------------------------------------------------------------------------
  # 1. Extract Model Components
  # --------------------------------------------------------------------------
  
  # --- Get parameters ---
  alpha <- object$alpha
  varphi <- object$varphi
  theta <- object$theta
  beta <- object$beta # Regression coefficients
  
  # --- Get link function ---
  link_structure <- make_link_structure(object$link)
  link_inv <- link_structure$linkinv
  link_fun <- link_structure$linkfun
  
  # --- Get lags ---
  ar_lags <- object$ar_lags
  ma_lags <- object$ma_lags
  has_ar <- length(ar_lags) > 0
  has_ma <- length(ma_lags) > 0
  has_regressors <- !is.null(beta) && length(beta) > 0
  
  # --- Validate regressors ---
  if (has_regressors) {
    if (is.null(xreg)) {
      stop(
        "Model was fit with regressors, but `xreg` is NULL. ",
        "Please provide `xreg` for forecasting."
      )
    }
    
    # Coerce to matrix immediately to prevent nrow() crashing
    xreg <- as.matrix(xreg)
    
    # Ensure object has the training xreg to calculate historical deviations
    if (is.null(object$xreg)) {
      stop(
        "The fitted model object does not contain `xreg`. ",
        "Historical regressors are required for AR terms in this specification."
      )
    }
    
    # Check dimensions
    if (nrow(xreg) != h) {
      stop("`xreg` must have ", h, " rows, but has ", nrow(xreg), " rows.")
    }
    if (ncol(xreg) != length(beta)) {
      stop(
        "`xreg` must have ", length(beta), " columns, but has ",
        ncol(xreg), " columns."
      )
    }
  } else {
    if (!is.null(xreg)) {
      warning(
        "`xreg` provided but model has no regressors. ",
        "`xreg` will be ignored."
      )
      xreg <- NULL
    }
  }
  
  # --------------------------------------------------------------------------
  # 2. Prepare Data Matrices
  # --------------------------------------------------------------------------
  
  # --- Get historical data ---
  y <- object$y
  n_obs <- object$n_obs
  
  # Get g(y)
  y_new <- link_fun(y)
  
  # Get historical errors (predictor scale)
  error_hat <- object$errorhat
  
  # --- Setup Padded Vectors ---
  # y_new_padded holds g(y_t) for t <= n_obs and eta_t for t > n_obs
  y_new_padded <- c(y_new, rep(NA_real_, h))
  error_padded <- c(error_hat, rep(NA_real_, h))
  
  # --- Setup Regressor Matrix (History + Future) ---
  if (has_regressors) {
    xreg_train <- as.matrix(object$xreg)
    xreg_test <- as.matrix(xreg)
    
    # Combine training and testing regressors into one large matrix.
    # This allows easy indexing for [t - ar_lags] regardless of whether the
    # lag falls in the past or future.
    xreg_combined <- rbind(xreg_train, xreg_test)
  }
  
  # Vector to store final forecasts
  forecast_values <- rep(NA_real_, h)
  
  # --------------------------------------------------------------------------
  # 3. Iterate and Compute Forecasts
  # --------------------------------------------------------------------------
  
  for (i in seq_len(h)) {
    t <- n_obs + i # The time step we are forecasting
    
    # --- 1. Base Level (Intercept) ---
    eta_forecast <- alpha
    
    # --- 2. Add Current Regression Component ---
    if (has_regressors) {
      # Add X_t * beta
      eta_forecast <- eta_forecast +
        as.numeric(crossprod(beta, xreg_combined[t, ]))
    }
    
    # --- 3. Add AR Component (on Deviations) ---
    if (has_ar) {
      # Calculate indices for the lags
      lag_indices <- t - ar_lags
      
      # Retrieve lagged values of Y (on link scale)
      y_lags <- y_new_padded[lag_indices]
      
      if (has_regressors) {
        # Retrieve lagged values of X * beta
        # We use xreg_combined to handle lags safely
        x_lags_beta <- xreg_combined[lag_indices, , drop = FALSE] %*% beta
        
        # AR term applies to the residual: (Y_{t-k} - X_{t-k} * beta)
        ar_term <- as.numeric(crossprod(varphi, y_lags - x_lags_beta))
      } else {
        # Standard AR if no regressors
        ar_term <- as.numeric(crossprod(varphi, y_lags))
      }
      
      eta_forecast <- eta_forecast + ar_term
    }
    
    # --- 4. Add MA Component ---
    if (has_ma) {
      # MA term applies to errors
      eta_forecast <- eta_forecast +
        as.numeric(crossprod(theta, error_padded[t - ma_lags]))
    }
    
    # --- Store Forecast ---
    mu_forecast <- link_inv(eta_forecast)
    forecast_values[i] <- mu_forecast
    
    # --- Update padded vectors for next iteration ---
    # The expected value of g(y_t) is the forecasted eta_t
    y_new_padded[t] <- eta_forecast
    
    # The expected value of future errors is 0
    error_padded[t] <- 0
  }
  
  # --------------------------------------------------------------------------
  # 4. Format and Return as "forecast" Object
  # --------------------------------------------------------------------------
  
  y_ts <- if (stats::is.ts(y)) y else stats::ts(y)
  ts_start <- stats::tsp(y_ts)[2] + (1 / stats::frequency(y_ts))
  ts_freq <- stats::frequency(y_ts)
  
  forecast_ts <- stats::ts(
    forecast_values,
    start = ts_start,
    frequency = ts_freq
  )
  
  # Build the standard forecast list
  res <- list(
    method    = "betaARMA",
    model     = object,
    level     = c(80, 95),
    mean      = forecast_ts,
    x         = y_ts,
    fitted    = stats::fitted(object),
    residuals = stats::residuals(object)
  )
  
  # Assign the CRAN standard class
  class(res) <- "forecast"
  
  return(res)

}

#' Diagnostic Plots for a Fitted betaARMA Model
#'
#' @description
#' Produces diagnostic plots for a \eqn{\beta}ARMA model fitted by
#' \code{\link{barma}}. By default (\code{which = "default"}), four panels are
#' displayed in a 2\eqn{\times}2 grid: observed vs. fitted values, residual
#' ACF, Ljung-Box p-values, and residual PACF.
#'
#' @param x
#'   An object of class \code{"barma"}, as returned by \code{\link{barma}}.
#' @param which
#'   A character string controlling which panel(s) to display. One of:
#'   \describe{
#'     \item{\code{"default"} (default)}{Four-panel 2\eqn{\times}2 grid:
#'       observed vs. fitted (top left), residual ACF (top right),
#'       Ljung-Box p-values (bottom left), and residual PACF (bottom right).}
#'     \item{\code{"all"}}{Six-panel 3\eqn{\times}2 grid:
#'       observed vs. fitted (top left), residuals over time (top right),
#'       residual ACF (middle left), residual PACF (middle right),
#'       Ljung-Box p-values (bottom left), Monti p-values (bottom right).}
#'     \item{\code{"fitted"}}{Single panel: observed vs. fitted values.}
#'     \item{\code{"tsplot"}}{Single panel: residuals plotted as a time series,
#'       with a horizontal reference line at zero and dashed lines at
#'       \eqn{\pm 3} (ad hoc threshold; not shown for \code{"raw"} residuals).}
#'     \item{\code{"acf"}}{Single panel: residual ACF.}
#'     \item{\code{"pacf"}}{Single panel: residual PACF.}
#'     \item{\code{"ljungbox"}}{Single panel: Ljung-Box p-values.}
#'     \item{\code{"monti"}}{Single panel: Monti test p-values.}
#'     \item{\code{"hist"}}{Single panel: residual distribution histogram with
#'       kernel density overlay.}
#'     \item{\code{"qq"}}{Single panel: Normal Q-Q plot. Always uses quantile
#'       residuals regardless of \code{residual_type}; a message is issued if
#'       the type is overridden.}
#'   }
#' @param residual_type
#'   Character string passed to \code{\link{residuals.barma}} controlling
#'   which residuals are computed and displayed. One of \code{"pearson"}
#'   (default), \code{"quantile"}, \code{"link"} or \code{"raw"}. Pearson
#'   residuals are recommended for ACF/PACF analysis and portmanteau
#'   tests (Scher, Cribari-Neto, Pumi, and Bayer, 2020); quantile residuals
#'   are recommended for normality assessment (Dunn and Smyth, 1996). See also
#'   Ferrari and Cribari-Neto (2004).
#' @param lag_max
#'   Maximum lag to display in the ACF, PACF, Ljung-Box, and Monti panels.
#'   Defaults to \code{24}, appropriate for monthly series.
#' @param title
#'   An optional character string used as the overall title of the plot.
#'   Defaults to \code{NULL} (no title).
#' @param colour_observed
#'   Colour for observed values in the fitted-values panel.
#'   Defaults to \code{"#00BFC4"} (teal).
#' @param colour_fitted
#'   Colour for fitted values in the fitted-values panel.
#'   Defaults to \code{"#F8766D"} (red).
#' @param colour_residual
#'   Colour for residual lines, ACF/PACF bars, portmanteau test points, and
#'   histogram fill. Defaults to \code{"#7CAE00"} (green).
#' @param ...
#'   Additional arguments (currently unused, included for S3 consistency).
#'
#' @details
#' **Residual type.** All panels (except \emph{Observed vs. Fitted}) use the
#' residuals selected by \code{residual_type}. The residual type is displayed
#' in each panel subtitle for transparency.
#'
#' **Reference lines in the residuals-over-time panel.** The dashed horizontal 
#' lines are drawn at \eqn{\pm 3} as an ad hoc threshold for identifying 
#' atypical observations. No distributional assumption is made; observations 
#' outside these bounds may warrant individual inspection. Pearson and 
#' link-scale residuals do not follow a standard normal distribution, so 
#' normal-distribution-based quantiles (e.g., \eqn{\pm 1.96} or 
#' \eqn{\pm 2.576}) are not appropriate reference values for such residuals. 
#' Quantile residuals are approximately distributed as \eqn{N(0,1)} under a 
#' correctly specified model, so \eqn{\pm 3} is a conservative but reasonable 
#' threshold for them.
#'
#' **Q-Q plot residuals.** The \code{"qq"} panel always uses quantile
#' residuals (Dunn and Smyth, 1996), regardless of \code{residual_type}.
#' Quantile residuals are the only type theoretically expected to follow
#' \eqn{N(0,1)} under a correctly specified model; using Pearson, raw, or
#' link-scale residuals in a normal Q-Q plot would produce misleading results.
#' A message is issued when \code{residual_type} is overridden.
#'
#' **Effective sample size.** Both the Ljung-Box and Monti test statistics
#' rely on the effective sample size \eqn{N_{eff} = n - \max(p, q)} (where
#' \eqn{\max(p, q)} corresponds to the maximum lag in the model).
#'
#' **Ljung-Box test.** The p-values are computed from the Ljung-Box test
#' statistic.
#' \eqn{Q_{LB}(k) = N_{eff}(N_{eff}+2)\sum_{j=1}^{k}\hat{\rho}_j^2/(N_{eff}-j)},
#' where \eqn{\hat{\rho}_j} is the \eqn{j}-th sample autocorrelation of the
#' residuals and \eqn{N_{eff} = n - \max(p, q)} is the effective sample size.
#' The degrees of freedom are adjusted by subtracting the total
#' number of estimated  AR and MA parameters,
#' \eqn{n_{ar} + n_{ma}} (Scher, Cribari-Neto, Pumi, and Bayer (2020)). The
#'  first \eqn{n_{ar} + n_{ma}} lags are set to \code{NA}
#' since the test statistic is not defined for \eqn{k \leq n_{ar} + n_{ma}}.
#'
#' **Monti test.** The p-values are computed from the test statistic
#' \eqn{M(k) = N_{eff}(N_{eff}+2)\sum_{j=1}^{k}\hat{\rho}_{jj}^2/(N_{eff}-j)},
#' where \eqn{\hat{\rho}_{jj}} is the \eqn{j}-th sample partial autocorrelation
#' of the residuals, and \eqn{N_{eff} = n - \max\_lag} is the effective sample
#' size after stripping initial NAs. The degrees of freedom are adjusted to
#' \eqn{k - (n_{ar} + n_{ma})}
#' (Monti, 1994; Scher, Cribari-Neto, Pumi, and Bayer (2020)). The first
#' \eqn{n_{ar} + n_{ma}} lags are set to \code{NA} as the statistic is not
#' defined for \eqn{k \leq n_{ar} + n_{ma}}.
#'
#' @return
#'   For grid requests (\code{"default"} or \code{"all"}), invisibly returns a
#'   \code{gtable} object produced by \code{gridExtra::arrangeGrob}, which can
#'   be saved with \code{ggplot2::ggsave}. For single-panel requests, invisibly
#'   returns the individual \code{ggplot} object.
#'
#' @seealso
#'   \code{\link{barma}} for model fitting;
#'   \code{\link{residuals.barma}} for the residual extraction method;
#'   \code{\link{fitted.barma}} for the fitted-value extraction method;
#'   \code{\link{forecast.barma}} for out-of-sample forecasting.
#'
#' @references
#'   Dunn, P. K., & Smyth, G. K. (1996). Randomized quantile residuals.
#'   \emph{Journal of Computational and Graphical Statistics}, 5(3), 236--244.
#'   \doi{10.1080/10618600.1996.10474708}
#'
#'   Ferrari, S. L. P., & Cribari-Neto, F. (2004). Beta regression for
#'   modelling rates and proportions. \emph{Journal of Applied Statistics},
#'   31(7), 799--815. \doi{10.1080/0266476042000214501}
#'
#'   Monti, A. C. (1994). A proposal for a residual autocorrelation test in
#'   linear models. \emph{Biometrika}, 81(4), 776--780.
#'   \doi{10.1093/biomet/81.4.776}
#'
#'   Rocha, A. V., & Cribari-Neto, F. (2009). Beta autoregressive moving
#'   average models. \emph{TEST}, 18(3), 529--545.
#'   \doi{10.1007/s11749-008-0112-z}
#'
#'   Rocha, A. V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
#'   moving average models. \emph{TEST}, 26, 451--459.
#'   \doi{10.1007/s11749-017-0528-4}
#'
#' @examples
#' data(brasilia_ts)
#' fit <- barma(brasilia_ts, ar = 1, ma = 1)
#'
#' # Default four-panel diagnostic grid (Pearson residuals)
#' plot(fit)
#'
#' # Six-panel diagnostic grid
#' plot(fit, which = "all")
#'
#' # Single panel: residuals as time series
#' plot(fit, which = "tsplot")
#'
#' # Single panel: Ljung-Box p-values only
#' plot(fit, which = "ljungbox")
#'
#' # Single panel: Monti p-values only
#' plot(fit, which = "monti")
#'
#' # Normal Q-Q plot (always uses quantile residuals)
#' plot(fit, which = "qq")
#'
#' # Histogram with quantile residuals
#' plot(fit, which = "hist", residual_type = "quantile")
#'
#' @importFrom forecast ggAcf ggPacf
#' @importFrom gridExtra arrangeGrob
#' @importFrom grid grid.draw
#' @importFrom ggplot2 ggplot aes geom_line geom_hline geom_point
#'   geom_histogram geom_density stat_qq stat_qq_line scale_colour_manual
#'   scale_y_continuous scale_x_continuous labs theme_minimal theme
#'   element_text element_blank after_stat
#' @importFrom stats acf pacf time fitted residuals density pchisq
#' @importFrom rlang .data
#' @export
plot.barma <- function(x,
                       which = c(
                         "default", "all", "fitted",
                         "tsplot", "acf", "pacf",
                         "ljungbox", "monti", "hist", "qq"
                       ),
                       residual_type = c("pearson", "quantile", "link", "raw"),
                       lag_max = 24,
                       title = NULL,
                       colour_observed = "#00BFC4",
                       colour_fitted = "#F8766D",
                       colour_residual = "#7CAE00",
                       ...) {
  # --------------------------------------------------------------------------
  # 0. Input validation
  # --------------------------------------------------------------------------
  if (!inherits(x, "barma")) {
    stop("'x' must be an object of class \"barma\".")
  }

  which <- match.arg(which)
  residual_type <- match.arg(residual_type)

  # Q-Q plot requires quantile residuals — override silently with a message
  if (which == "qq" && residual_type != "quantile") {
    message(
      "Note: the Q-Q plot uses quantile residuals regardless of ",
      "'residual_type'. Switching to residual_type = \"quantile\"."
    )
    residual_type <- "quantile"
  }

  # Label used in panel subtitles to make residual type explicit
  resid_label <- switch(residual_type,
    pearson  = "Pearson residuals",
    raw      = "Raw residuals",
    quantile = "Quantile residuals",
    link     = "Link-scale residuals"
  )

  # --------------------------------------------------------------------------
  # 1. Extract components from the barma object
  # --------------------------------------------------------------------------
  y_obs <- as.numeric(x$y)
  y_fit <- as.numeric(fitted(x))
  y_res <- as.numeric(residuals(x, type = residual_type))
  t_axis <- as.numeric(time(x$y))
  n <- length(y_obs)

  # Pad fitted values with leading NAs
  n_pad_fit <- n - length(y_fit)
  if (n_pad_fit > 0) {
    y_fit <- c(rep(NA_real_, n_pad_fit), y_fit)
  }

  # Guard against edge cases where residual lengths differ
  n_pad_res <- n - length(y_res)
  if (n_pad_res > 0) {
    y_res_padded <- c(rep(NA_real_, n_pad_res), y_res)
  } else {
    y_res_padded <- y_res
  }

  # Strip NAs for panels that require a clean residual vector
  y_res_clean <- y_res_padded[!is.na(y_res_padded)]
  n_clean <- length(y_res_clean)

  # Degrees of freedom for portmanteau tests: Number of estimated parameters
  n_ar <- length(x$ar_lags)
  n_ma <- length(x$ma_lags)
  fitdf <- n_ar + n_ma

  lags <- seq_len(lag_max)

  # --------------------------------------------------------------------------
  # 2. Compute Ljung-Box p-values (vectorized)
  #
  # Original Ljung-Box formula:
  # Q(k) = N_eff * (N_eff + 2) * sum_{j=1}^k r_j^2 / (N_eff - j)
  #
  # Variable Mapping in this code:
  # N_eff = n_clean (effective sample size after stripping initial NAs).
  # j     = lags    (the sequence 1, 2, ..., lag_max).
  #
  # Degrees of freedom for pchisq: k - fitdf
  # (Scher, Cribari-Neto, Pumi, and Bayer (2020)).
  # The first 'fitdf' lags are set to NA because the chi-square distribution
  # requires strictly positive degrees of freedom (k > fitdf).
  # --------------------------------------------------------------------------
  valid_lags <- lags > fitdf

  # acf()$acf includes lag 0 (always 1.0); drop it to get lags 1..lag_max
  acf_vals <- as.numeric(
    stats::acf(y_res_clean, lag.max = lag_max, plot = FALSE)$acf
  )[-1]

  lb_inner_terms <- (acf_vals^2) / (n_clean - lags)
  Q_stats <- n_clean * (n_clean + 2) * cumsum(lb_inner_terms)

  lb_pvals <- rep(NA_real_, lag_max)
  lb_pvals[valid_lags] <- stats::pchisq(
    Q_stats[valid_lags],
    df         = lags[valid_lags] - fitdf,
    lower.tail = FALSE
  )
  lb_df <- data.frame(lag = lags, pvalue = lb_pvals)

  # --------------------------------------------------------------------------
  # 3. Compute Monti test p-values (vectorized)
  #
  # Original Monti (1994) formula:
  # M(k) = N_eff * (N_eff + 2) * sum_{j=1}^k rho_jj^2 / (N_eff - j)
  #
  # Variable Mapping in this code:
  # N_eff = n_clean (effective sample size after stripping initial NAs).
  # j     = lags    (the sequence 1, 2, ..., lag_max).
  #
  # Degrees of freedom for pchisq: k - (n_ar + n_ma)
  # (Scher, Cribari-Neto, Pumi, and Bayer (2020)).
  # --------------------------------------------------------------------------
  pacf_vals <- as.numeric(
    stats::pacf(y_res_clean, lag.max = lag_max, plot = FALSE)$acf
  )

  inner_terms <- (pacf_vals^2) / (n_clean - lags)
  M_stats <- n_clean * (n_clean + 2) * cumsum(inner_terms)

  monti_pvals <- rep(NA_real_, lag_max)
  valid_lags <- lags > fitdf
  monti_pvals[valid_lags] <- stats::pchisq(
    M_stats[valid_lags],
    df         = lags[valid_lags] - fitdf,
    lower.tail = FALSE
  )
  monti_df <- data.frame(lag = lags, pvalue = monti_pvals)

  # --------------------------------------------------------------------------
  # 4. Shared data frame and ggplot theme
  # --------------------------------------------------------------------------
  df <- data.frame(
    time     = t_axis,
    observed = y_obs,
    fitted   = y_fit,
    residual = y_res_padded
  )

  barma_theme <- ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.text       = ggplot2::element_text(size = 9),
      axis.title      = ggplot2::element_text(size = 9),
      legend.text     = ggplot2::element_text(size = 9),
      legend.title    = ggplot2::element_blank(),
      legend.position = "bottom"
    )

  # Shared x-axis breaks aligned to seasonal lags (6, 12, 18, 24, ...)
  x_breaks <- ggplot2::scale_x_continuous(breaks = seq(6, lag_max, by = 6))

  # --------------------------------------------------------------------------
  # 5. Build all individual panels
  # --------------------------------------------------------------------------

  # --- Panel 1: Observed vs. Fitted ---
  p_fitted <- ggplot2::ggplot(df, ggplot2::aes(x = .data$time)) +
    ggplot2::geom_line(ggplot2::aes(y = .data$observed, colour = "Observed"),
      linewidth = 0.6, na.rm = TRUE
    ) +
    ggplot2::geom_line(ggplot2::aes(y = .data$fitted, colour = "Fitted"),
      linewidth = 0.6, na.rm = TRUE
    ) +
    ggplot2::scale_colour_manual(
      values = c("Observed" = colour_observed, "Fitted" = colour_fitted),
      breaks = c("Observed", "Fitted")
    ) +
    ggplot2::labs(x = "Time", y = "Value", title = "Observed vs. Fitted") +
    barma_theme

  # --- Panel 2: Residuals over time ---
  # Reference lines at ±3 as an ad hoc threshold. No distributional
  # assumption is made — see @details for the rationale.

  # 2.1 Dynamically set the subtitle based on residual type
  resid_subtitle <- if (residual_type == "raw") {
    resid_label
  } else {
    paste0(resid_label, "  |  dashed lines: \u00b13 (ad hoc)")
  }

  # 2.2. Build the base plot (without the +/- 3 lines)
  p_resid <- ggplot2::ggplot(
    df,
    ggplot2::aes(
      x = .data$time,
      y = .data$residual
    )
  ) +
    ggplot2::geom_line(
      colour = colour_residual, linewidth = 0.6,
      na.rm = TRUE
    ) +
    ggplot2::geom_hline(
      yintercept = 0, linetype = "solid",
      colour = "grey20", linewidth = 0.3
    ) +
    ggplot2::labs(
      x        = "Time",
      y        = resid_label,
      title    = "Residuals over Time",
      subtitle = resid_subtitle
    ) +
    barma_theme

  # 2.3. Conditionally add the dashed reference lines
  if (residual_type != "raw") {
    p_resid <- p_resid +
      ggplot2::geom_hline(
        yintercept = c(-3, 3), linetype = "dashed",
        colour = "grey40", linewidth = 0.4
      )
  }

  # --- Panel 3: Residual ACF ---
  # suppressMessages() silences "Scale for x is already present" warning
  # that arises when overriding the default x scale set by ggAcf internally.
  p_acf <- suppressMessages(
    forecast::ggAcf(y_res_clean, lag.max = lag_max) +
      x_breaks +
      ggplot2::labs(
        title = "Residual ACF",
        subtitle = resid_label,
        x = "Lag", y = "ACF"
      ) +
      barma_theme
  )

  # --- Panel 4: Residual PACF ---
  # suppressMessages() silences "Scale for x is already present" warning
  # that arises when overriding the default x scale set by ggPacf internally.
  p_pacf <- suppressMessages(
    forecast::ggPacf(y_res_clean, lag.max = lag_max) +
      x_breaks +
      ggplot2::labs(
        title = "Residual PACF",
        subtitle = resid_label,
        x = "Lag", y = "PACF"
      ) +
      barma_theme
  )

  # --- Panel 5: Ljung-Box p-values ---
  p_lb <- ggplot2::ggplot(
    lb_df,
    ggplot2::aes(x = .data$lag, y = .data$pvalue)
  ) +
    ggplot2::geom_point(colour = colour_residual, size = 2, na.rm = TRUE) +
    ggplot2::geom_hline(
      yintercept = 0.05, linetype = "dashed",
      colour = "grey40", linewidth = 0.4
    ) +
    x_breaks +
    ggplot2::scale_y_continuous(limits = c(0, 1)) +
    ggplot2::labs(
      x = "Lag", y = "p-value",
      title = "Ljung-Box p-values",
      subtitle = resid_label
    ) +
    barma_theme

  # --- Panel 6: Monti test p-values ---
  p_monti <- ggplot2::ggplot(
    monti_df,
    ggplot2::aes(x = .data$lag, y = .data$pvalue)
  ) +
    ggplot2::geom_point(colour = colour_residual, size = 2, na.rm = TRUE) +
    ggplot2::geom_hline(
      yintercept = 0.05, linetype = "dashed",
      colour = "grey40", linewidth = 0.4
    ) +
    x_breaks +
    ggplot2::scale_y_continuous(limits = c(0, 1)) +
    ggplot2::labs(
      x = "Lag", y = "p-value",
      title = "Monti p-values",
      subtitle = resid_label
    ) +
    barma_theme

  # --- Panel 7: Residual distribution histogram (built on demand) ---
  needs_hist <- which %in% c("hist")
  p_hist <- if (needs_hist) {
    ggplot2::ggplot(
      data.frame(residual = y_res_clean),
      ggplot2::aes(x = .data$residual)
    ) +
      ggplot2::geom_histogram(
        ggplot2::aes(y = ggplot2::after_stat(density)),
        bins = 20,
        fill = colour_residual,
        colour = "white",
        alpha = 0.8
      ) +
      ggplot2::geom_density(linewidth = 0.7, colour = "grey20") +
      ggplot2::labs(
        x = resid_label,
        y = "Density",
        title = "Residual Distribution",
        subtitle = resid_label
      ) +
      barma_theme
  } else {
    NULL
  }

  # --- Panel 8: Normal Q-Q plot (built on demand; always quantile residuals) ---
  # Quantile residuals are the only type with a theoretical N(0,1) guarantee
  # under a correctly specified model (Dunn & Smyth, 1996). Residuals for the
  # Q-Q plot are therefore always computed with type = "quantile", independent
  # of the global residual_type chosen by the user.
  needs_qq <- which %in% c("qq", "all")
  p_qq <- if (needs_qq) {
    qq_res <- as.numeric(residuals(x, type = "quantile"))
    qq_res <- qq_res[!is.na(qq_res)]
    n_qq <- length(qq_res)

    conf_level <- 0.95
    p_vals <- (seq_len(n_qq) - 0.5) / n_qq
    q_theor <- stats::qnorm(p_vals)
    se <- sqrt(p_vals * (1 - p_vals) / n_qq) / stats::dnorm(q_theor)
    z_val <- stats::qnorm((1 + conf_level) / 2)
    mean_qq <- mean(qq_res, na.rm = TRUE)
    sd_qq <- stats::sd(qq_res, na.rm = TRUE)

    qq_band_df <- data.frame(
      theoretical = q_theor,
      ymin = (q_theor - z_val * se) * sd_qq + mean_qq,
      ymax = (q_theor + z_val * se) * sd_qq + mean_qq
    )

    ggplot2::ggplot() +
      ggplot2::geom_ribbon(
        data = qq_band_df,
        ggplot2::aes(
          x = .data$theoretical,
          ymin = .data$ymin,
          ymax = .data$ymax
        ),
        fill = colour_residual, alpha = 0.2
      ) +
      ggplot2::stat_qq(
        data = data.frame(residual = qq_res),
        ggplot2::aes(sample = .data$residual),
        colour = colour_residual, size = 2, alpha = 0.6
      ) +
      ggplot2::stat_qq_line(
        data = data.frame(residual = qq_res),
        ggplot2::aes(sample = .data$residual),
        colour = "grey20", linewidth = 0.7
      ) +
      ggplot2::labs(
        x        = "Theoretical Quantiles",
        y        = "Sample Quantiles",
        title    = "Normal Q-Q Plot",
        subtitle = "Quantile residuals"
      ) +
      barma_theme
  } else {
    NULL
  }

  # --------------------------------------------------------------------------
  # 6. Return single panel if requested
  # --------------------------------------------------------------------------
  if (!(which %in% c("all", "default"))) {
    p_single <- switch(which,
      fitted   = p_fitted,
      tsplot   = p_resid,
      acf      = p_acf,
      pacf     = p_pacf,
      ljungbox = p_lb,
      monti    = p_monti,
      hist     = p_hist,
      qq       = p_qq
    )
    print(p_single)
    return(invisible(p_single))
  }

  # --------------------------------------------------------------------------
  # 7. Return "default" (4-panel grid):
  #    Observed vs. Fitted  |  Ljung-Box p-values
  #    Residual ACF         |  Residual PACF
  # --------------------------------------------------------------------------
  if (which == "default") {
    g <- gridExtra::arrangeGrob(
      grobs = list(
        p_fitted, p_lb,
        p_acf, p_pacf
      ),
      ncol = 2,
      top = title
    )
    grid::grid.draw(g)
    return(invisible(g))
  }

  # --------------------------------------------------------------------------
  # 8. Return "all" (6-panel grid):
  #    Observed vs. Fitted  |  Residuals over Time
  #    Residual ACF         |  Residual PACF
  #    Ljung-Box p-values   |  Monti p-values
  # --------------------------------------------------------------------------
  if (which == "all") {
    g <- gridExtra::arrangeGrob(
      grobs = list(
        p_fitted, p_resid,
        p_acf,    p_pacf,
        p_lb,     p_monti
      ),
      ncol = 2,
      top = title
    )
    grid::grid.draw(g)
    return(invisible(g))
  }
}

#' Extract Coefficients from a barma Model
#'
#' @description
#' S3 method for extracting the vector of estimated coefficients from a fitted 
#' model object of class `"barma"`.
#'
#' @param object A fitted model object of class `"barma"`.
#' @param ... Additional arguments (currently ignored).
#'
#' @return
#' A named numeric vector of all estimated coefficients 
#' (e.g., alpha, varphi, theta, phi).
#'
#' @importFrom stats coef
#' @export
#' @method coef barma
coef.barma <- function(object, ...) {
  return(object$coef)
}

#' Fit Beta Autoregressive Moving Average (BARMA) Models
#'
#' @title Fit Beta Autoregressive Moving Average (BARMA) Models via
#' Conditional Maximum Likelihood
#'
#' @description
#' Fits a Beta Autoregressive Moving Average (BARMA) model to time series data
#' valued in (0, 1) using Conditional Maximum Likelihood Estimation (CMLE).
#' The function performs complete model estimation including parameter
#' estimation, hypothesis testing infrastructure, and model diagnostics.
#'
#' @details
#' This function fits the BARMA(p,q) model as proposed by Rocha & Cribari-Neto
#' (2009, with erratum 2017). It serves as the main wrapper for the optimization
#' process, calling specialized helper functions for likelihood computation,
#' gradient calculation, and Fisher Information Matrix estimation.
#'
#' **Model Specification**: The BARMA model is defined as:
#'
#' \deqn{g(\mu_t) = \alpha + X_t\beta + \sum_{i=1}^{p} \varphi_i
#' (g(y_{t-i}) - X_{t-i}\beta) + \sum_{j=1}^{q} \theta_j \epsilon_{t-j}}
#'
#' where \eqn{y_t | F_{t-1} \sim Beta(\mu_t\phi, (1-\mu_t)\phi)},
#' \eqn{g(\cdot)} is the link function, and \eqn{F_{t-1}} is the information
#' set at time t-1.
#'
#' **Model Types** (specified via `ar` and `ma` arguments):
#' \itemize{
#'   \item **BARMA(p,q):** Both `ar` and `ma` are specified.
#'   \item **BAR(p):** Only `ar` is specified; omit `ma` or pass `integer(0)`.
#'   \item **BMA(q):** Only `ma` is specified; omit `ar` or pass `integer(0)`.
#' }
#'
#' **External Regressors**: Covariates can be included via `xreg`. The model
#' becomes a regression BARMA, where the mean depends on current covariates
#' and lagged responses/errors.
#'
#' **Optimization**: The function uses quasi-Newton algorithms
#' (BFGS or L-BFGS-B) via \code{\link[stats]{optim}}, or the
#' \code{\link[lbfgs]{lbfgs}} function, utilizing analytic gradients for
#' efficiency. Initial values are obtained from the internal `start_values()`
#' function.
#'
#' **Implementation Notes**:
#' - The conditional log-likelihood is computed by conditioning on the first
#'   \eqn{m = \max(p,q)} observations, which are required to initialize the
#'   recursive structure of the model.
#' - The 2017 Erratum corrections are implemented for correct handling of
#'   moving average components in the score vector and Fisher Information Matrix.
#' - All computations are vectorized where possible for efficiency.
#'
#' @author
#' Everton da Costa (Federal University of Pernambuco,
#' \email{everto.cost@gmail.com});
#' Francisco Cribari-Neto (Federal University of Pernambuco,
#' \email{francisco.cribari@ufpe.br})
#'
#' @note
#' The original version of this function was developed by Fabio M. Bayer
#' (Federal University of Santa Maria, \email{bayer@ufsm.br}). It has been
#' substantially modified and improved by Everton da Costa, with suggestions
#' and contributions from Francisco Cribari-Neto.
#'
#' @references
#' Rocha, A.V., & Cribari-Neto, F. (2009). Beta autoregressive moving
#' average models. \emph{TEST}, 18(3), 529-545.
#' \doi{10.1007/s11749-008-0112-z}
#'
#' Rocha, A.V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
#' moving average models. \emph{TEST}, 26, 451-459.
#' \doi{10.1007/s11749-017-0528-4}
#'
#' Cribari-Neto, F., Costa, E., & Fonseca, R.V. (2025). Numerical stability
#' enhancements in beta autoregressive moving average model estimation.
#' \emph{Brazilian Journal of Probability and Statistics}, 39(4), 410-437.
#' \doi{10.1214/25-BJPS645}
#'
#' @param y
#' A time series object (`ts`) with values strictly in the open interval (0, 1).
#' Must have at least \eqn{\max(p,q) + 1} observations.
#'
#' @param ar
#' A numeric vector specifying autoregressive (AR) lags
#' (e.g., `c(1, 2)` for AR(2)). Defaults to `integer(0)`, which omits the
#' AR component entirely. Absence should be expressed by omitting this
#' argument or passing `integer(0)`.
#'
#' @param ma
#' A numeric vector specifying moving average (MA) lags
#' (e.g., `1` for MA(1)). Defaults to `integer(0)`, which omits the
#' MA component entirely. Absence should be expressed by omitting this
#' argument or passing `integer(0)`.
#'
#' @param link
#' The link function connecting the mean \eqn{\mu_t} to the linear predictor
#' \eqn{\eta_t}. One of:
#' \itemize{
#'   \item `"logit"` (default): \eqn{g(x) = \log(x/(1-x))}
#'   \item `"probit"`: \eqn{g(x) = \Phi^{-1}(x)}
#'   \item `"cloglog"`: Complementary log-log link
#'   \item `"loglog"`: Log-log link
#' }
#'
#' @param xreg
#' A matrix or data frame of external regressors (covariates), optional.
#' Must have the same number of rows as `y`. If provided, its columns
#' are included in the linear predictor with associated coefficients in `beta`.
#'
#' @param optimization
#' A named list controlling the optimization procedure. Recognized fields:
#' \describe{
#'   \item{\code{method}}{Character string specifying the algorithm. One of
#'     \code{"BFGS"} (default), \code{"L-BFGS-B"}, or \code{"lbfgs"}.}
#'   \item{\code{lower}}{Numeric vector or scalar of lower bounds, used only
#'     by \code{"L-BFGS-B"}. Defaults to \code{-Inf}.}
#'   \item{\code{upper}}{Numeric vector or scalar of upper bounds, used only
#'     by \code{"L-BFGS-B"}. Defaults to \code{Inf}.}
#' }
#'
#' @param penalty
#' Logical. If \code{TRUE}, the ridge penalization scheme of Cribari-Neto,
#' Costa and Fonseca (2025) is applied during conditional maximum likelihood
#' estimation. The penalty term \eqn{(n - a)\lambda_n \|\boldsymbol{\nu}\|^2}
#' is subtracted from the conditional log-likelihood, where
#' \eqn{\lambda_n = 1/(n - a)^{0.9}} and \eqn{\boldsymbol{\nu}} collects
#' \eqn{\alpha}, \eqn{\boldsymbol{\varphi}}, and \eqn{\boldsymbol{\theta}}.
#' The precision parameter \eqn{\phi} and regression coefficients
#' \eqn{\boldsymbol{\beta}} are excluded from penalization.
#' Ridge penalization enhances the curvature of the conditional log-likelihood
#' surface, reducing convergence failures and the occurrence of implausible
#' estimates. Defaults to \code{FALSE}.
#'
#' @return
#' An object of class `"barma"` containing:
#' \item{coef}{Named vector of all estimated parameters, ordered as:
#'   alpha, AR parameters, MA parameters, beta parameters, phi.}
#' \item{vcov}{The variance-covariance matrix of the estimators, computed as
#'   the inverse of the observed Fisher Information Matrix.}
#' \item{model}{A summary table with coefficients, standard errors, z-statistics,
#'   and p-values for hypothesis tests \eqn{H_0: \theta_i = 0}.}
#' \item{fitted}{Fitted conditional mean values as a `ts` object
#'   (NA-padded for the first \eqn{m = \max(p,q)} observations).}
#' \item{muhat}{Alias for `fitted` (fitted mean values).}
#' \item{etahat}{Estimated linear predictor values (full vector, NA-padded).}
#' \item{errorhat}{Estimated errors on predictor scale (full vector, 0-padded).}
#' \item{loglik}{The conditional log-likelihood at the CMLE.}
#' \item{fisher_info_mat}{The expected Fisher Information Matrix
#' (Rocha & Cribari-Neto, 2009).}
#' \item{conv}{Convergence code from `optim` (0 = success).}
#' \item{alpha, beta, varphi, theta, phi}{Individual parameter estimates.}
#' \item{start_values}{Initial parameter values used in optimization.}
#' \item{call}{The original function call.}
#' \item{opt}{Cleaned output list from the optimization procedure.}
#' \item{opt_raw}{Raw output object returned directly by `optim()` or `lbfgs()`.}
#'
#' @seealso
#' \code{\link{simu_barma}} for simulation,
#' \code{\link{loglik_barma}} for likelihood computation,
#' \code{\link{score_vector_barma}} for gradient computation,
#' \code{\link{fim_barma}} for Fisher Information Matrix
#'
#' @examples
#' \donttest{
#' # Example 1: Fit a BAR(1) model (no MA component)
#' set.seed(2025)
#' y_sim_bar1 <- simu_barma(
#'   n      = 250,
#'   alpha  = 0.0,
#'   varphi = 0.6,
#'   phi    = 25.0,
#'   link   = "logit",
#'   freq   = 12
#' )
#'
#' fit_bar1 <- barma(y_sim_bar1, ar = 1, link = "logit")
#' summary(fit_bar1)
#' coef(fit_bar1)
#'
#' # Example 2: Fit a BARMA(1, 1) model
#' set.seed(2025)
#' y_sim_barma11 <- simu_barma(
#'   n      = 250,
#'   alpha  = 0.0,
#'   varphi = 0.6,
#'   theta  = 0.3,
#'   phi    = 25.0,
#'   link   = "logit",
#'   freq   = 12
#' )
#'
#' fit_barma11 <- barma(y_sim_barma11, ar = 1, ma = 1, link = "logit")
#' summary(fit_barma11)
#'
#' # Example 3: Fit a BMA(1) model (no AR component)
#' set.seed(2025)
#' y_sim_bma1 <- simu_barma(
#'   n      = 250,
#'   alpha  = 0.0,
#'   theta  = 0.3,
#'   phi    = 20.0,
#'   link   = "logit",
#'   freq   = 12
#' )
#'
#' fit_bma1 <- barma(y_sim_bma1, ma = 1, link = "logit")
#' summary(fit_bma1)
#'
#' # Example 4: BARMA(1, 1) model with harmonic seasonal regressors
#' hs <- sin(2 * pi * seq_along(y_sim_barma11) / 12)
#' hc <- cos(2 * pi * seq_along(y_sim_barma11) / 12)
#' X <- cbind(hs = hs, hc = hc)
#'
#' fit_barma11_xreg <- barma(
#'   y_sim_barma11,
#'   ar   = 1,
#'   ma   = 1,
#'   link = "logit",
#'   xreg = X
#' )
#' summary(fit_barma11_xreg)
#'
#' # Example 5: Fit a BARMA(1, 1) model using L-BFGS-B
#' set.seed(2025)
#' y_sim_barma11_LBFGSB <- simu_barma(
#'   n      = 250,
#'   alpha  = 0.0,
#'   varphi = 0.6,
#'   theta  = 0.3,
#'   phi    = 25.0,
#'   link   = "logit",
#'   freq   = 12
#' )
#'
#' fit_barma11_LBFGSB <- barma(
#'   y_sim_barma11_LBFGSB,
#'   ar = 1,
#'   ma = 1,
#'   link = "logit",
#'   optimization = list(
#'     method = "L-BFGS-B",
#'     upper = c(Inf, 1, 1, Inf)
#'   )
#' )
#'
#' summary(fit_barma11_LBFGSB)
#' }
#'
#' @importFrom stats is.ts optim dbeta frequency pnorm start ts
#'
#' @export
barma <- function(
    y,
    ar = integer(0),
    ma = integer(0),
    link = "logit",
    xreg = NULL,
    optimization = list(method = "BFGS", lower = -Inf, upper = Inf),
    penalty = FALSE) {
  # 0. CAPTURE CALL ==========================================================
  z <- list(call = match.call())

  # 1. VALIDATE INPUT PARAMETERS =============================================

  # Check time series object -------------------------------------------------
  if (!is.ts(y)) {
    stop(
      "'y' must be a time series object (ts). ",
      "Use ts(data, frequency = ...) to convert."
    )
  }

  # Check for missing values -------------------------------------------------
  if (any(is.na(y))) {
    stop("'y' contains missing values (NA).")
  }

  # Check unit interval bounds -----------------------------------------------
  if (min(y) <= 0 || max(y) >= 1) {
    stop(
      "'y' values must be strictly in (0, 1). ",
      "Current range: [", round(min(y), 4), ", ", round(max(y), 4), "]."
    )
  }

  # Check link function ------------------------------------------------------
  valid_links <- c("logit", "probit", "cloglog", "loglog")
  if (!is.character(link) || !(link %in% valid_links)) {
    stop(
      "'link' must be one of: ",
      paste(valid_links, collapse = ", "), ". ",
      "Current: '", link, "'."
    )
  }

  # Check optimization argument ----------------------------------------------
  if (!is.list(optimization)) {
    stop("'optimization' must be a named list, e.g. list(method = 'BFGS').")
  }

  valid_methods <- c("BFGS", "L-BFGS-B", "lbfgs")

  if (is.null(optimization$method)) {
    stop(
      "'optimization$method' is missing. Must be one of: ",
      paste(valid_methods, collapse = ", "), "."
    )
  }

  if (!optimization$method %in% valid_methods) {
    stop(
      "Unknown method '", optimization$method, "'. Must be one of: ",
      paste(valid_methods, collapse = ", "), "."
    )
  }

  # Check xreg ----------------------------------------------------------------
  if (!is.null(xreg)) {
    if (!is.matrix(xreg)) {
      xreg <- as.matrix(xreg)
    }

    if (nrow(xreg) != length(y)) {
      stop(
        "'xreg' must have ", length(y), " rows to match length of 'y'. ",
        "Current: ", nrow(xreg), "."
      )
    }

    if (any(is.na(xreg))) {
      stop("'xreg' contains missing values (NA).")
    }
  }

  # 2. DETERMINE MODEL STRUCTURE =============================================

  # Backward compatibility ---------------------------------------------------
  # in v1.0.1, absence was expressed as ar = NA or ma = NA.
  # Convert silently to integer(0) and warn the user.

  if (length(ar) == 1 && is.na(ar)) {
    warning(
      "Passing 'ar = NA' is deprecated. ",
      "Use 'ar = integer(0)' or omit the argument to indicate no AR component."
    )
    ar <- integer(0)
  }
  if (length(ma) == 1 && is.na(ma)) {
    warning(
      "Passing 'ma = NA' is deprecated. ",
      "Use 'ma = integer(0)' or omit the argument to indicate no MA component."
    )
    ma <- integer(0)
  }

  # Coerce to integer — enforces correct type if user passes numeric ---------
  ar_lags <- as.integer(ar)
  ma_lags <- as.integer(ma)

  has_xreg <- !is.null(xreg)

  n_ar_params <- length(ar_lags)
  n_ma_params <- length(ma_lags)
  n_beta_params <- if (has_xreg) ncol(xreg) else 0L

  has_ar <- n_ar_params > 0
  has_ma <- n_ma_params > 0

  # Maximum lag required to initialize the recursive structure
  ar_order <- if (has_ar) max(ar_lags) else 0L
  ma_order <- if (has_ma) max(ma_lags) else 0L
  max_lag <- max(ar_order, ma_order)

  n_obs <- length(y)

  if (n_obs <= max_lag) {
    stop(
      "Insufficient observations (", n_obs, ") ",
      "for specified lag structure (max_lag = ", max_lag, "). ",
      "Need at least ", max_lag + 1, " observations."
    )
  }

  # Parameter names for output -----------------------------------------------
  names_varphi <- paste0("varphi", ar_lags)
  names_theta <- paste0("theta", ma_lags)

  beta_names <- if (has_xreg) {
    if (!is.null(colnames(xreg))) {
      colnames(xreg)
    } else {
      paste0("beta", seq_len(n_beta_params))
    }
  } else {
    character(0)
  }

  # 3. SETUP LINK FUNCTION AND TRANSFORMED DATA ==============================

  # Get link function structure
  link_structure <- make_link_structure(link)
  linkfun <- link_structure$linkfun
  linkinv <- link_structure$linkinv

  # Transform response variable to predictor scale
  y_transformed <- linkfun(y)

  # 4. SETUP PARAMETER INDICES ===============================================
  par_idx <- make_par_indices(n_ar_params, n_ma_params, n_beta_params)

  idx_alpha <- par_idx$idx_alpha
  idx_varphi <- par_idx$idx_varphi
  idx_theta <- par_idx$idx_theta
  idx_beta <- par_idx$idx_beta
  idx_phi <- par_idx$idx_phi
  n_params <- par_idx$n_params

  # 5. GET INITIAL PARAMETER VALUES ==========================================

  # Call start_values function to get initial parameter estimates
  init_pars <- start_values(
    y,
    ar = ar_lags,
    ma = ma_lags,
    xreg = xreg,
    link = link
  )

  if (is.null(init_pars) || length(init_pars) != n_params) {
    warning(
      "start_values() returned unexpected length or NULL. ",
      "Optimization may fail or take longer."
    )
  }

  # 6. OPTIMIZATION (CMLE) ===================================================

  z$opt_method <- optimization$method
  z$opt_bounds <- !(all(optimization$lower == -Inf) &&
                      all(optimization$upper == Inf))

  # Call optim with BFGS algorithm using analytic gradient
  opt_list <- run_optimizer(
    y = y,
    init_pars = init_pars,
    ar_lags = ar_lags,
    ma_lags = ma_lags,
    xreg = xreg,
    link = link,
    par_idx = par_idx,
    optimization = optimization,
    penalty = penalty
  )

  # Check convergence
  if (opt_list$opt$convergence != 0) {
    warning(
      "Optimization did not converge (method = '", optimization$method, "'). ",
      "Convergence code: ", opt_list$opt$convergence, ". ",
      "Results may be unreliable. ",
      "Consider checking input data."
    )
  }

  z$opt <- opt_list$opt
  z$opt_raw <- opt_list$opt_raw
  z$conv <- opt_list$opt$convergence
  z$loglik <- opt_list$opt$loglik

  coef_raw <- opt_list$opt$par

  # Extract estimates
  z$alpha <- coef_raw[idx_alpha]
  z$varphi <- coef_raw[idx_varphi]
  z$theta <- coef_raw[idx_theta]
  z$beta <- coef_raw[idx_beta]
  z$phi <- coef_raw[idx_phi]

  z$coef <- coef_raw

  # 7. COMPUTE FISHER INFORMATION MATRIX & FITTED VALUES =====================

  # Call fim_barma to compute Fisher Information Matrix and fitted values
  fim_results <- fim_barma(
    y = y,
    ar = ar_lags,
    ma = ma_lags,
    alpha = z$alpha,
    varphi = z$varphi,
    theta = z$theta,
    phi = z$phi,
    link = link,
    xreg = xreg,
    beta = z$beta,
    penalty = penalty
  )

  # Extract FIM and model fit statistics
  z$fisher_info_mat <- fim_results$fisher_info_mat
  z$fitted <- fim_results$fitted_ts # ts object with NAs
  z$muhat <- fim_results$fitted_ts # Alias for fitted
  z$etahat <- fim_results$etahat_full # Full vector, NA-padded
  z$errorhat <- fim_results$errorhat_full # Full vector, 0-padded

  # 8. STORE ADDITIONAL INFORMATION FOR METHODS ==============================

  z$y <- y
  z$link <- link
  z$start_values <- init_pars
  z$n_params <- n_params
  z$n_obs <- n_obs
  z$max_lag <- max_lag
  z$ar_lags <- ar_lags
  z$ma_lags <- ma_lags
  z$xreg <- xreg # Store for predict() and residuals() methods
  z$penalty <- penalty
  z$lambda <- if (penalty) .barma_ridge_lambda(n_obs, max_lag)

  # 9. ASSIGN CLASS AND RETURN ===============================================

  class(z) <- "barma"

  return(z)
}

#' @title Score Vector for the BARMA Model
#' @description Computes the score vector (gradient of the log-likelihood)
#'   for the Beta Autoregressive Moving Average (BARMA) model at a given
#'   parameter vector. This function is designed for users who:
#' \itemize{
#'   \item Implement custom optimization algorithms
#'   \item Verify theoretical properties
#'   \item Conduct simulation studies
#'   \item Debug model fitting
#'   \item Integrate BARMA models into their own workflows
#' }
#'
#' @param y
#' A numeric vector representing the time series data, with values
#' strictly in (0, 1).
#'
#' @param ar
#' A numeric vector specifying the autoregressive (AR) lags
#' (e.g., \code{c(1, 2)}). Defaults to \code{integer(0)}, which omits the
#' AR component entirely. Absence should be expressed by omitting this
#' argument or passing \code{integer(0)}.
#'
#' @param ma
#' A numeric vector specifying the moving average (MA) lags
#' (e.g., \code{1}). Defaults to \code{integer(0)}, which omits the
#' MA component entirely. Absence should be expressed by omitting this
#' argument or passing \code{integer(0)}.
#'
#' @param alpha
#' The intercept term (numeric scalar).
#'
#' @param varphi
#' A numeric vector of autoregressive (AR) parameters.
#' Absence should be expressed by omitting this argument or passing
#' \code{numeric(0)}.
#'
#' @param theta
#' A numeric vector of moving average (MA) parameters.
#' Absence should be expressed by omitting this argument or passing
#' \code{numeric(0)}.
#'
#' @param phi
#' The precision parameter of the BARMA model (must be positive and finite).
#' Larger values indicate less variance for a given mean.
#'
#' @param link
#' A character string specifying the link function:
#' \code{"logit"} (default), \code{"probit"}, or \code{"cloglog"}.
#'
#' @param xreg
#' A matrix or data frame of static regressors (optional).
#' Must have the same number of rows as length of \code{y}.
#'
#' @param beta
#' A numeric vector of regression coefficients for \code{xreg} (optional).
#' Length must match number of columns in \code{xreg}.
#'
#' @param penalty
#' Logical. If \code{TRUE}, the gradient of the ridge penalty of
#' Cribari-Neto, Costa and Fonseca (2025) is subtracted from the score
#' vector, yielding the penalized score
#' \eqn{S_{\text{pen}}(\boldsymbol{\nu}) = S(\boldsymbol{\nu}) -
#' 2(n-a)\lambda_n \boldsymbol{\nu}}, where \eqn{\lambda_n =
#' 1/(n-a)^{0.9}} and \eqn{\boldsymbol{\nu}} collects \eqn{\alpha},
#' \eqn{\boldsymbol{\varphi}}, and \eqn{\boldsymbol{\theta}}
#' (\eqn{\phi} and \eqn{\boldsymbol{\beta}} are excluded).
#' Defaults to \code{FALSE}.
#'
#' @return A numeric vector of the same length as the parameter vector,
#'   giving the partial derivatives of the log-likelihood with respect
#'   to each parameter. The order of the components is:
#'   \code{(alpha, varphi, theta, beta, phi)}.
#'
#' @seealso \code{\link{barma}}, \code{\link{loglik_barma}},
#'   \code{\link{fim_barma}}
#'
#' @examples
#' \donttest{
#'   # Example 1: Score vector for a BAR(1) model (no MA component)
#'   set.seed(2025)
#'   y_sim_bar1 <- simu_barma(
#'     n      = 250,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     phi    = 25.0,
#'     link   = "logit",
#'     freq   = 12
#'   )
#'
#'   score_vector_barma(
#'     y      = y_sim_bar1,
#'     ar     = 1,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     theta  = numeric(0),
#'     phi    = 25.0,
#'     link   = "logit"
#'   )
#'
#'   # Example 2: Score vector for a BARMA(1, 1) model
#'   set.seed(2025)
#'   y_sim_barma11 <- simu_barma(
#'     n      = 250,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     theta  = 0.3,
#'     phi    = 25.0,
#'     link   = "logit",
#'     freq   = 12
#'   )
#'
#'   score_vector_barma(
#'     y      = y_sim_barma11,
#'     ar     = 1,
#'     ma     = 1,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     theta  = 0.3,
#'     phi    = 25.0,
#'     link   = "logit"
#'   )
#'
#'   # Example 3: Score vector for a BMA(1) model (no AR component)
#'   set.seed(2025)
#'   y_sim_bma1 <- simu_barma(
#'     n      = 250,
#'     alpha  = 0.0,
#'     theta  = 0.3,
#'     phi    = 20.0,
#'     link   = "logit",
#'     freq   = 12
#'   )
#'
#'   score_vector_barma(
#'     y      = y_sim_bma1,
#'     ma     = 1,
#'     alpha  = 0.0,
#'     varphi = numeric(0),
#'     theta  = 0.3,
#'     phi    = 20.0,
#'     link   = "logit"
#'   )
#' }
#'
#' @export
score_vector_barma <- function(
    y,
    ar   = integer(0),
    ma   = integer(0),
    alpha,
    varphi = numeric(0),
    theta = numeric(0),
    phi,
    link = "logit",
    xreg = NULL,
    beta = NULL, 
    penalty = FALSE
) {
  
  # ------------------------------------------------------------------------
  # 1. Validate Precision Parameter
  # ------------------------------------------------------------------------
  if (phi <= 0 || !is.finite(phi)) {
    warning("phi must be positive and finite; returning zero gradient")
    n_params <- 1 + length(varphi) + length(theta) + 1 + length(beta)
    return(rep(0, n_params))
  }
  
  # ------------------------------------------------------------------------
  # 2. Determine Model Structure
  # ------------------------------------------------------------------------
  
  # Lag vectors
  ar_lags <- ar
  ma_lags <- ma
  
  # Force varphi / theta to numeric(0) when the corresponding component
  # is absent, so that drop(crossprod(numeric(0), numeric(0))) == 0.
  varphi <- if (length(ar_lags) > 0) varphi else numeric(0)
  theta  <- if (length(ma_lags) > 0) theta  else numeric(0)
  
  # Setup Regressors
  has_xreg <- !is.null(xreg)
  
  if (has_xreg) {
    if (!is.matrix(xreg)) xreg <- as.matrix(xreg)
    if (is.null(beta)) stop("If 'xreg' is provided, 'beta' is required.")
    if (ncol(xreg) != length(beta)) {
      stop("Length of 'beta' must match columns of 'xreg'.")
    }
    if (nrow(xreg) != length(y)) {
      stop("Number of rows in 'xreg' must match length of 'y'.")
    }
    # Pre-compute X * beta for efficiency
    xb <- as.vector(xreg %*% beta)
    n_beta_params <- length(beta)
  } else {
    xb <- numeric(length(y))
    n_beta_params <- 0
  }
  
  n_ar_params <- length(varphi)
  n_ma_params <- length(theta)
  
  # Validate parameter-lag consistency
  if (n_ar_params != length(ar_lags)) {
    stop("Mismatch between 'ar' lags and 'varphi' parameters.")
  }
  if (n_ma_params != length(ma_lags)) {
    stop("Mismatch between 'ma' lags and 'theta' parameters.")
  }
  
  # ------------------------------------------------------------------------
  # 3. Setup Link Functions and Time Series Properties
  # ------------------------------------------------------------------------
  
  # Get link function structures
  link_structure <- make_link_structure(link)
  linkfun <- link_structure$linkfun
  linkinv <- link_structure$linkinv
  mu_eta_fun <- link_structure$mu.eta
  
  # Transform response using link function
  y_transformed <- linkfun(y)
  
  # Determine maximum lag
  ar_order <- if (length(ar_lags) > 0) max(ar_lags) else 0
  ma_order <- if (length(ma_lags) > 0) max(ma_lags) else 0
  max_lag  <- max(ar_order, ma_order)
  n_obs <- length(y)
  
  if (n_obs <= max_lag) {
    warning("Insufficient observations for the specified lag structure")
    n_params <- 1 + n_ar_params + n_ma_params + 1 + n_beta_params
    return(rep(0, n_params))
  }
  
  # ------------------------------------------------------------------------
  # 4. Recursive Calculation: Predictor, Error, and Derivatives
  # ------------------------------------------------------------------------
  error <- rep(0, n_obs)
  eta   <- rep(NA_real_, n_obs)
  
  # Initialize derivative matrices
  d_eta_d_alpha  <- rep(0, n_obs)
  d_eta_d_varphi <- matrix(0, nrow = n_obs, ncol = n_ar_params)
  d_eta_d_theta  <- matrix(0, nrow = n_obs, ncol = n_ma_params)
  d_eta_d_beta   <- matrix(0, nrow = n_obs, ncol = n_beta_params)
  
  for (t in (max_lag + 1):n_obs) {
    
    # Part A: Compute Linear Predictor (eta)
    eta[t] <- alpha + xb[t] +
      drop(crossprod(varphi, y_transformed[t - ar_lags] - xb[t - ar_lags])) +
      drop(crossprod(theta,  error[t - ma_lags]))
    
    error[t] <- y_transformed[t] - eta[t]
    
    # Part B: Compute Recursive Derivatives
    
    # 1. Derivative w.r.t. alpha
    d_eta_d_alpha[t] <- 1 -
      drop(crossprod(theta, d_eta_d_alpha[t - ma_lags]))
    
    # 2. Derivative w.r.t. AR (varphi)
    if (n_ar_params > 0) {
      d_eta_d_varphi[t, ] <- y_transformed[t - ar_lags] - xb[t - ar_lags] -
        drop(crossprod(theta, d_eta_d_varphi[t - ma_lags, , drop = FALSE]))
    }
    
    # 3. Derivative w.r.t. MA (theta)
    if (n_ma_params > 0) {
      d_eta_d_theta[t, ] <- error[t - ma_lags] -
        drop(crossprod(theta, d_eta_d_theta[t - ma_lags, , drop = FALSE]))
    }
    
    # 4. Derivative w.r.t. beta (regressors)
    if (n_beta_params > 0) {
      d_eta_d_beta[t, ] <- xreg[t, ] -
        drop(crossprod(varphi, xreg[t - ar_lags, , drop = FALSE])) -
        drop(crossprod(theta,  d_eta_d_beta[t - ma_lags, , drop = FALSE]))
    }
  }
  
  # ------------------------------------------------------------------------
  # 5. Calculate Score Vector Components
  # ------------------------------------------------------------------------
  eta_effective <- eta[(max_lag + 1):n_obs]
  y_effective   <- y[(max_lag + 1):n_obs]
  mu_effective  <- linkinv(eta = eta_effective)
  
  # Numerical stability check
  if (any(mu_effective <= 0 | mu_effective >= 1 | !is.finite(mu_effective))) {
    warning("mu_effective out of bounds; returning zero gradient")
    n_params <- 1 + n_ar_params + n_ma_params + 1 + n_beta_params
    return(rep(0, n_params))
  }
  
  mu_eta_val <- mu_eta_fun(eta = eta_effective)
  ystar  <- log(y_effective / (1 - y_effective))
  mustar <- digamma(mu_effective * phi) - digamma((1 - mu_effective) * phi)
  
  # Chain rule common term: dL/deta = dL/dmu * dmu/deta
  common_term <- mu_eta_val * (ystar - mustar)
  
  # Compute final scores. Using crossprod for sum(vector * vector)
  
  idx <- (max_lag + 1):n_obs
  
  score_alpha <- as.numeric(
    phi * crossprod(d_eta_d_alpha[idx], common_term)
  )
  
  score_varphi <- as.numeric(
    phi * crossprod(d_eta_d_varphi[idx, , drop = FALSE], common_term)
  )
  
  score_theta <- as.numeric(
    phi * crossprod(d_eta_d_theta[idx, , drop = FALSE], common_term)
  )
  
  score_beta <- as.numeric(
    phi * crossprod(d_eta_d_beta[idx, , drop = FALSE], common_term)
  )
  
  score_phi <- sum(
    mu_effective * (ystar - mustar) +
      log(1 - y_effective) -
      digamma((1 - mu_effective) * phi) +
      digamma(phi)
  )
  
  # ------------------------------------------------------------------------
  # 6. Assemble and Return
  # ------------------------------------------------------------------------
  final_score <- c(score_alpha,
                   score_varphi, 
                   score_theta, 
                   score_beta,
                   score_phi)
  
  # --------------------------------------------------------------------------
  # 7. RIDGE PENALIZATION (gradient of the penalty term)
  # --------------------------------------------------------------------------
  if (penalty) {
    
    # Penalized score function (Cribari-Neto, Costa and Fonseca, 2025, Section 2):
    #   S_pen(nu) = S(nu) - 2 * (n - a) * lambda_n * nu
    # where lambda_n is computed by .barma_ridge_lambda().
    # phi and beta are excluded from penalization (see paper, Section 2).
    lambda <- .barma_ridge_lambda(n_obs, max_lag)
    
    # Gradient of the penalty w.r.t. each parameter: 
    # d/d(nu_j) [(n-a)*lambda*nu_j^2] = 2*(n-a)*lambda*nu_j
    # Order must match final_score: (alpha, varphi, theta, beta, phi)
    penalty_grad_raw <- c(
      2 * alpha,              # intercept
      2 * varphi,             # AR parameters
      2 * theta,              # MA parameters
      rep(0, n_beta_params),  # beta: not penalized
      0                       # phi:  not penalized
    )
    
    penalty_term <- (n_obs - max_lag) * lambda * penalty_grad_raw
    
    final_score <- final_score - penalty_term
    
  }
  
  if (any(!is.finite(final_score))) {
    warning("Non-finite values in score vector; returning zeros")
    return(rep(0, length(final_score)))
  }
  
  return(as.numeric(final_score))
}

#' Log-Likelihood for BARMA Models
#'
#' @title
#' Compute Conditional Log-Likelihood for Beta Autoregressive
#' Moving Average Models
#'
#' @description
#' Computes the conditional log-likelihood of a Beta Autoregressive
#' Moving Average (BARMA) model. This function is designed for users who:
#' \itemize{
#'   \item Implement custom optimization algorithms
#'   \item Verify theoretical properties
#'   \item Conduct simulation studies
#'   \item Debug model fitting
#'   \item Integrate BARMA models into their own workflows
#' }
#'
#' @details
#' The log-likelihood is computed as:
#' \deqn{\ell = \sum_{t=m+1}^{n} \log f(y_t | \mu_t, \phi)}
#'
#' where \eqn{f} is the Beta density with shape parameters
#' \eqn{shape1 = \mu_t * \phi} and \eqn{shape2 = (1-\mu_t)*\phi}.
#'
#' The linear predictor is constructed as:
#' \deqn{\eta_t = \alpha + X_t \beta + \sum_{i=1}^{p} \varphi_i
#' (y_{t-i} - X_{t-i}\beta) + \sum_{j=1}^{q} \theta_j \epsilon_{t-j}}
#'
#' **Important**: This function implements the corrections from the 2017
#' Erratum (Rocha & Cribari-Neto, 2017) for moving average components.
#' See References section for details.
#'
#' **Parameter Order**: Parameters should be supplied in the order:
#' alpha, varphi (AR), theta (MA), phi, beta (regressors).
#' This matches the parameter order used by \code{\link{barma}}.
#'
#' @param y
#' A numeric vector representing the time series data, with values
#' strictly in (0, 1).
#'
#' @param ar
#' A numeric vector specifying the autoregressive (AR) lags
#' (e.g., \code{c(1, 2)}). Defaults to \code{integer(0)}, which omits the
#' AR component entirely. Absence should be expressed by omitting this
#' argument or passing \code{integer(0)}.
#'
#' @param ma
#' A numeric vector specifying the moving average (MA) lags
#' (e.g., \code{1}). Defaults to \code{integer(0)}, which omits the
#' MA component entirely. Absence should be expressed by omitting this
#' argument or passing \code{integer(0)}.
#'
#' @param alpha
#' The intercept term (numeric scalar).
#'
#' @param varphi
#' A numeric vector of autoregressive (AR) parameters.
#' Absence should be expressed by omitting this argument or passing
#' \code{numeric(0)}.
#'
#' @param theta
#' A numeric vector of moving average (MA) parameters.
#' Absence should be expressed by omitting this argument or passing
#' \code{numeric(0)}.
#'
#' @param phi
#' The precision parameter of the BARMA model (must be positive and finite).
#' Larger values indicate less variance for a given mean.
#'
#' @param link
#' A character string specifying the link function:
#' \code{"logit"} (default), \code{"probit"}, or \code{"cloglog"}.
#'
#' @param xreg
#' A matrix or data frame of static regressors (optional).
#' Must have the same number of rows as length of \code{y}.
#'
#' @param beta
#' A numeric vector of regression coefficients for \code{xreg} (optional).
#' Length must match number of columns in \code{xreg}.
#'
#' @param penalty
#' Logical. If \code{TRUE}, a ridge penalty is subtracted from the
#' conditional log-likelihood, yielding the penalized conditional
#' log-likelihood proposed by Cribari-Neto, Costa and Fonseca (2025).
#' The penalty term is \eqn{(n - a) \lambda_n \|\boldsymbol{\nu}\|^2},
#' where \eqn{\lambda_n = 1/(n - a)^{0.9}} and \eqn{\boldsymbol{\nu}}
#' collects \eqn{\alpha}, \eqn{\boldsymbol{\varphi}}, and
#' \eqn{\boldsymbol{\theta}} (the precision parameter \eqn{\phi} and
#' regression coefficients \eqn{\boldsymbol{\beta}} are excluded).
#' Defaults to \code{FALSE}.
#'
#' @return
#' A numeric scalar representing the (penalized) conditional log-likelihood
#' value. When \code{penalty = FALSE} (default), the standard conditional
#' log-likelihood is returned. When \code{penalty = TRUE}, the ridge-penalized
#' conditional log-likelihood of Cribari-Neto, Costa and Fonseca (2025) is
#' returned instead.
#' Returns \code{-Inf} if:
#' \itemize{
#'   \item \code{phi} is non-positive or non-finite
#'   \item Insufficient observations for specified lag structure
#'   \item Fitted values are outside (0, 1)
#'   \item Any numerical issues occur
#' }
#'
#' @references
#' Rocha, A.V., & Cribari-Neto, F. (2009). Beta autoregressive moving
#' average models. \emph{TEST}, 18(3), 529-545.
#' \doi{10.1007/s11749-008-0112-z}
#'
#' Rocha, A.V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
#' moving average models. \emph{TEST}, 26, 451-459.
#' \doi{10.1007/s11749-017-0528-4}
#'
#' Cribari-Neto, F., Costa, E., & Fonseca, R.V. (2025). Numerical stability
#' enhancements in beta autoregressive moving average model estimation.
#' \emph{Brazilian Journal of Probability and Statistics}, 39(4), 410-437.
#' \doi{10.1214/25-BJPS645}
#'
#' @seealso
#' \code{\link{barma}} for model fitting,
#' \code{\link{score_vector_barma}} for gradient computation,
#' \code{\link{fim_barma}} for Fisher Information Matrix
#'
#' @examples
#' \donttest{
#'   # Example 1: Log-likelihood for a BAR(1) model (no MA component)
#'   set.seed(2025)
#'   y_sim_bar1 <- simu_barma(
#'     n      = 250,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     phi    = 25.0,
#'     link   = "logit",
#'     freq   = 12
#'   )
#'
#'   loglik_barma(
#'     y      = y_sim_bar1,
#'     ar     = 1,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     theta  = numeric(0),
#'     phi    = 25.0,
#'     link   = "logit"
#'   )
#'
#'   # Example 2: Log-likelihood for a BARMA(1, 1) model
#'   set.seed(2025)
#'   y_sim_barma11 <- simu_barma(
#'     n      = 250,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     theta  = 0.3,
#'     phi    = 25.0,
#'     link   = "logit",
#'     freq   = 12
#'   )
#'
#'   loglik_barma(
#'     y      = y_sim_barma11,
#'     ar     = 1,
#'     ma     = 1,
#'     alpha  = 0.0,
#'     varphi = 0.6,
#'     theta  = 0.3,
#'     phi    = 25.0,
#'     link   = "logit"
#'   )
#'
#'   # Example 3: Log-likelihood for a BMA(1) model (no AR component)
#'   set.seed(2025)
#'   y_sim_bma1 <- simu_barma(
#'     n      = 250,
#'     alpha  = 0.0,
#'     theta  = 0.3,
#'     phi    = 20.0,
#'     link   = "logit",
#'     freq   = 12
#'   )
#'
#'   loglik_barma(
#'     y      = y_sim_bma1,
#'     ma     = 1,
#'     alpha  = 0.0,
#'     varphi = numeric(0),
#'     theta  = 0.3,
#'     phi    = 20.0,
#'     link   = "logit"
#'   )
#'
#'   # Example 4: Ridge-penalized log-likelihood for a BARMA(1, 1) model
#'   # Uses penalty = TRUE as recommended by Cribari-Neto, Costa and
#'   # Fonseca (2025, BJPS) to improve numerical stability.
#'   loglik_barma(
#'     y       = y_sim_barma11,
#'     ar      = 1,
#'     ma      = 1,
#'     alpha   = 0.0,
#'     varphi  = 0.6,
#'     theta   = 0.3,
#'     phi     = 25.0,
#'     link    = "logit",
#'     penalty = TRUE
#'   )
#' }
#'
#' @export
loglik_barma <- function(
    y,
    ar   = integer(0),
    ma   = integer(0),
    alpha,
    varphi = numeric(0),
    theta = numeric(0),
    phi,
    link = "logit",
    xreg = NULL,
    beta = NULL, 
    penalty = FALSE
) {
  
  # --------------------------------------------------------------------------
  # 1. VALIDATE INPUT PARAMETERS 
  # --------------------------------------------------------------------------
  
  # Validate response variable
  if (!is.numeric(y)) {
    stop("y must be a numeric vector.")
  }
  if (length(y) < 2) {
    stop("y must have at least 2 observations.")
  }
  if (anyNA(y)) {
    stop("y contains missing values (NA).")
  }
  if (min(y) <= 0 || max(y) >= 1) {
    stop(
      "Response values must be strictly in (0, 1). ",
      "Current range: [", round(min(y), 4), ", ", round(max(y), 4), "]."
    )
  }
  
  # Validate precision parameter
  if (!is.numeric(phi) || length(phi) != 1) {
    stop("phi must be a single numeric value.")
  }
  if (phi <= 0 || !is.finite(phi)) {
    return(-Inf)
  }
  
  # --------------------------------------------------------------------------
  # 2. DETERMINE MODEL STRUCTURE 
  # --------------------------------------------------------------------------
  
  # Lag vectors
  ar_lags <- ar
  ma_lags <- ma
  
  # Force varphi / theta to numeric(0) when the corresponding component
  # is absent, so that drop(crossprod(numeric(0), numeric(0))) == 0.
  varphi <- if (length(ar_lags) > 0) varphi else numeric(0)
  theta  <- if (length(ma_lags) > 0) theta  else numeric(0)
  
  # Validate parameter-lag consistency
  if (length(varphi) != length(ar_lags)) {
    stop(
      "Length of 'varphi' (", length(varphi), ") ",
      "must match length of 'ar' (", length(ar_lags), ")."
    )
  }
  if (length(theta) != length(ma_lags)) {
    stop(
      "Length of 'theta' (", length(theta), ") ",
      "must match length of 'ma' (", length(ma_lags), ")."
    )
  }
  
  # Setup Regressors
  if (!is.null(xreg)) {
    if (!is.matrix(xreg)) xreg <- as.matrix(xreg)
    
    if (is.null(beta)) {
      stop("If 'xreg' is provided, 'beta' must also be provided.")
    }
    if (!is.numeric(beta)) {
      stop("beta must be numeric.")
    }
    if (ncol(xreg) != length(beta)) {
      stop(
        "Length of 'beta' (", length(beta), ") ",
        "must match columns of 'xreg' (", ncol(xreg), ")."
      )
    }
    if (nrow(xreg) != length(y)) {
      stop(
        "Number of rows in 'xreg' (", nrow(xreg), ") ",
        "must match length of 'y' (", length(y), ")."
      )
    }
    if (anyNA(xreg)) {
      stop("xreg contains missing values (NA).")
    }
    
    # Pre-compute X * beta for efficiency (vectorized operation)
    xb <- as.vector(xreg %*% beta)
  } else {
    # Zero contribution from regressors if none provided
    xb <- numeric(length(y))
  }
  
  # --------------------------------------------------------------------------
  # 3. SETUP LINK FUNCTIONS AND TIME SERIES PROPERTIES
  # --------------------------------------------------------------------------
  
  # Get link function structures
  link_structure <- make_link_structure(link)
  linkfun <- link_structure$linkfun
  linkinv <- link_structure$linkinv
  
  # Transform response using link function
  y_transformed <- linkfun(y)
  
  # Determine maximum lag
  ar_order <- if (length(ar_lags) > 0) max(ar_lags) else 0
  ma_order <- if (length(ma_lags) > 0) max(ma_lags) else 0
  max_lag  <- max(ar_order, ma_order)
  n_obs <- length(y)
  
  # Check for sufficient observations
  if (n_obs <= max_lag) {
    warning(
      "Insufficient observations: have ", n_obs, 
      " observations but need > ", max_lag, " (max lag order)."
    )
    return(-Inf)
  }
  
  # --------------------------------------------------------------------------
  # 4. CALCULATE ERROR AND PREDICTOR
  # --------------------------------------------------------------------------
  
  # Initialize containers for linear predictor and errors
  # Observations 1:max_lag will have 0/NA values
  error <- rep(0, n_obs)
  eta   <- rep(NA_real_, n_obs)
  
  # Recursively compute predictor and errors.
  for (t in (max_lag + 1):n_obs) {
    eta[t] <- alpha + xb[t] +
      drop(crossprod(varphi, y_transformed[t - ar_lags] - xb[t - ar_lags])) +
      drop(crossprod(theta,  error[t - ma_lags]))
    
    error[t] <- y_transformed[t] - eta[t]
  }
  
  # --------------------------------------------------------------------------
  # 5. CALCULATE THE FINAL LOG-LIKELIHOOD
  # --------------------------------------------------------------------------
  
  # Extract observations used in likelihood
  idx_effective <- (max_lag + 1):n_obs
  eta_eff <- eta[idx_effective]
  y_eff   <- y[idx_effective]
  
  # Transform linear predictor to mean scale
  mu_eff <- linkinv(eta = eta_eff)
  
  # Check for valid mu values (must be in (0,1))
  if (any(mu_eff <= 0 | mu_eff >= 1 | !is.finite(mu_eff))) {
    return(-Inf)
  }
  
  # Compute log-likelihood using Beta density
  # dbeta parameterization: shape1 = mu*phi, shape2 = (1-mu)*phi
  ll_terms <- dbeta(
    y_eff,
    shape1 = mu_eff * phi,
    shape2 = (1 - mu_eff) * phi,
    log = TRUE
  )
  
  # Check for numerical issues (non-finite values)
  if (any(!is.finite(ll_terms))) {
    return(-Inf)
  }
  
  # Accumulate log-likelihood before optional penalization
  sum_ll_terms <- sum(ll_terms)
  
  # --------------------------------------------------------------------------
  # 6. RIDGE PENALIZATION
  # --------------------------------------------------------------------------
  if (penalty) {
    
    # Penalized log-likelihood per Equation (2) of Cribari-Neto, Costa and
    # Fonseca (2025, BJPS, Section 2):
    #   ell_pen(nu) = ell(nu) - (n - a) * lambda_n * ||nu||^2
    # where lambda_n is computed by .barma_ridge_lambda().
    # phi and beta are excluded from penalization (see paper, Section 2).
    lambda <- .barma_ridge_lambda(n_obs, max_lag)
    
    # L2 norm of penalized parameters (beta excluded per the paper)
    l2_norm_sq   <- alpha^2 + sum(varphi^2) + sum(theta^2)
    penalty_term <- (n_obs - max_lag) * lambda * l2_norm_sq
    
    sum_ll_terms <- sum_ll_terms - penalty_term
    
  }
  
  return(sum_ll_terms)
  
}

#' Residuals for a Fitted betaARMA Model
#'
#' @description
#' Computes residuals for a fitted \eqn{\beta}ARMA model object of class
#' \code{"barma"}. Four types are available, controlled by the \code{type}
#' argument.
#'
#' @details
#' **Pearson residuals** (default) are defined on the response scale as:
#' \deqn{r_t = \frac{y_t - \hat{\mu}_t}{\sqrt{\hat{\mu}_t(1-\hat{\mu}_t)/(1+\hat{\phi})}}}
#' where \eqn{\hat{\mu}_t} is the fitted conditional mean and \eqn{\hat{\phi}}
#' is the estimated precision parameter. This residual was defined in Ferrari
#' and Cribari-Neto (2004) for beta regression models and is adopted here in
#' the \eqn{\beta}ARMA setting. These residuals are appropriate for ACF/PACF
#' analysis and portmanteau tests (Ljung-Box and Monti).
#'
#' **Quantile residuals** (Dunn and Smyth, 1996) are defined as:
#' \deqn{r_t^Q = \Phi^{-1}\!\left(F(y_t;\,\hat{\mu}_t\hat{\phi},\,(1-\hat{\mu}_t)\hat{\phi})\right)}
#' where \eqn{F(\cdot\,;\,a,b)} is the Beta CDF with shape parameters
#' \eqn{a = \hat{\mu}_t\hat{\phi}} and \eqn{b = (1-\hat{\mu}_t)\hat{\phi}},
#' and \eqn{\Phi^{-1}} is the standard normal quantile function. Under a
#' correctly specified model, quantile residuals are approximately i.i.d.
#' \eqn{N(0,1)}, making them suitable for normality assessment via
#' histograms and Q-Q plots.
#' 
#' **Link-scale residuals** are defined on the predictor scale as:
#' \deqn{r_t^L = \frac{g(y_t) - \hat{\eta}_t}{\sqrt{[g'(\hat{\mu}_t)]^2
#'   \cdot \hat{\mu}_t(1-\hat{\mu}_t)/(1+\hat{\phi})}}}
#' where \eqn{g(\cdot)} is the link function, \eqn{\hat{\eta}_t} is the
#' fitted linear predictor, and \eqn{g'(\hat{\mu}_t) = d\eta/d\mu}. This
#' residual was used in earlier versions of the package and is retained for
#' compatibility.
#' 
#' **Raw residuals** are the simple difference on the response scale:
#' \deqn{r_t^{\mathrm{raw}} = y_t - \hat{\mu}_t}
#'
#'
#' @param object
#'   A fitted model object of class \code{"barma"}, as returned by
#'   \code{\link{barma}}.
#' @param type
#'   Character string specifying the type of residuals to compute. One of:
#'   \describe{
#'     \item{\code{"pearson"} (default)}{Pearson residuals on the response
#'       scale, as defined in Ferrari and Cribari-Neto (2004). Recommended
#'       for ACF/PACF analysis and portmanteau tests.}
#'     \item{\code{"quantile"}}{Quantile residuals (Dunn and Smyth, 1996),
#'       approximately i.i.d. \eqn{N(0,1)} under a correctly specified model.
#'       Recommended for normality assessment.}
#'     \item{\code{"link"}}{Residuals on the predictor (link) scale. Retained
#'       for compatibility with earlier versions of the package.}
#'     \item{\code{"raw"}}{Raw residuals \eqn{y_t - \hat{\mu}_t} on the
#'       response scale.}
#'   }
#' @param ...
#'   Additional arguments (currently unused, included for S3 consistency).
#'
#' @return
#'   A numeric \code{ts} object of the same length as the response series,
#'   with the first \code{max_lag} values set to \code{NA}. The \code{ts}
#'   attributes (\code{start} and \code{frequency}) are inherited from the
#'   original response series stored in \code{object$y}.
#'
#' @references
#'   Dunn, P. K., & Smyth, G. K. (1996). Randomized quantile residuals.
#'   \emph{Journal of Computational and Graphical Statistics}, 5(3), 236--244.
#'   \doi{10.1080/10618600.1996.10474708}
#'
#'   Ferrari, S. L. P., & Cribari-Neto, F. (2004). Beta regression for
#'   modelling rates and proportions. \emph{Journal of Applied Statistics},
#'   31(7), 799--815. \doi{10.1080/0266476042000214501}
#'
#'   Rocha, A. V., & Cribari-Neto, F. (2009). Beta autoregressive moving
#'   average models. \emph{TEST}, 18(3), 529--545.
#'   \doi{10.1007/s11749-008-0112-z}
#'
#'   Rocha, A. V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
#'   moving average models. \emph{TEST}, 26, 451--459.
#'   \doi{10.1007/s11749-017-0528-4}
#'
#' @seealso
#'   \code{\link{barma}} for model fitting;
#'   \code{\link{fitted.barma}} for fitted values;
#'   \code{\link{plot.barma}} for diagnostic plots.
#'
#' @examples
#' data(brasilia_ts)
#' fit <- barma(brasilia_ts, ar = 1, ma = 1)
#'
#' # Pearson residuals (default) -- Ferrari & Cribari-Neto (2004)
#' res <- residuals(fit)
#'
#' # Raw residuals
#' res_raw <- residuals(fit, type = "raw")
#'
#' # Quantile residuals -- recommended for normality assessment
#' res_q <- residuals(fit, type = "quantile")
#'
#' # Link-scale residuals
#' res_link <- residuals(fit, type = "link")
#'
#' @importFrom stats qnorm pbeta
#' @export
#' @method residuals barma
residuals.barma <- function(object,
                            type = c("pearson", "quantile", "link", "raw"),
                            ...) {
  
  type <- match.arg(type)
  
  # ---------------------------------------------------------------------------
  # 1. Extract components from the barma object
  # ---------------------------------------------------------------------------
  y <- object$y
  if (is.null(y)) {
    stop("Original time series 'y' not found in the 'barma' object.")
  }
  
  fitted_mus <- object$fitted  # Fitted means mu_hat (full length, NA-padded ts)
  phi        <- object$phi     # Estimated precision parameter
  max_lag    <- object$max_lag # Number of initial NAs (= max(ar, ma) lags)
  
  if (is.null(max_lag)) {
    stop("'max_lag' not found in the 'barma' object.")
  }
  
  n_obs <- length(y)
  
  if (n_obs <= max_lag) {
    warning("Insufficient observations relative to max_lag.")
    return(rep(NA_real_, n_obs))
  }
  
  # ---------------------------------------------------------------------------
  # 2. Restrict to effective observations (drop the first max_lag NAs)
  # ---------------------------------------------------------------------------
  idx_effective   <- (max_lag + 1):n_obs
  y_effective     <- as.numeric(y[idx_effective])
  muhat_effective <- as.numeric(fitted_mus[idx_effective])
  
  # ---------------------------------------------------------------------------
  # 3. Compute residuals by type
  # ---------------------------------------------------------------------------
  if (type == "pearson") {
    
    # Pearson residuals on the response scale.
    # Reference: Ferrari & Cribari-Neto (2004), adapted to the betaARMA setting.
    # r_t = (y_t - mu_hat_t) / sqrt( mu_hat_t*(1 - mu_hat_t) / (1 + phi) )
    Vmu_hat <- muhat_effective * (1 - muhat_effective)
    resids  <- (y_effective - muhat_effective) / sqrt(Vmu_hat / (1 + phi))
    
  } else if (type == "raw") {
    
    # Raw residuals on the response scale: y_t - mu_hat_t
    resids <- y_effective - muhat_effective
    
  } else if (type == "quantile") {
    
    # Quantile residuals: Phi^{-1}( F(y_t; mu_hat_t*phi, (1-mu_hat_t)*phi) )
    # Reference: Dunn & Smyth (1996).
    # Under a correctly specified model, these are approximately i.i.d. N(0,1).
    shape1 <- muhat_effective * phi
    shape2 <- (1 - muhat_effective) * phi
    resids  <- stats::qnorm(
      stats::pbeta(y_effective, shape1 = shape1, shape2 = shape2)
    )
    
  } else {
    
    # Link-scale residuals: (g(y_t) - eta_hat_t) / sd(r_t) on predictor scale.
    # Retained for compatibility with earlier versions of the package.
    etahat <- object$etahat  # Fitted linear predictor (full length, NA-padded)
    link   <- object$link
    
    link_structure <- make_link_structure(link)
    linkfun    <- link_structure$linkfun   # g(mu): maps (0,1) -> R
    mu.eta_fun <- link_structure$mu.eta    # dmu/deta
    
    etahat_effective <- etahat[idx_effective]
    
    # Numerator: g(y_t) - eta_hat_t
    num <- linkfun(y_effective) - etahat_effective
    
    # Denominator: sqrt( [g'(mu_hat_t)]^2 * mu_hat_t*(1-mu_hat_t) / (1+phi) )
    # where g'(mu) = deta/dmu = 1 / (dmu/deta)
    dmu_deta_sq <- mu.eta_fun(eta = etahat_effective)^2
    Vmu_hat     <- muhat_effective * (1 - muhat_effective)
    den         <- sqrt((1 / dmu_deta_sq) * Vmu_hat / (1 + phi))
    
    resids <- num / den
    
  }
  
  # ---------------------------------------------------------------------------
  # 4. Pad the first max_lag positions with NA and return as ts object
  # ---------------------------------------------------------------------------
  final_resids <- c(rep(NA_real_, max_lag), resids)
  
  if (stats::is.ts(y)) {
    final_resids <- stats::ts(
      final_resids,
      start     = stats::start(y),
      frequency = stats::frequency(y)
    )
  }
  
  return(final_resids)
}

# source_code.txt

#' Print Method for a barma Summary
#'
#' @description
#' S3 method for printing the detailed summary of a `"barma"` model object.
#'
#' @details
#' This function formats and displays the summary list created by
#' `summary.barma()`, including the call, coefficient table, and
#' information criteria.
#'
#' @param x A fitted model summary object of class `"summary.barma"`.
#' @param ... Additional arguments (currently ignored).
#'
#' @return
#' Invisibly returns the original object `x`.
#'
#' @export
#' @method print summary.barma
print.summary.barma <- function(x, ...) {
  # ---------------------------------------------------------------------- #
  # 1. Print Model and Call
  # ---------------------------------------------------------------------- #
  cat("Beta Autoregressive Moving Average Model\n")

  if (!is.null(x$call)) {
    cat("\nCall:\n", paste(deparse(x$call), collapse = "\n"), "\n",
      sep = ""
    )
  }

  # ---------------------------------------------------------------------- #
  # 2. Print Link Function
  # ---------------------------------------------------------------------- #
  if (!is.null(x$link)) {
    cat("\nLink function:", x$link, "\n")
  }

  # ---------------------------------------------------------------------- #
  # 2.5 Print Penalization Information
  # ---------------------------------------------------------------------- #
  if (isTRUE(x$penalty)) {
    lambda_str <- if (!is.null(x$lambda)) format(x$lambda, digits = 4) else "Unknown"
    cat(sprintf("Penalization : Ridge (lambda = %s)\n", lambda_str))
  }

  # ---------------------------------------------------------------------- #
  # 3. Print Coefficients Table
  # ---------------------------------------------------------------------- #
  if (!is.null(x$coefficients)) {
    cat("\nCoefficients:\n")
    stats::printCoefmat(x$coefficients,
      digits = max(3, getOption("digits") - 3),
      signif.stars = getOption("show.signif.stars"),
      na.print = "NA"
    )
  } else {
    cat("\nNo coefficients table found.\n")
  }

  # ---------------------------------------------------------------------- #
  # 4. Print Optimization and Convergence Status
  # ---------------------------------------------------------------------- #
  if (!is.null(x$conv)) {
    # 1. Extract raw method
    raw_method <- if (!is.null(x$opt_method)) x$opt_method else "Unknown"

    # 2. Format the string with package information
    method_str <- switch(raw_method,
      "BFGS"     = "BFGS (stats::optim)",
      "L-BFGS-B" = "L-BFGS-B (stats::optim)",
      "lbfgs"    = "L-BFGS (lbfgs::lbfgs)",
      raw_method
    ) # Fallback to raw name if no match

    # 3. Append bounds information if applicable
    if (raw_method == "L-BFGS-B" && isTRUE(x$opt_bounds)) {
      method_str <- paste0(method_str, " with box constraints")
    }

    # 4. Print the final diagnostic message
    if (x$conv == 0) {
      if (is.null(x$opt$counts)) {
        cat(sprintf(
          "\nOptimization: %s converged.\n",
          method_str
        ))
      } else {
        cat(sprintf(
          "\nOptimization: %s converged after %s objective function evaluations.\n",
          method_str, x$opt$counts[1]
        ))
      }
    } else {
      cat(sprintf(
        "\nWarning: Optimization algorithm [%s] did not converge (Code: %s).\n",
        method_str, x$conv
      ))
    }
  }

  # ---------------------------------------------------------------------- #
  # 5. Print Log-Likelihood and Information Criteria
  # ---------------------------------------------------------------------- #
  if (!is.null(x$loglik)) {
    cat("---\n")
    cat("Log-likelihood:", formatC(x$loglik, digits = 4), "\n")
    cat(
      "AIC:", formatC(x$aic, digits = 4),
      "  BIC:", formatC(x$bic, digits = 4),
      "  HQ:", formatC(x$hq, digits = 4), "\n"
    )
  }

  invisible(x)
}

#' Monthly relative humidity in Brasília
#'
#' @description
#' Monthly relative humidity in Brasília, Brazil, expressed as a proportion.
#' The data were obtained from the NASA POWER project and cover
#' January 1999 to June 2024.
#'
#' @format A \code{ts} object with monthly observations, frequency 12,
#'   starting in January 1999.
#'
#' @source NASA Prediction of Worldwide Energy Resources (POWER) project.
#'   \url{https://power.larc.nasa.gov/}
#'
#' @references
#' Cribari-Neto, F., Costa, E., & Fonseca, R. V. (2025). Numerical stability
#' enhancements in beta autoregressive moving average model estimation.
#' \emph{Brazilian Journal of Probability and Statistics}, 39(4), 410--437.
#' \doi{10.1214/25-BJPS645}
#'
#' @seealso \code{\link{brasilia_df}} for the same data as a data frame.
#'
#' @examples
#' data(brasilia_ts)
#' plot(brasilia_ts, ylab = "Relative humidity (proportion)", xlab = "Year")
"brasilia_ts"


#' Monthly relative humidity in Brasília (data frame)
#'
#' @description
#' The same series as \code{\link{brasilia_ts}}, stored as a data frame
#' with a \code{yearmon} time column.
#'
#' @format A data frame with 2 columns:
#' \describe{
#'   \item{time}{Month of observation (\code{yearmon}).}
#'   \item{y}{Relative humidity as a proportion (numeric, in (0, 1)).}
#' }
#'
#' @source NASA Prediction of Worldwide Energy Resources (POWER) project.
#'   \url{https://power.larc.nasa.gov/}
#'
#' @seealso \code{\link{brasilia_ts}} for the same data as a \code{ts} object.
#'
#' @examples
#' data(brasilia_df)
#' head(brasilia_df)
"brasilia_df"

#' Print Method for a barma Model
#'
#' @description
#' S3 method for printing a summary of a fitted `"barma"` model object.
#'
#' @details
#' This function provides a concise summary, showing the function call 
#' that generated the model, the link function used, and the final 
#' estimated coefficients.
#'
#' @param x A fitted model object of class `"barma"`.
#' @param ... Additional arguments (currently ignored).
#'
#' @return
#' Invisibly returns the original object `x`.
#'
#' @export
#' @method print barma
print.barma <- function(x, ...) {
  
  # Print the title
  cat("Beta Autoregressive Moving Average Model\n")
  
  # Print the call, handling line breaks
  if (!is.null(x$call)) {
    cat("\nCall:\n", paste(deparse(x$call), collapse = "\n"), "\n", 
        sep = "")
  }
  
  # Print the link function
  if (!is.null(x$link)) {
    cat("\nLink function:", x$link, "\n")
  }
  
  # Print the coefficients
  if (!is.null(x$coef)) {
    cat("\nCoefficients:\n")
    # This is the CRAN standard for printing a 1D named vector safely
    print.default(format(x$coef, digits = max(3, getOption("digits") - 3)), 
                  print.gap = 2, quote = FALSE)
  } else {
    cat("\nNo coefficients found.\n")
  }
  
  # Add a warning if convergence was not reached
  if (!is.null(x$conv) && x$conv != 0) {
    cat("\nWarning: Optimization algorithm did not converge.\n")
  }
  
  # Return the object invisibly, which is standard for print methods
  invisible(x)
}

#' Simulate a Beta Autoregressive Moving Average (BARMA) Time Series
#'
#' @description
#' Generates a random time series from a Beta Autoregressive Moving Average
#' (BARMA) model. The function can simulate BARMA(p,q), BAR(p), or BMA(q)
#' processes.
#'
#' @details
#' The model type is determined by the `varphi` and `theta` parameters:
#' \itemize{
#'   \item **BARMA(p,q):** Both `varphi` and `theta` are provided.
#'   \item **BAR(p):** Only `varphi` is provided (`theta` is `NA`).
#'   \item **BMA(q):** Only `theta` is provided (`varphi` is `NA`).
#' }
#'
#' @param n The desired length of the final time series (after burn-in).
#' @param varphi A numeric vector of autoregressive (AR) parameters (\eqn{\varphi}).
#'   Default is `NA` for models without an AR component.
#' @param theta A numeric vector of moving average (MA) parameters (\eqn{\theta}).
#'   Default is `NA` for models without an MA component.
#' @param alpha The intercept term (\eqn{\alpha}) in the linear predictor.
#'   Default is `0.0`.
#' @param phi The precision parameter (\eqn{\phi > 0}) of the Beta distribution.
#'   Higher values result in less variance. Default is `20`.
#' @param freq The frequency of the resulting `ts` object (e.g., 12 for monthly,
#'   4 for quarterly). Default is `12`.
#' @param link The link function to connect the mean \eqn{\mu_t} to the linear
#'   predictor. Must be one of `"logit"` (default), `"probit"`, `"cloglog"`, or
#'   `"loglog"`.
#'
#' @importFrom stats rbeta
#'
#' @return A `ts` object of length `n` containing the simulated Beta-distributed
#'   time series.
#'
#' @author
#' Original R code by: Fabio M. Bayer (Federal University of Santa Maria, <bayer@ufsm.br>)
#' Modified and improved by: Everton da Costa (Federal University of Pernambuco, <everto.cost@gmail.com>)
#'
#' @seealso \code{\link{barma}}, \code{\link{make_link_structure}}
#'
#' @examples
#' # Set seed for reproducibility
#' # --- Example 1: Simulate a BAR(1) process ---
#' # y_t depends on y_{t-1}
#' set.seed(2025)
#' bar1_series <- simu_barma(n = 250, alpha = 0.0, varphi = 0.5, phi = 20,
#' link = "logit")
#' plot(bar1_series, main = "Simulated BAR(1) Process", ylab = "Value")
#'
#' # --- Example 2: Simulate a BMA(1) process ---
#' # y_t depends on the previous error term
#' set.seed(2025)
#' bma1_series <- simu_barma(n = 250, alpha = 0.0, theta = -0.2, phi = 20)
#' plot(bma1_series, main = "Simulated BMA(1) Process", ylab = "Value")
#'
#' # --- Example 3: Simulate a BARMA(2,1) process with a cloglog link ---
#' set.seed(2025)
#' barma21_series <- simu_barma(
#'   n = 200,
#'   alpha = 0.0,
#'   varphi = c(0.4, 0.2), # AR(2) components
#'   theta = -0.3,         # MA(1) component
#'   phi = 20,
#'   link = "cloglog"
#' )
#' plot(barma21_series, main = "Simulated BARMA(2,1) Process", ylab = "Value")
#'
#' @export
simu_barma <- function(n,
                       alpha = 0.0,
                       varphi = NA, theta = NA,
                       phi = 20,
                       freq = 12, link = "logit") {


  ar <- NA
  ma <- NA

  if (any(is.na(varphi) == F)) ar <- 1:length(varphi)
  if (any(is.na(theta) == F)) ma <- 1:length(theta)

  # Link functions
  # The link function structure provides the necessary components
  # to perform the \beta ARMA model simulation with the specified link function.
  # Available link functions: "logit", "loglog", "cloglog"
  link_structure <- make_link_structure(link)

  linkfun <- link_structure$linkfun
  linkinv <- link_structure$linkinv
  mu.eta  <- link_structure$mu.eta


  # ===========================================================================
  # BARMA model
  # ===========================================================================
  if (any(is.na(varphi) == F) && any(is.na(theta) == F)) {
    # This section simulates a \beta ARMA(p ,q) process
    # where both AR and MA orders are specified

    # Compute the AR and MA orders
    p <- max(ar)
    q <- max(ma)
    m <- max(p, q)

    ynew <- rep(alpha, (n + m))
    mu <- linkinv(ynew)

    # E(error) = 0
    error <- rep(0, n + m)
    eta <- y <- rep(NA, n + m)

    for (i in (m + 1):(n + m)) {

      eta[i] <- alpha + varphi %*% ynew[i - ar] + theta %*% error[i - ma]
      mu[i] <- linkinv(eta[i])
      y[i] <- rbeta(1, mu[i] * phi, (1 - mu[i]) * phi)
      ynew[i] <- linkfun(y[i])
      error[i] <- ynew[i] - eta[i]

    }

    return(ts(y[(m + 1):(n + m)], frequency = freq))
  }


  # ===========================================================================
  # BAR model
  # ===========================================================================
  if (any(is.na(varphi) == F) && any(is.na(theta) == T)) {
    # This section simulates a \beta AR(p) process
    # where only the AR order is specified

    # Compute the AR order
    p <- max(ar)
    m <- p

    ynew <- rep(alpha, (n + m))
    mu <- linkinv(ynew)

    eta <- y <- rep(NA, n + m)

    for (i in (m + 1):(n + m)) {

      eta[i] <- alpha + varphi %*% ynew[i - ar]
      mu[i] <- linkinv(eta[i])
      y[i] <- rbeta(1, mu[i] * phi, (1 - mu[i]) * phi)
      ynew[i] <- linkfun(y[i])

    }

    return(ts(y[(m + 1):(n + m)], frequency = freq))
  }


  # ===========================================================================
  # BMA model
  # ===========================================================================
  if (any(is.na(varphi) == T) && any(is.na(theta) == F)) {
    # This section simulates a \beta MA(q) process
    # where only the MA order is specified

    # Compute the MA order
    q <- max(ma)
    m <- q

    ynew <- rep(alpha, (n + m))
    mu <- linkinv(ynew)

    # E(error)=0
    eta <- y <- error <- rep(0, n + m)

    for (i in (m + 1):(n + m)) {

      eta[i] <- alpha + theta %*% error[i - ma]
      mu[i] <- linkinv(eta[i])
      y[i] <- rbeta(1, mu[i] * phi, (1 - mu[i]) * phi)
      ynew[i] <- linkfun(y[i])
      error[i] <- ynew[i] - eta[i]

    }

    return(ts(y[(m + 1):(n + m)], frequency = freq))
  }


}

#' Summarize a barma Model Fit
#'
#' (Full documentation block...)
#'
#' @export
#' @method summary barma
#' @param object A fitted model object of class `"barma"`.
#' @param ... Additional arguments (currently ignored).
#'
#' @return
#' A list object of class `"summary.barma"`...
summary.barma <- function(object, ...) {
  
  # ------------------------------------------------------------------------- #
  # --- 1. Calculate vcov and model table ---
  # ------------------------------------------------------------------------- #
  
  # Inverse of Fisher information matrix
  vcov <- try(solve(object$fisher_info_mat, tol = 1e-20), silent = TRUE)
  
  # Check if the inverse matrix is possible
  if (inherits(vcov, "try-error")) {
    
    warning("FISHER'S INFORMATION MATRIX IS NOT INVERTIBLE! ")
    
    # Create a coefficient table with NAs for std. errors
    n_params <- length(object$coef)
    model_table <- cbind(
      Estimate = round(object$coef, 4),
      `Std. Error` = rep(NA_real_, n_params),
      `z value` = rep(NA_real_, n_params),
      `Pr(>|z|)` = rep(NA_real_, n_params)
    )
    
  } else {
    
    # Calculate Std. Errors and p-values
    stderror <- sqrt(diag(vcov))
    z_zstat <- object$coef / stderror 
    z_pvalues <- 2 * (1 - pnorm(abs(z_zstat)))
    
    model_table <- cbind(
      "Estimate" = object$coef,
      "Std. Error" = stderror,
      "z value" = z_zstat,
      "Pr(>|z|)" = z_pvalues
    )
  }
  
  # ------------------------------------------------------------------------- #
  # --- 2. Calculate Information Criteria ---
  # ------------------------------------------------------------------------- #
  aux_info1 <- -2 * object$loglik
  aux_info2 <- object$n_params 
  
  # Use effective number of observations
  # log_n <- log(object$n_obs - object$max_lag) 
  
  # Use the number of observations
  log_n <- log(object$n_obs)
  
  aic <- aux_info1 + 2 * aux_info2
  bic <- aux_info1 + log_n * aux_info2
  hq <- aux_info1 + log(log_n) * 2 * aux_info2
  
  # ------------------------------------------------------------------------- #
  # --- 3. Assemble the summary object ---
  # ------------------------------------------------------------------------- #
  summary_list <- list(
    call = object$call,
    link = object$link,
    coefficients = model_table,
    vcov = vcov,
    loglik = object$loglik,
    aic = aic,
    bic = bic,
    hq = hq,
    conv = object$conv,
    opt = object$opt,
    opt_method = object$opt_method,
    opt_bounds = object$opt_bounds,
    penalty = object$penalty,
    lambda = object$lambda
  )
  
  # Assign the new class
  class(summary_list) <- "summary.barma"
  
  return(summary_list)
}

#' Build parameter index list for BARMA models
#'
#' Constructs the named list of positional indices mapping each parameter
#' group to its position in the flat parameter vector. Called by both
#' `barma()` and `run_optimizer()` to guarantee a single source of truth
#' for the parameter order.
#'
#' Parameter order:
#'   1. alpha   — intercept
#'   2. varphi  — AR coefficients (if present)
#'   3. theta   — MA coefficients (if present)
#'   4. beta    — regression coefficients (if present)
#'   5. phi     — precision parameter
#'
#' @param n_ar_params   Integer. Number of AR parameters.
#' @param n_ma_params   Integer. Number of MA parameters.
#' @param n_beta_params Integer. Number of regression parameters.
#'
#' @return A named list with components:
#'   \describe{
#'     \item{`idx_alpha`}{Integer scalar. Index of the intercept.}
#'     \item{`idx_varphi`}{Integer vector. Indices of AR coefficients,
#'       or `integer(0)` if absent.}
#'     \item{`idx_theta`}{Integer vector. Indices of MA coefficients,
#'       or `integer(0)` if absent.}
#'     \item{`idx_beta`}{Integer vector. Indices of regression coefficients,
#'       or `integer(0)` if absent.}
#'     \item{`idx_phi`}{Integer scalar. Index of the precision parameter.}
#'     \item{`n_params`}{Integer scalar. Total number of parameters.}
#'   }
#' @keywords internal
#' @noRd
make_par_indices <- function(
    n_ar_params, 
    n_ma_params, 
    n_beta_params
) {
  
  idx_alpha  <- 1L
  last_idx   <- 1L
  
  idx_varphi <- seq_len(n_ar_params)   + last_idx
  last_idx   <- last_idx + n_ar_params
  
  idx_theta  <- seq_len(n_ma_params)   + last_idx
  last_idx   <- last_idx + n_ma_params
  
  idx_beta   <- seq_len(n_beta_params) + last_idx
  last_idx   <- last_idx + n_beta_params
  
  idx_phi    <- last_idx + 1L
  n_params   <- idx_phi
  
  list(
    idx_alpha  = idx_alpha,
    idx_varphi = idx_varphi,
    idx_theta  = idx_theta,
    idx_beta   = idx_beta,
    idx_phi    = idx_phi,
    n_params   = n_params
  )
}

# -------------------------------------------------------------------------- #
# OPTIMIZATION (CMLE / PCMLE) — internal helper
# Not exported. Called by barma().
# -------------------------------------------------------------------------- #

#' Run optimization for BARMA models (CMLE / PCMLE)
#'
#' Internal function that wraps `stats::optim` (BFGS or L-BFGS-B) and
#' `lbfgs::lbfgs` into a unified interface for standard Conditional Maximum 
#' Likelihood Estimation (CMLE) or Penalized CMLE (PCMLE) of BARMA/BARMAX models.
#'
#' @param y Numeric vector of observations on the unit interval (0, 1).
#' @param init_pars Numeric vector of initial parameter values. Order must
#'   match: `alpha`, `varphi` (AR), `theta` (MA), `beta` (regression), `phi`.
#' @param ar_lags Integer vector of AR lag indices, or `integer(0)` if absent.
#' @param ma_lags Integer vector of MA lag indices, or `integer(0)` if absent.
#' @param xreg Numeric matrix of exogenous regressors, or `NULL` if absent.
#' @param link Character string specifying the link function (e.g. `"logit"`).
#' @param par_idx Named list of parameter indices as returned by 
#'   \code{make_par_indices()}.
#' @param optimization A named list controlling the optimizer. Recognized
#'   fields:
#'   \describe{
#'     \item{`method`}{Character. One of `"BFGS"` (default), `"L-BFGS-B"`,
#'       or `"lbfgs"`.}
#'     \item{`lower`}{Numeric vector or scalar. Lower bounds for `"L-BFGS-B"`.
#'       Default `-Inf`.}
#'     \item{`upper`}{Numeric vector or scalar. Upper bounds for `"L-BFGS-B"`.
#'       Default `Inf`.}
#'   }
#' @param penalty Logical. If \code{TRUE}, applies ridge penalization (PCMLE). 
#'   If \code{FALSE}, performs standard CMLE.
#'
#' @return A named list with components:
#'   \describe{
#'     \item{`opt`}{A list containing the estimated `par`, maximized `loglik`, 
#'       `counts` of function evaluations, the `convergence` code, and any 
#'       diagnostic `message`.}
#'     \item{`raw`}{The unmodified object returned by the underlying optimizer.}
#'   }
#'
#' @seealso [loglik_barma()], [score_vector_barma()], [barma()]
#' @keywords internal
#' @noRd
run_optimizer <- function(
    y,
    init_pars,
    ar_lags,
    ma_lags,
    xreg,
    link,
    par_idx,
    optimization,
    penalty
) {
  
  # 1. Setup =================================================================
  method <- optimization$method
  
  # Fallback to -Inf/Inf if the user omitted them from the list
  lower <- if (!is.null(optimization$lower)) optimization$lower else -Inf
  upper <- if (!is.null(optimization$upper)) optimization$upper else Inf
  
  has_xreg <- !is.null(xreg)
  
  n_ar_params   <- length(ar_lags)
  n_ma_params   <- length(ma_lags)
  n_beta_params <- if (has_xreg) ncol(xreg) else 0L
  
  # 2. Parameter indices =====================================================
  
  # Parameter order (matching init_pars order):
  #   1. alpha   — intercept
  #   2. varphi  — AR coefficients (if present)
  #   3. theta   — MA coefficients (if present)
  #   4. beta    — regression coefficients (if present)
  #   5. phi     — precision parameter
  
  idx_alpha  <- par_idx$idx_alpha
  idx_varphi <- par_idx$idx_varphi
  idx_theta  <- par_idx$idx_theta
  idx_beta   <- par_idx$idx_beta
  idx_phi    <- par_idx$idx_phi
  n_params   <- par_idx$n_params
  
  if (length(init_pars) != n_params)
    stop(
      "'init_pars' length (", length(init_pars), ") does not match ",
      "the expected number of parameters (", n_params, ")."
    )
  
  # 3. Helper closures =======================================================
  
  call_barma <- function(fn, x) {
    fn(
      y      = y,
      ar     = ar_lags,
      ma     = ma_lags,
      alpha  = x[idx_alpha],
      varphi = x[idx_varphi],
      theta  = x[idx_theta],
      phi    = x[idx_phi],
      link   = link,
      xreg   = xreg,
      beta   = x[idx_beta],
      penalty = penalty
    )
  }
  
  neg_loglik       <- function(x) -call_barma(loglik_barma,       x)
  neg_score_vector <- function(x) -call_barma(score_vector_barma, x)
  
  # 4. Execute Optimization ================================================
  
  opt_final <- tryCatch({
    
    if (method == "BFGS") {
      
      opt_aux <- stats::optim(
        par    = init_pars,
        fn     = neg_loglik,
        gr     = neg_score_vector,
        method = "BFGS"
      )
      
    } else if (method == "L-BFGS-B") {
      
      opt_aux <- stats::optim(
        par    = init_pars,
        fn     = neg_loglik,
        gr     = neg_score_vector,
        method = "L-BFGS-B",
        lower  = lower,
        upper  = upper
      )
      
    } else if (method == "lbfgs") {
      
      opt_aux <- lbfgs::lbfgs(
        vars      = init_pars,
        call_eval = neg_loglik,
        call_grad = neg_score_vector,
        invisible = 1
      )
      
    } else {
      
      stop("Unrecognized optimization method: '", method, 
           "'. Use 'BFGS', 'L-BFGS-B', or 'lbfgs'.")
      
    }
  
  # 5. Output ==============================================================
  
  beta_names <- if (has_xreg) {
    if (!is.null(colnames(xreg))) colnames(xreg) else 
      paste0("beta", seq_len(n_beta_params))
  } else {
    character(0)
  }
  par_names <- c(
    "alpha",
    if (n_ar_params   > 0) paste0("varphi", ar_lags),
    if (n_ma_params   > 0) paste0("theta",  ma_lags),
    beta_names,
    "phi"
  )
  
  counts <- if (method %in% c("BFGS", "L-BFGS-B")) {
    opt_aux$counts
  } else {
    NULL
  }
  
  opt_aux_par <- opt_aux$par
  names(opt_aux_par) <- par_names
  
  output <- list(
    
    opt = list(
      par         = opt_aux_par,
      loglik      = -opt_aux$value,
      convergence = opt_aux$convergence,
      message     = opt_aux$message,
      counts      = counts
    ),
    
    opt_raw = opt_aux
    
  )
  
  # Error message ==========================================================
  
  }, error = function(e) {
    stop("Optimization failed (method = '", method, "'): ", conditionMessage(e))
  })
  
  return(output)
  
}

---
title: "Application: Modeling Relative Humidity in Brasília"
author: "Everton da Costa, Francisco Cribari-Neto, and Rodney V. Fonseca"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
    number_sections: true
    fig_caption: true
bibliography: ../inst/REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{Application: Modeling Relative Humidity in Brasília}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  fig.width = 7,
  fig.height = 3.5,
  fig.align = "center",
  warning   = FALSE,
  message   = FALSE
)
```

> **Note:** This vignette requires suggested packages. Install them with:
> `install.packages(c("ggplot2", "gridExtra", "zoo", "forecast", "scales", "moments"))`

# Introduction

This vignette illustrates a complete $\beta$ARMA modeling workflow applied to
monthly relative humidity data from Brasília, Brazil, using the `betaARMA`
package. The analysis has two complementary objectives.

The first objective concerns **numerical stability**. Conditional maximum
likelihood estimation (CMLE) of $\beta\text{ARMA}$ models can fail in
practice: Cribari-Neto, Costa, and Fonseca (2025) show, via Monte Carlo
simulation, that convergence failure rates can reach 18.68% in challenging
scenarios (e.g., $\beta\text{ARMA}(1,4)$ with $n = 125$).
Their paper also
demonstrates, using two empirical datasets, that certain model specifications
only converge when ridge penalization is applied. A natural question then
arises: once the penalized estimator (PCMLE) achieves convergence, how
reliable is the resulting model? Does it produce a well-specified fit that
passes residual diagnostics? This vignette addresses that question directly
by presenting a specification for which CMLE fails and PCMLE succeeds, and
by carrying out a full residual analysis on the penalized model.

The second objective is to present a **full modeling workflow** — from
exploratory analysis through feature engineering, model fitting, residual
diagnostics, and out-of-sample forecasting — applied to the complete Brasília
humidity series. This mirrors the applied methodology of Costa, Cribari-Neto,
and Scher (2024), published in the *Journal of Hydrology*.

Replication code and data are openly available at
<https://github.com/Everton-da-Costa/BarmaRidgeBJPS2025>.


# Data and Exploratory Analysis

```{r libraries}
library(betaARMA)
library(forecast)
library(ggplot2)
library(gridExtra)
library(zoo)
library(scales)
library(moments)
```

```{r config_plot, echo=FALSE}
# Shared ggplot configuration
sample_size     <- length(brasilia_ts)
end_time_series <- time(brasilia_ts)[sample_size] + 1

ggplot_size_font <- 9

ggplot_theme <- theme_minimal() + 
  theme(
  legend.position = "bottom",
  title        = element_text(size = ggplot_size_font),
  axis.text    = element_text(size = ggplot_size_font),
  axis.title   = element_text(size = ggplot_size_font),
  legend.text  = element_text(size = ggplot_size_font),
  legend.title = element_text(size = ggplot_size_font)
)

ggplot_scale_y <- scale_y_continuous(
  breaks = seq(0.0, 1.00, 0.10)
)

ggplot_scale_x <- scale_x_continuous(
  breaks = seq(1999, end_time_series, 2),
  limits = c(1999, end_time_series)
)
```

The dataset consists of 306 monthly observations of average relative humidity
in Brasília, Brazil, covering January 1999 to June 2024. The data were
obtained from NASA's POWER project. Relative humidity is naturally bounded
in $(0, 1)$, making it well suited for $\beta$ARMA modeling.


# A Case of Numerical Instability

To motivate the penalized estimator, we focus on a specific subsample of the
Brasília dataset. In Cribari-Neto, Costa, and Fonseca (2025), this subsample was
used to illustrate convergence failure under standard CMLE, and it was shown
that ridge penalization successfully restores convergence.

While that paper established the numerical stability of the penalized
estimator, the objective here is different: to verify the statistical adequacy
of the penalized fit through residual diagnostics, and to show how this workflow
can guide model reduction. To that end, we specify a $\beta\text{ARMA}$
model with $\text{AR} = (10, 18)$, $\text{MA} = (1, 11, 13)$,
and harmonic regressors.

## Train--Test Split

```{r subsample}
y_subsample_ts <- window(
  brasilia_ts,
  start = c(2006, 02),
  end   = c(2019, 07)
)

y_train <- window(
  y_subsample_ts,
  end = c(2018, 07)
)

y_test <- window(
  y_subsample_ts,
  start = c(2018, 08)
)
```

The figure below shows the full time series with the training period
delimited by dashed vertical lines.

```{r ggplot_subsample, echo=FALSE}
len_y_ts        <- length(y_train)
start_subsample <- time(y_train)[1]
end_subsample   <- time(y_train)[len_y_ts]

ggplot_subsample <- ggplot(brasilia_df, aes(time, y)) +
  geom_point(size = 0.5) +
  geom_line(aes(group = 1)) +
  geom_vline(xintercept = start_subsample,
             linetype = "dashed", linewidth = 0.5, color = "red") +
  geom_vline(xintercept = end_subsample,
             linetype = "dashed", linewidth = 0.5, color = "red") +
  labs(x = "Year", y = "Relative humidity (proportion)") +
  ggplot_scale_x +
  ggplot_scale_y +
  ggplot_theme
```

```{r print_ggplot_subsample, echo=FALSE, fig.cap="Relative humidity in Brasília. Dashed red lines delimit the training period used in the instability analysis."}
print(ggplot_subsample)
```

## Descriptive Statistics

```{r descriptive_subsample, echo=FALSE}
descriptive_sub_df <- data.frame(
  Min             = min(y_subsample_ts),
  Max             = max(y_subsample_ts),
  Median          = median(y_subsample_ts),
  Mean            = mean(y_subsample_ts),
  SD              = sd(y_subsample_ts),
  Skewness        = moments::skewness(y_subsample_ts),
  Excess_kurtosis = moments::kurtosis(y_subsample_ts)
)
descriptive_sub_df <- round(descriptive_sub_df, 2)
```

```{r print_descriptive_subsample, echo=FALSE}
knitr::kable(
  descriptive_sub_df,
  caption = "Descriptive statistics of the monthly relative humidity in
  Brasília — subsample (February 2006 to July 2019)."
)
```

## Feature Engineering

To capture annual seasonality we include two harmonic regressors,
$s_t = \sin(2\pi t / 12)$ and $c_t = \cos(2\pi t / 12)$, which together
represent a smooth annual wave.

```{r regressors}
freq              <- frequency(y_train)
n_test            <- length(y_test)
trend_index_train <- seq_along(y_train)
trend_index_test  <- (max(trend_index_train) + 1):
                     (max(trend_index_train) + n_test)

hs_train <- sin(2 * pi * trend_index_train / freq)
hc_train <- cos(2 * pi * trend_index_train / freq)
hs_test  <- sin(2 * pi * trend_index_test  / freq)
hc_test  <- cos(2 * pi * trend_index_test  / freq)

X_train <- cbind(hs = hs_train, hc = hc_train)
X_test  <- cbind(hs = hs_test,  hc = hc_test)
```

## Standard CMLE

```{r fit_cmle}
ar_lags <- c(10, 18)
ma_lags <- c(1, 11, 13)

fit_cmle <- barma(
  y       = y_train,
  ar      = ar_lags,
  ma      = ma_lags,
  penalty = FALSE,
  xreg    = X_train
)
```

```{r summary_cmle}
summary(fit_cmle)
```

The optimizer failed to converge (`conv = 1`), and the estimates are
unreliable. Without a penalized alternative, an analyst would have little 
recourse but to abandon this lag structure and search for a different 
specification.

## Penalized CMLE (PCMLE)

Applying ridge penalization to the same specification restores convergence.

```{r fit_pcmle}
fit_pcmle <- barma(
  y       = y_train,
  ar      = ar_lags,
  ma      = ma_lags,
  penalty = TRUE,
  xreg    = X_train
)
```

```{r summary_pcmle}
summary(fit_pcmle)
```

The penalized estimator successfully converges. The ridge penalty value
$\lambda_n = 1/(n - a)^{0.9}$ is chosen to balance bias and variance, as
established in Cribari-Neto, Costa, and Fonseca (2025).

This illustrates a practical use case for PCMLE. When standard CMLE presents
a convergence failure, an analyst might discard the entire lag structure
without further investigation. By applying PCMLE, convergence is achieved
and the parameter estimates become available for inspection. 
Furthermore,
the diagnostic plots below for the PCMLE fit confirm that the residuals exhibit 
no serial correlation (Scher, Cribari-Neto, Pumi, and Bayer, 2020),
indicating that the lag structure adequately captures 
the dynamics of the series.

```{r diagnostics_pcmle, fig.height=7.0, fig.cap="Diagnostic plots for the PCMLE fit: AR=(10,18), MA=(1,11,13)."}
plot(fit_pcmle, which = "all")
```

The observed and fitted values track each other closely. All ACF and PACF
spikes lie within the confidence bands, and both the Ljung-Box and Monti
p-values remain consistently above 0.05 across all evaluated lags, providing
no evidence of residual serial correlation.

Upon reviewing the PCMLE summary, we note that the $\theta_{11}$ estimate
is not statistically significant (p-value = 0.488). This suggests that the
MA(11) lag does not contribute meaningfully to the model dynamics. Removing
it yields the reduced specification $\text{AR} = (10, 18)$,
$\text{MA} = (1, 13)$, for which the likelihood surface is no longer flat
and standard CMLE converges. Without PCMLE, the non-significance of
$\theta_{11}$ would never have been discovered, and the valid underlying
structure of the series would have remained inaccessible.

## Reduced Model

```{r fit_cmle_reduced}
ar_lags_reduced <- c(10, 18)
ma_lags_reduced <- c(1, 13)

fit_cmle_reduced <- barma(
  y       = y_train,
  ar      = ar_lags_reduced,
  ma      = ma_lags_reduced,
  penalty = FALSE,
  xreg    = X_train
)
```

```{r summary_cmle_reduced}
summary(fit_cmle_reduced)
```

The reduced CMLE model converges and all coefficients are statistically
significant. We proceed with residual diagnostics and forecasting using
this parsimonious specification.

## Residual Diagnostics

The panels below assess whether the fitted model adequately captures the
dynamics of the Brasília relative humidity series.

```{r diagnostics_all, fig.height=7.0, fig.cap="Diagnostic plots for the reduced CMLE model AR=(10,18), MA=(1,13)."}
plot(fit_cmle_reduced, which = "all")
```

The observed and fitted values track each other closely throughout the
sample period. All ACF and PACF spikes lie within the confidence bands,
and both the Ljung-Box and Monti p-values remain consistently above 0.05
across all evaluated lags, providing no evidence of residual serial
correlation.

```{r diagnostics_qq, fig.height=4.0, fig.width=4.5, fig.cap="Normal Q-Q plot of quantile residuals for the reduced CMLE model."}
plot(fit_cmle_reduced, which = "qq")
```

The quantile residuals fall closely along the reference line within the
95% confidence band, supporting the normality assumption. One observation
in the upper tail departs slightly from the line but does not indicate
a systematic departure from the assumed distribution.

The largest residual corresponds to October 2011 (quantile residual = 4.37), 
a month that marks the climatological transition from the dry to the rainy 
season in Brasília — the period of highest interannual humidity variability. 
This observation coincides with the onset of an extreme drought period 
documented across Brazil from 2011 onward (Cunha et al., 2019). The second 
largest residual corresponds to January 2015 (−3.44), coinciding with the 
strong 2015–2016 El Niño event, which suppressed rainfall across central Brazil.
These isolated departures reflect genuine meteorological anomalies rather than
systematic model misspecification.

# Forecasting

```{r forecast_setup}
# Define the translation dictionary
month_map <- c(
  "Jan" = "Jan", "Fev" = "Feb", "Mar" = "Mar", "Abr" = "Apr",
  "Mai" = "May", "Jun" = "Jun", "Jul" = "Jul", "Ago" = "Aug",
  "Set" = "Sep", "Out" = "Oct", "Nov" = "Nov", "Dez" = "Dec"
)

dates     <- as.Date(zoo::as.yearmon(time(y_test)))
pt_months <- tools::toTitleCase(format(dates, "%b"))
months    <- unname(month_map[pt_months])

forecasts_df <- data.frame(
  observed = as.numeric(y_test),
  forecast = forecast(fit_cmle_reduced, xreg = X_test, h = n_test)$mean,
  month    = months
)
forecasts_df$time <- seq_len(n_test)
```

```{r ggplot_forecast, echo=FALSE}
ggplot_forecast <- ggplot(forecasts_df, aes(x = time)) +
  geom_line(aes(y = forecast, colour = "Forecast"), linewidth = 0.8) +
  geom_point(aes(y = observed, shape = "Observed"),
             color = "#00BFC4", size = 3) +
  scale_colour_manual(name   = " ",
                      values = c("Forecast" = "#C77CFF")) +
  scale_shape_manual(name   = " ",
                     values = c("Observed" = 16)) +
  scale_x_continuous(breaks = seq_len(n_test),
                     limits = c(1, n_test),
                     labels = forecasts_df$month) +
  scale_y_continuous(breaks = seq(0.3, 1.0, 0.1),
                     limits = c(0.3, 1.0),
                     labels = number_format(accuracy = 0.1)) +
  labs(x = "Time (months)", y = "Relative humidity (proportion)") +
  ggplot_theme
```

```{r print_forecast, echo=FALSE, fig.cap="Out-of-sample forecasts from the reduced CMLE model against the observed test data (teal dots)."}
print(ggplot_forecast)
```

```{r forecast_accuracy}
mae  <- function(obs, pred) mean(abs(obs - pred))
rmse <- function(obs, pred) sqrt(mean((obs - pred)^2))

obs <- forecasts_df$observed

accuracy_table <- data.frame(
  Model = "betaARMA CMLE (reduced)",
  MAE   = round(mae(obs,  forecasts_df$forecast) * 100, 2),
  RMSE  = round(rmse(obs, forecasts_df$forecast) * 100, 2)
)

knitr::kable(
  accuracy_table,
  caption = paste0(
    "MAE and RMSE ($\\times 100$) over the ",
    n_test,
    "-month forecast horizon."
  )
)
```


# Conclusion

This vignette illustrated the practical value of PCMLE as a diagnostic tool
in $\beta$ARMA modeling, using monthly relative humidity data from Brasília.

Standard CMLE failed to converge for the $\beta\text{ARMA}$ model with
$\text{AR} = (10, 18)$, $\text{MA} = (1, 11, 13)$. Applying ridge penalization (PCMLE) restored convergence and
made the parameter estimates available for inspection. Residual diagnostics
on the PCMLE fit confirmed that the lag structure adequately captured the
series dynamics. Inspection of the PCMLE estimates revealed that
$\theta_{11}$ was not statistically significant, motivating the reduction to the parsimonious $\beta\text{ARMA}$ model with
$\text{AR} = (10, 18)$, $\text{MA} = (1, 13)$. The reduced
CMLE model converged, passed all residual diagnostics, and produced accurate
out-of-sample forecasts, with all predictions guaranteed to lie in $(0, 1)$.

This workflow — CMLE fails, PCMLE reveals the structure, reduction yields a
stable CMLE model — demonstrates that PCMLE is not merely a fallback
estimator but an active analytical tool that can guide model selection in
challenging estimation scenarios. For the complete simulation study,
bootstrap procedures, and extended empirical results, we refer the reader to
Cribari-Neto, Costa, and Fonseca (2025).


# References

Costa, E., Cribari-Neto, F., & Scher, V. T. (2024). Test inferences and link
function selection in dynamic beta modeling of seasonal hydro-environmental
time series with temporary abnormal regimes. *Journal of Hydrology*, 638,
131489. <https://doi.org/10.1016/j.jhydrol.2024.131489>

Cribari-Neto, F., Costa, E., & Fonseca, R. V. (2025). Numerical stability
enhancements in beta autoregressive moving average model estimation.
*Brazilian Journal of Probability and Statistics*, 39(4), 410--437.
<https://doi.org/10.1214/25-BJPS645>

Cunha, A. P. M. A., Zeri, M., Deusdará Leal, K., Costa, L., Cuartas, L. A., 
Marengo, J. A., ... & Cunningham, C. (2019). Extreme drought events over 
Brazil from 2011 to 2019. *Atmosphere*, 10(11), 642. 
<https://doi.org/10.3390/atmos10110642>

Rocha, A. V., & Cribari-Neto, F. (2009). Beta autoregressive moving average
models. *TEST*, 18(3), 529--545.
<https://doi.org/10.1007/s11749-008-0112-z>

Rocha, A. V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
moving average models. *TEST*, 26, 451--459.
<https://doi.org/10.1007/s11749-017-0528-4>

Scher, V. T., Cribari-Neto, F., Pumi, G., & Bayer, F. M. (2020).
Goodness-of-fit tests for $\beta$ARMA hydrological time series modeling.
*Environmetrics*, 31(3), e2607.
<https://doi.org/10.1002/env.2607>



# Reproducibility

The analysis was conducted in the following computational environment.

```{r session_info, echo=FALSE}
cat("=================================================================\n")
cat("Session Information\n")
cat("=================================================================\n")
cat("This report was generated at:",
    format(Sys.time(), "%B %d, %Y at %I:%M %p"), "\n\n")
print(sessionInfo())
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/start_values.R
\name{start_values}
\alias{start_values}
\title{Generate Initial Values for BARMA Model Estimation}
\usage{
start_values(y, link, ar = integer(0), ma = integer(0), xreg = NA)
}
\arguments{
\item{y}{A numeric time series with values in the open interval (0, 1).}

\item{link}{A string specifying the link function for the mean.}

\item{ar}{A numeric vector of autoregressive (AR) lags. Defaults to
\code{integer(0)}, which omits the AR component entirely.}

\item{ma}{A numeric vector of moving average (MA) lags. Defaults to
\code{integer(0)}, which omits the MA component entirely.}

\item{xreg}{An optional numeric matrix or data frame of exogenous variables.}
}
\value{
A named numeric vector containing the initial values for the model
parameters (`alpha`, `varphi`, `theta`, `beta`, `phi`), ready to be
used by an optimization routine.
}
\description{
This function calculates reasonable starting values for the parameters of
various Beta Autoregressive Moving Average (BARMA) models. The method is
based on the approach proposed by Ferrari & Cribari-Neto (2004) for
beta regression, adapted here for the time series context.
}
\details{
The function computes initial values by fitting a linear model
(`lm.fit`) to the link-transformed response variable `g(y)`.
This provides a computationally cheap and stable way to initialize the
main optimization algorithm.
}
\author{
Original code by Fabio M. Bayer (bayer@ufsm.br).
Substantially modified and improved by Everton da Costa
(everto.cost@gmail.com).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barma.R
\name{barma}
\alias{barma}
\title{Fit Beta Autoregressive Moving Average (BARMA) Models via
Conditional Maximum Likelihood}
\usage{
barma(
  y,
  ar = integer(0),
  ma = integer(0),
  link = "logit",
  xreg = NULL,
  optimization = list(method = "BFGS", lower = -Inf, upper = Inf),
  penalty = FALSE
)
}
\arguments{
\item{y}{A time series object (`ts`) with values strictly in the open interval (0, 1).
Must have at least \eqn{\max(p,q) + 1} observations.}

\item{ar}{A numeric vector specifying autoregressive (AR) lags
(e.g., `c(1, 2)` for AR(2)). Defaults to `integer(0)`, which omits the
AR component entirely. Absence should be expressed by omitting this
argument or passing `integer(0)`.}

\item{ma}{A numeric vector specifying moving average (MA) lags
(e.g., `1` for MA(1)). Defaults to `integer(0)`, which omits the
MA component entirely. Absence should be expressed by omitting this
argument or passing `integer(0)`.}

\item{link}{The link function connecting the mean \eqn{\mu_t} to the linear predictor
\eqn{\eta_t}. One of:
\itemize{
  \item `"logit"` (default): \eqn{g(x) = \log(x/(1-x))}
  \item `"probit"`: \eqn{g(x) = \Phi^{-1}(x)}
  \item `"cloglog"`: Complementary log-log link
  \item `"loglog"`: Log-log link
}}

\item{xreg}{A matrix or data frame of external regressors (covariates), optional.
Must have the same number of rows as `y`. If provided, its columns
are included in the linear predictor with associated coefficients in `beta`.}

\item{optimization}{A named list controlling the optimization procedure. Recognized fields:
\describe{
  \item{\code{method}}{Character string specifying the algorithm. One of
    \code{"BFGS"} (default), \code{"L-BFGS-B"}, or \code{"lbfgs"}.}
  \item{\code{lower}}{Numeric vector or scalar of lower bounds, used only
    by \code{"L-BFGS-B"}. Defaults to \code{-Inf}.}
  \item{\code{upper}}{Numeric vector or scalar of upper bounds, used only
    by \code{"L-BFGS-B"}. Defaults to \code{Inf}.}
}}

\item{penalty}{Logical. If \code{TRUE}, the ridge penalization scheme of Cribari-Neto,
Costa and Fonseca (2025) is applied during conditional maximum likelihood
estimation. The penalty term \eqn{(n - a)\lambda_n \|\boldsymbol{\nu}\|^2}
is subtracted from the conditional log-likelihood, where
\eqn{\lambda_n = 1/(n - a)^{0.9}} and \eqn{\boldsymbol{\nu}} collects
\eqn{\alpha}, \eqn{\boldsymbol{\varphi}}, and \eqn{\boldsymbol{\theta}}.
The precision parameter \eqn{\phi} and regression coefficients
\eqn{\boldsymbol{\beta}} are excluded from penalization.
Ridge penalization enhances the curvature of the conditional log-likelihood
surface, reducing convergence failures and the occurrence of implausible
estimates. Defaults to \code{FALSE}.}
}
\value{
An object of class `"barma"` containing:
\item{coef}{Named vector of all estimated parameters, ordered as:
  alpha, AR parameters, MA parameters, beta parameters, phi.}
\item{vcov}{The variance-covariance matrix of the estimators, computed as
  the inverse of the observed Fisher Information Matrix.}
\item{model}{A summary table with coefficients, standard errors, z-statistics,
  and p-values for hypothesis tests \eqn{H_0: \theta_i = 0}.}
\item{fitted}{Fitted conditional mean values as a `ts` object
  (NA-padded for the first \eqn{m = \max(p,q)} observations).}
\item{muhat}{Alias for `fitted` (fitted mean values).}
\item{etahat}{Estimated linear predictor values (full vector, NA-padded).}
\item{errorhat}{Estimated errors on predictor scale (full vector, 0-padded).}
\item{loglik}{The conditional log-likelihood at the CMLE.}
\item{fisher_info_mat}{The expected Fisher Information Matrix
(Rocha & Cribari-Neto, 2009).}
\item{conv}{Convergence code from `optim` (0 = success).}
\item{alpha, beta, varphi, theta, phi}{Individual parameter estimates.}
\item{start_values}{Initial parameter values used in optimization.}
\item{call}{The original function call.}
\item{opt}{Cleaned output list from the optimization procedure.}
\item{opt_raw}{Raw output object returned directly by `optim()` or `lbfgs()`.}
}
\description{
Fits a Beta Autoregressive Moving Average (BARMA) model to time series data
valued in (0, 1) using Conditional Maximum Likelihood Estimation (CMLE).
The function performs complete model estimation including parameter
estimation, hypothesis testing infrastructure, and model diagnostics.
}
\details{
Fit Beta Autoregressive Moving Average (BARMA) Models


This function fits the BARMA(p,q) model as proposed by Rocha & Cribari-Neto
(2009, with erratum 2017). It serves as the main wrapper for the optimization
process, calling specialized helper functions for likelihood computation,
gradient calculation, and Fisher Information Matrix estimation.

**Model Specification**: The BARMA model is defined as:

\deqn{g(\mu_t) = \alpha + X_t\beta + \sum_{i=1}^{p} \varphi_i
(g(y_{t-i}) - X_{t-i}\beta) + \sum_{j=1}^{q} \theta_j \epsilon_{t-j}}

where \eqn{y_t | F_{t-1} \sim Beta(\mu_t\phi, (1-\mu_t)\phi)},
\eqn{g(\cdot)} is the link function, and \eqn{F_{t-1}} is the information
set at time t-1.

**Model Types** (specified via `ar` and `ma` arguments):
\itemize{
  \item **BARMA(p,q):** Both `ar` and `ma` are specified.
  \item **BAR(p):** Only `ar` is specified; omit `ma` or pass `integer(0)`.
  \item **BMA(q):** Only `ma` is specified; omit `ar` or pass `integer(0)`.
}

**External Regressors**: Covariates can be included via `xreg`. The model
becomes a regression BARMA, where the mean depends on current covariates
and lagged responses/errors.

**Optimization**: The function uses quasi-Newton algorithms
(BFGS or L-BFGS-B) via \code{\link[stats]{optim}}, or the
\code{\link[lbfgs]{lbfgs}} function, utilizing analytic gradients for
efficiency. Initial values are obtained from the internal `start_values()`
function.

**Implementation Notes**:
- The conditional log-likelihood is computed by conditioning on the first
  \eqn{m = \max(p,q)} observations, which are required to initialize the
  recursive structure of the model.
- The 2017 Erratum corrections are implemented for correct handling of
  moving average components in the score vector and Fisher Information Matrix.
- All computations are vectorized where possible for efficiency.
}
\note{
The original version of this function was developed by Fabio M. Bayer
(Federal University of Santa Maria, \email{bayer@ufsm.br}). It has been
substantially modified and improved by Everton da Costa, with suggestions
and contributions from Francisco Cribari-Neto.
}
\examples{
\donttest{
# Example 1: Fit a BAR(1) model (no MA component)
set.seed(2025)
y_sim_bar1 <- simu_barma(
  n      = 250,
  alpha  = 0.0,
  varphi = 0.6,
  phi    = 25.0,
  link   = "logit",
  freq   = 12
)

fit_bar1 <- barma(y_sim_bar1, ar = 1, link = "logit")
summary(fit_bar1)
coef(fit_bar1)

# Example 2: Fit a BARMA(1, 1) model
set.seed(2025)
y_sim_barma11 <- simu_barma(
  n      = 250,
  alpha  = 0.0,
  varphi = 0.6,
  theta  = 0.3,
  phi    = 25.0,
  link   = "logit",
  freq   = 12
)

fit_barma11 <- barma(y_sim_barma11, ar = 1, ma = 1, link = "logit")
summary(fit_barma11)

# Example 3: Fit a BMA(1) model (no AR component)
set.seed(2025)
y_sim_bma1 <- simu_barma(
  n      = 250,
  alpha  = 0.0,
  theta  = 0.3,
  phi    = 20.0,
  link   = "logit",
  freq   = 12
)

fit_bma1 <- barma(y_sim_bma1, ma = 1, link = "logit")
summary(fit_bma1)

# Example 4: BARMA(1, 1) model with harmonic seasonal regressors
hs <- sin(2 * pi * seq_along(y_sim_barma11) / 12)
hc <- cos(2 * pi * seq_along(y_sim_barma11) / 12)
X <- cbind(hs = hs, hc = hc)

fit_barma11_xreg <- barma(
  y_sim_barma11,
  ar   = 1,
  ma   = 1,
  link = "logit",
  xreg = X
)
summary(fit_barma11_xreg)

# Example 5: Fit a BARMA(1, 1) model using L-BFGS-B
set.seed(2025)
y_sim_barma11_LBFGSB <- simu_barma(
  n      = 250,
  alpha  = 0.0,
  varphi = 0.6,
  theta  = 0.3,
  phi    = 25.0,
  link   = "logit",
  freq   = 12
)

fit_barma11_LBFGSB <- barma(
  y_sim_barma11_LBFGSB,
  ar = 1,
  ma = 1,
  link = "logit",
  optimization = list(
    method = "L-BFGS-B",
    upper = c(Inf, 1, 1, Inf)
  )
)

summary(fit_barma11_LBFGSB)
}

}
\references{
Rocha, A.V., & Cribari-Neto, F. (2009). Beta autoregressive moving
average models. \emph{TEST}, 18(3), 529-545.
\doi{10.1007/s11749-008-0112-z}

Rocha, A.V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
moving average models. \emph{TEST}, 26, 451-459.
\doi{10.1007/s11749-017-0528-4}

Cribari-Neto, F., Costa, E., & Fonseca, R.V. (2025). Numerical stability
enhancements in beta autoregressive moving average model estimation.
\emph{Brazilian Journal of Probability and Statistics}, 39(4), 410-437.
\doi{10.1214/25-BJPS645}
}
\seealso{
\code{\link{simu_barma}} for simulation,
\code{\link{loglik_barma}} for likelihood computation,
\code{\link{score_vector_barma}} for gradient computation,
\code{\link{fim_barma}} for Fisher Information Matrix
}
\author{
Everton da Costa (Federal University of Pernambuco,
\email{everto.cost@gmail.com});
Francisco Cribari-Neto (Federal University of Pernambuco,
\email{francisco.cribari@ufpe.br})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{brasilia_ts}
\alias{brasilia_ts}
\title{Monthly relative humidity in Brasília}
\format{
A \code{ts} object with monthly observations, frequency 12,
  starting in January 1999.
}
\source{
NASA Prediction of Worldwide Energy Resources (POWER) project.
  \url{https://power.larc.nasa.gov/}
}
\usage{
brasilia_ts
}
\description{
Monthly relative humidity in Brasília, Brazil, expressed as a proportion.
The data were obtained from the NASA POWER project and cover
January 1999 to June 2024.
}
\examples{
data(brasilia_ts)
plot(brasilia_ts, ylab = "Relative humidity (proportion)", xlab = "Year")
}
\references{
Cribari-Neto, F., Costa, E., & Fonseca, R. V. (2025). Numerical stability
enhancements in beta autoregressive moving average model estimation.
\emph{Brazilian Journal of Probability and Statistics}, 39(4), 410--437.
\doi{10.1214/25-BJPS645}
}
\seealso{
\code{\link{brasilia_df}} for the same data as a data frame.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.barma.R
\name{print.barma}
\alias{print.barma}
\title{Print Method for a barma Model}
\usage{
\method{print}{barma}(x, ...)
}
\arguments{
\item{x}{A fitted model object of class `"barma"`.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns the original object `x`.
}
\description{
S3 method for printing a summary of a fitted `"barma"` model object.
}
\details{
This function provides a concise summary, showing the function call 
that generated the model, the link function used, and the final 
estimated coefficients.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/score_vector_barma.R
\name{score_vector_barma}
\alias{score_vector_barma}
\title{Score Vector for the BARMA Model}
\usage{
score_vector_barma(
  y,
  ar = integer(0),
  ma = integer(0),
  alpha,
  varphi = numeric(0),
  theta = numeric(0),
  phi,
  link = "logit",
  xreg = NULL,
  beta = NULL,
  penalty = FALSE
)
}
\arguments{
\item{y}{A numeric vector representing the time series data, with values
strictly in (0, 1).}

\item{ar}{A numeric vector specifying the autoregressive (AR) lags
(e.g., \code{c(1, 2)}). Defaults to \code{integer(0)}, which omits the
AR component entirely. Absence should be expressed by omitting this
argument or passing \code{integer(0)}.}

\item{ma}{A numeric vector specifying the moving average (MA) lags
(e.g., \code{1}). Defaults to \code{integer(0)}, which omits the
MA component entirely. Absence should be expressed by omitting this
argument or passing \code{integer(0)}.}

\item{alpha}{The intercept term (numeric scalar).}

\item{varphi}{A numeric vector of autoregressive (AR) parameters.
Absence should be expressed by omitting this argument or passing
\code{numeric(0)}.}

\item{theta}{A numeric vector of moving average (MA) parameters.
Absence should be expressed by omitting this argument or passing
\code{numeric(0)}.}

\item{phi}{The precision parameter of the BARMA model (must be positive and finite).
Larger values indicate less variance for a given mean.}

\item{link}{A character string specifying the link function:
\code{"logit"} (default), \code{"probit"}, or \code{"cloglog"}.}

\item{xreg}{A matrix or data frame of static regressors (optional).
Must have the same number of rows as length of \code{y}.}

\item{beta}{A numeric vector of regression coefficients for \code{xreg} (optional).
Length must match number of columns in \code{xreg}.}

\item{penalty}{Logical. If \code{TRUE}, the gradient of the ridge penalty of
Cribari-Neto, Costa and Fonseca (2025) is subtracted from the score
vector, yielding the penalized score
\eqn{S_{\text{pen}}(\boldsymbol{\nu}) = S(\boldsymbol{\nu}) -
2(n-a)\lambda_n \boldsymbol{\nu}}, where \eqn{\lambda_n =
1/(n-a)^{0.9}} and \eqn{\boldsymbol{\nu}} collects \eqn{\alpha},
\eqn{\boldsymbol{\varphi}}, and \eqn{\boldsymbol{\theta}}
(\eqn{\phi} and \eqn{\boldsymbol{\beta}} are excluded).
Defaults to \code{FALSE}.}
}
\value{
A numeric vector of the same length as the parameter vector,
  giving the partial derivatives of the log-likelihood with respect
  to each parameter. The order of the components is:
  \code{(alpha, varphi, theta, beta, phi)}.
}
\description{
Computes the score vector (gradient of the log-likelihood)
  for the Beta Autoregressive Moving Average (BARMA) model at a given
  parameter vector. This function is designed for users who:
\itemize{
  \item Implement custom optimization algorithms
  \item Verify theoretical properties
  \item Conduct simulation studies
  \item Debug model fitting
  \item Integrate BARMA models into their own workflows
}
}
\examples{
\donttest{
  # Example 1: Score vector for a BAR(1) model (no MA component)
  set.seed(2025)
  y_sim_bar1 <- simu_barma(
    n      = 250,
    alpha  = 0.0,
    varphi = 0.6,
    phi    = 25.0,
    link   = "logit",
    freq   = 12
  )

  score_vector_barma(
    y      = y_sim_bar1,
    ar     = 1,
    alpha  = 0.0,
    varphi = 0.6,
    theta  = numeric(0),
    phi    = 25.0,
    link   = "logit"
  )

  # Example 2: Score vector for a BARMA(1, 1) model
  set.seed(2025)
  y_sim_barma11 <- simu_barma(
    n      = 250,
    alpha  = 0.0,
    varphi = 0.6,
    theta  = 0.3,
    phi    = 25.0,
    link   = "logit",
    freq   = 12
  )

  score_vector_barma(
    y      = y_sim_barma11,
    ar     = 1,
    ma     = 1,
    alpha  = 0.0,
    varphi = 0.6,
    theta  = 0.3,
    phi    = 25.0,
    link   = "logit"
  )

  # Example 3: Score vector for a BMA(1) model (no AR component)
  set.seed(2025)
  y_sim_bma1 <- simu_barma(
    n      = 250,
    alpha  = 0.0,
    theta  = 0.3,
    phi    = 20.0,
    link   = "logit",
    freq   = 12
  )

  score_vector_barma(
    y      = y_sim_bma1,
    ma     = 1,
    alpha  = 0.0,
    varphi = numeric(0),
    theta  = 0.3,
    phi    = 20.0,
    link   = "logit"
  )
}

}
\seealso{
\code{\link{barma}}, \code{\link{loglik_barma}},
  \code{\link{fim_barma}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forecast.barma.R
\name{forecast.barma}
\alias{forecast.barma}
\title{Forecast a barma Model}
\usage{
\method{forecast}{barma}(object, h = 6, xreg = NULL, ...)
}
\arguments{
\item{object}{A fitted model object of class `barma`. Must contain
`object$xreg` if regressors were used.}

\item{h}{The number of steps to forecast ahead (forecast horizon).
Default is 6.}

\item{xreg}{A matrix of future regressor values for the forecast horizon.
Should have h rows and the same number of columns as xreg used in model
fitting.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A `ts` object containing the point forecasts for h steps ahead.
}
\description{
S3 method for producing forecasts from a fitted `barma` model object.
}
\details{
This function computes dynamic, multi-step-ahead point forecasts.
It implements a "Regression with ARMA errors" logic, where the AR components
are applied to the deviations from the regression line:
\eqn{AR(g(y_{t-k}) - x_{t-k}^\top \beta)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simu_barma.R
\name{simu_barma}
\alias{simu_barma}
\title{Simulate a Beta Autoregressive Moving Average (BARMA) Time Series}
\usage{
simu_barma(
  n,
  alpha = 0,
  varphi = NA,
  theta = NA,
  phi = 20,
  freq = 12,
  link = "logit"
)
}
\arguments{
\item{n}{The desired length of the final time series (after burn-in).}

\item{alpha}{The intercept term (\eqn{\alpha}) in the linear predictor.
Default is `0.0`.}

\item{varphi}{A numeric vector of autoregressive (AR) parameters (\eqn{\varphi}).
Default is `NA` for models without an AR component.}

\item{theta}{A numeric vector of moving average (MA) parameters (\eqn{\theta}).
Default is `NA` for models without an MA component.}

\item{phi}{The precision parameter (\eqn{\phi > 0}) of the Beta distribution.
Higher values result in less variance. Default is `20`.}

\item{freq}{The frequency of the resulting `ts` object (e.g., 12 for monthly,
4 for quarterly). Default is `12`.}

\item{link}{The link function to connect the mean \eqn{\mu_t} to the linear
predictor. Must be one of `"logit"` (default), `"probit"`, `"cloglog"`, or
`"loglog"`.}
}
\value{
A `ts` object of length `n` containing the simulated Beta-distributed
  time series.
}
\description{
Generates a random time series from a Beta Autoregressive Moving Average
(BARMA) model. The function can simulate BARMA(p,q), BAR(p), or BMA(q)
processes.
}
\details{
The model type is determined by the `varphi` and `theta` parameters:
\itemize{
  \item **BARMA(p,q):** Both `varphi` and `theta` are provided.
  \item **BAR(p):** Only `varphi` is provided (`theta` is `NA`).
  \item **BMA(q):** Only `theta` is provided (`varphi` is `NA`).
}
}
\examples{
# Set seed for reproducibility
# --- Example 1: Simulate a BAR(1) process ---
# y_t depends on y_{t-1}
set.seed(2025)
bar1_series <- simu_barma(n = 250, alpha = 0.0, varphi = 0.5, phi = 20,
link = "logit")
plot(bar1_series, main = "Simulated BAR(1) Process", ylab = "Value")

# --- Example 2: Simulate a BMA(1) process ---
# y_t depends on the previous error term
set.seed(2025)
bma1_series <- simu_barma(n = 250, alpha = 0.0, theta = -0.2, phi = 20)
plot(bma1_series, main = "Simulated BMA(1) Process", ylab = "Value")

# --- Example 3: Simulate a BARMA(2,1) process with a cloglog link ---
set.seed(2025)
barma21_series <- simu_barma(
  n = 200,
  alpha = 0.0,
  varphi = c(0.4, 0.2), # AR(2) components
  theta = -0.3,         # MA(1) component
  phi = 20,
  link = "cloglog"
)
plot(barma21_series, main = "Simulated BARMA(2,1) Process", ylab = "Value")

}
\seealso{
\code{\link{barma}}, \code{\link{make_link_structure}}
}
\author{
Original R code by: Fabio M. Bayer (Federal University of Santa Maria, <bayer@ufsm.br>)
Modified and improved by: Everton da Costa (Federal University of Pernambuco, <everto.cost@gmail.com>)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_link_structure.R
\name{make_link_structure}
\alias{make_link_structure}
\title{Create Link Function Structure for BARMA Models}
\usage{
make_link_structure(link = "logit")
}
\arguments{
\item{link}{A character string specifying the link function.
Defaults to \code{"logit"}. Accepted values are \code{"logit"},
\code{"probit"}, \code{"cloglog"}, and \code{"loglog"}.}
}
\value{
A list with three components:
\item{linkfun}{The link function \eqn{g(\mu)}, transforming
  \eqn{\mu \in (0,1)} to \eqn{\eta \in (-\infty, \infty)}.}
\item{linkinv}{The inverse link function \eqn{g^{-1}(\eta)},
  transforming \eqn{\eta} back to \eqn{\mu}.}
\item{mu.eta}{The derivative \eqn{d\mu/d\eta} of the inverse
  link function.}
}
\description{
A helper function that constructs a list containing the link function,
its inverse, and the derivative of the mean function. It extends the
standard \code{make.link} by adding support for the \code{"loglog"} link.
}
\details{
This function is used by \code{\link{barma}}, \code{\link{loglik_barma}},
\code{\link{score_vector_barma}}, and \code{\link{fim_barma}} to handle
the \code{link} argument in a standardized way. It is also exported for
users who implement custom workflows.

For the \code{"logit"}, \code{"probit"}, and \code{"cloglog"} links,
the function acts as a wrapper around the base R \code{make.link}.

For the \code{"loglog"} link, which is not available in \code{make.link},
the necessary components are defined explicitly:
\itemize{
  \item Link function: \eqn{g(\mu) = -\log(-\log(\mu))}
  \item Inverse link: \eqn{g^{-1}(\eta) = \exp(-\exp(-\eta))}
  \item Derivative \eqn{\frac{d\mu}{d\eta}}:
    \eqn{\exp(-\exp(-\eta) - \eta)}
}

If an unsupported link is provided, the function stops with an error
message listing the available options.
}
\examples{
# --- Create a logit link structure ---
logit_link <- make_link_structure(link = "logit")

# Apply the link function
mu <- 0.5
eta <- logit_link$linkfun(mu)
print(eta) # Should be 0

# Apply the inverse link function
mu_restored <- logit_link$linkinv(eta)
print(mu_restored) # Should be 0.5

# --- Create a loglog link structure ---
loglog_link <- make_link_structure(link = "loglog")

# Apply the loglog link function
mu_loglog <- 0.8
eta_loglog <- loglog_link$linkfun(mu_loglog)
print(eta_loglog)

# Apply the inverse loglog link function
mu_restored_loglog <- loglog_link$linkinv(eta_loglog)
print(mu_restored_loglog) # Should be ~0.8

}
\seealso{
\code{\link{barma}}, \code{\link[stats]{make.link}}
}
\author{
Original R code by Fabio M. Bayer
(Federal University of Santa Maria, \email{bayer@ufsm.br}).
Modified and improved by Everton da Costa
(Federal University of Pernambuco, \email{everto.cost@gmail.com}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/start_values.R
\name{.get_phi_start}
\alias{.get_phi_start}
\title{Internal Helper to Calculate Initial Phi}
\usage{
.get_phi_start(fit, n_eff, linkinv, mu.eta)
}
\arguments{
\item{fit}{An 'lm.fit' object.}

\item{n_eff}{The effective number of observations used in the fit.}

\item{linkinv}{The inverse link function.}

\item{mu.eta}{The derivative of the mean w.r.t. eta (d(mu)/d(eta)).}
}
\value{
A single numeric value for phi_start.
}
\description{
Calculates the initial value for the precision parameter phi based on the
residuals of an initial 'lm.fit' object. The formula is adapted and
empirically improved from Ferrari & Cribari-Neto (2004) — specifically,
the "-1" term in the original precision estimator is omitted, as Monte
Carlo simulations showed better optimization performance without it.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.barma.R
\name{coef.barma}
\alias{coef.barma}
\title{Extract Coefficients from a barma Model}
\usage{
\method{coef}{barma}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class `"barma"`.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A named numeric vector of all estimated coefficients 
(e.g., alpha, varphi, theta, phi).
}
\description{
S3 method for extracting the vector of estimated coefficients from a fitted 
model object of class `"barma"`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{brasilia_df}
\alias{brasilia_df}
\title{Monthly relative humidity in Brasília (data frame)}
\format{
A data frame with 2 columns:
\describe{
  \item{time}{Month of observation (\code{yearmon}).}
  \item{y}{Relative humidity as a proportion (numeric, in (0, 1)).}
}
}
\source{
NASA Prediction of Worldwide Energy Resources (POWER) project.
  \url{https://power.larc.nasa.gov/}
}
\usage{
brasilia_df
}
\description{
The same series as \code{\link{brasilia_ts}}, stored as a data frame
with a \code{yearmon} time column.
}
\examples{
data(brasilia_df)
head(brasilia_df)
}
\seealso{
\code{\link{brasilia_ts}} for the same data as a \code{ts} object.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residuals.barma.R
\name{residuals.barma}
\alias{residuals.barma}
\title{Residuals for a Fitted betaARMA Model}
\usage{
\method{residuals}{barma}(object, type = c("pearson", "quantile", "link", "raw"), ...)
}
\arguments{
\item{object}{A fitted model object of class \code{"barma"}, as returned by
\code{\link{barma}}.}

\item{type}{Character string specifying the type of residuals to compute. One of:
\describe{
  \item{\code{"pearson"} (default)}{Pearson residuals on the response
    scale, as defined in Ferrari and Cribari-Neto (2004). Recommended
    for ACF/PACF analysis and portmanteau tests.}
  \item{\code{"quantile"}}{Quantile residuals (Dunn and Smyth, 1996),
    approximately i.i.d. \eqn{N(0,1)} under a correctly specified model.
    Recommended for normality assessment.}
  \item{\code{"link"}}{Residuals on the predictor (link) scale. Retained
    for compatibility with earlier versions of the package.}
  \item{\code{"raw"}}{Raw residuals \eqn{y_t - \hat{\mu}_t} on the
    response scale.}
}}

\item{...}{Additional arguments (currently unused, included for S3 consistency).}
}
\value{
A numeric \code{ts} object of the same length as the response series,
  with the first \code{max_lag} values set to \code{NA}. The \code{ts}
  attributes (\code{start} and \code{frequency}) are inherited from the
  original response series stored in \code{object$y}.
}
\description{
Computes residuals for a fitted \eqn{\beta}ARMA model object of class
\code{"barma"}. Four types are available, controlled by the \code{type}
argument.
}
\details{
**Pearson residuals** (default) are defined on the response scale as:
\deqn{r_t = \frac{y_t - \hat{\mu}_t}{\sqrt{\hat{\mu}_t(1-\hat{\mu}_t)/(1+\hat{\phi})}}}
where \eqn{\hat{\mu}_t} is the fitted conditional mean and \eqn{\hat{\phi}}
is the estimated precision parameter. This residual was defined in Ferrari
and Cribari-Neto (2004) for beta regression models and is adopted here in
the \eqn{\beta}ARMA setting. These residuals are appropriate for ACF/PACF
analysis and portmanteau tests (Ljung-Box and Monti).

**Quantile residuals** (Dunn and Smyth, 1996) are defined as:
\deqn{r_t^Q = \Phi^{-1}\!\left(F(y_t;\,\hat{\mu}_t\hat{\phi},\,(1-\hat{\mu}_t)\hat{\phi})\right)}
where \eqn{F(\cdot\,;\,a,b)} is the Beta CDF with shape parameters
\eqn{a = \hat{\mu}_t\hat{\phi}} and \eqn{b = (1-\hat{\mu}_t)\hat{\phi}},
and \eqn{\Phi^{-1}} is the standard normal quantile function. Under a
correctly specified model, quantile residuals are approximately i.i.d.
\eqn{N(0,1)}, making them suitable for normality assessment via
histograms and Q-Q plots.

**Link-scale residuals** are defined on the predictor scale as:
\deqn{r_t^L = \frac{g(y_t) - \hat{\eta}_t}{\sqrt{[g'(\hat{\mu}_t)]^2
  \cdot \hat{\mu}_t(1-\hat{\mu}_t)/(1+\hat{\phi})}}}
where \eqn{g(\cdot)} is the link function, \eqn{\hat{\eta}_t} is the
fitted linear predictor, and \eqn{g'(\hat{\mu}_t) = d\eta/d\mu}. This
residual was used in earlier versions of the package and is retained for
compatibility.

**Raw residuals** are the simple difference on the response scale:
\deqn{r_t^{\mathrm{raw}} = y_t - \hat{\mu}_t}
}
\examples{
data(brasilia_ts)
fit <- barma(brasilia_ts, ar = 1, ma = 1)

# Pearson residuals (default) -- Ferrari & Cribari-Neto (2004)
res <- residuals(fit)

# Raw residuals
res_raw <- residuals(fit, type = "raw")

# Quantile residuals -- recommended for normality assessment
res_q <- residuals(fit, type = "quantile")

# Link-scale residuals
res_link <- residuals(fit, type = "link")

}
\references{
Dunn, P. K., & Smyth, G. K. (1996). Randomized quantile residuals.
  \emph{Journal of Computational and Graphical Statistics}, 5(3), 236--244.
  \doi{10.1080/10618600.1996.10474708}

  Ferrari, S. L. P., & Cribari-Neto, F. (2004). Beta regression for
  modelling rates and proportions. \emph{Journal of Applied Statistics},
  31(7), 799--815. \doi{10.1080/0266476042000214501}

  Rocha, A. V., & Cribari-Neto, F. (2009). Beta autoregressive moving
  average models. \emph{TEST}, 18(3), 529--545.
  \doi{10.1007/s11749-008-0112-z}

  Rocha, A. V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
  moving average models. \emph{TEST}, 26, 451--459.
  \doi{10.1007/s11749-017-0528-4}
}
\seealso{
\code{\link{barma}} for model fitting;
  \code{\link{fitted.barma}} for fitted values;
  \code{\link{plot.barma}} for diagnostic plots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik_barma.R
\name{loglik_barma}
\alias{loglik_barma}
\title{Compute Conditional Log-Likelihood for Beta Autoregressive
Moving Average Models}
\usage{
loglik_barma(
  y,
  ar = integer(0),
  ma = integer(0),
  alpha,
  varphi = numeric(0),
  theta = numeric(0),
  phi,
  link = "logit",
  xreg = NULL,
  beta = NULL,
  penalty = FALSE
)
}
\arguments{
\item{y}{A numeric vector representing the time series data, with values
strictly in (0, 1).}

\item{ar}{A numeric vector specifying the autoregressive (AR) lags
(e.g., \code{c(1, 2)}). Defaults to \code{integer(0)}, which omits the
AR component entirely. Absence should be expressed by omitting this
argument or passing \code{integer(0)}.}

\item{ma}{A numeric vector specifying the moving average (MA) lags
(e.g., \code{1}). Defaults to \code{integer(0)}, which omits the
MA component entirely. Absence should be expressed by omitting this
argument or passing \code{integer(0)}.}

\item{alpha}{The intercept term (numeric scalar).}

\item{varphi}{A numeric vector of autoregressive (AR) parameters.
Absence should be expressed by omitting this argument or passing
\code{numeric(0)}.}

\item{theta}{A numeric vector of moving average (MA) parameters.
Absence should be expressed by omitting this argument or passing
\code{numeric(0)}.}

\item{phi}{The precision parameter of the BARMA model (must be positive and finite).
Larger values indicate less variance for a given mean.}

\item{link}{A character string specifying the link function:
\code{"logit"} (default), \code{"probit"}, or \code{"cloglog"}.}

\item{xreg}{A matrix or data frame of static regressors (optional).
Must have the same number of rows as length of \code{y}.}

\item{beta}{A numeric vector of regression coefficients for \code{xreg} (optional).
Length must match number of columns in \code{xreg}.}

\item{penalty}{Logical. If \code{TRUE}, a ridge penalty is subtracted from the
conditional log-likelihood, yielding the penalized conditional
log-likelihood proposed by Cribari-Neto, Costa and Fonseca (2025).
The penalty term is \eqn{(n - a) \lambda_n \|\boldsymbol{\nu}\|^2},
where \eqn{\lambda_n = 1/(n - a)^{0.9}} and \eqn{\boldsymbol{\nu}}
collects \eqn{\alpha}, \eqn{\boldsymbol{\varphi}}, and
\eqn{\boldsymbol{\theta}} (the precision parameter \eqn{\phi} and
regression coefficients \eqn{\boldsymbol{\beta}} are excluded).
Defaults to \code{FALSE}.}
}
\value{
A numeric scalar representing the (penalized) conditional log-likelihood
value. When \code{penalty = FALSE} (default), the standard conditional
log-likelihood is returned. When \code{penalty = TRUE}, the ridge-penalized
conditional log-likelihood of Cribari-Neto, Costa and Fonseca (2025) is
returned instead.
Returns \code{-Inf} if:
\itemize{
  \item \code{phi} is non-positive or non-finite
  \item Insufficient observations for specified lag structure
  \item Fitted values are outside (0, 1)
  \item Any numerical issues occur
}
}
\description{
Computes the conditional log-likelihood of a Beta Autoregressive
Moving Average (BARMA) model. This function is designed for users who:
\itemize{
  \item Implement custom optimization algorithms
  \item Verify theoretical properties
  \item Conduct simulation studies
  \item Debug model fitting
  \item Integrate BARMA models into their own workflows
}
}
\details{
Log-Likelihood for BARMA Models


The log-likelihood is computed as:
\deqn{\ell = \sum_{t=m+1}^{n} \log f(y_t | \mu_t, \phi)}

where \eqn{f} is the Beta density with shape parameters
\eqn{shape1 = \mu_t * \phi} and \eqn{shape2 = (1-\mu_t)*\phi}.

The linear predictor is constructed as:
\deqn{\eta_t = \alpha + X_t \beta + \sum_{i=1}^{p} \varphi_i
(y_{t-i} - X_{t-i}\beta) + \sum_{j=1}^{q} \theta_j \epsilon_{t-j}}

**Important**: This function implements the corrections from the 2017
Erratum (Rocha & Cribari-Neto, 2017) for moving average components.
See References section for details.

**Parameter Order**: Parameters should be supplied in the order:
alpha, varphi (AR), theta (MA), phi, beta (regressors).
This matches the parameter order used by \code{\link{barma}}.
}
\examples{
\donttest{
  # Example 1: Log-likelihood for a BAR(1) model (no MA component)
  set.seed(2025)
  y_sim_bar1 <- simu_barma(
    n      = 250,
    alpha  = 0.0,
    varphi = 0.6,
    phi    = 25.0,
    link   = "logit",
    freq   = 12
  )

  loglik_barma(
    y      = y_sim_bar1,
    ar     = 1,
    alpha  = 0.0,
    varphi = 0.6,
    theta  = numeric(0),
    phi    = 25.0,
    link   = "logit"
  )

  # Example 2: Log-likelihood for a BARMA(1, 1) model
  set.seed(2025)
  y_sim_barma11 <- simu_barma(
    n      = 250,
    alpha  = 0.0,
    varphi = 0.6,
    theta  = 0.3,
    phi    = 25.0,
    link   = "logit",
    freq   = 12
  )

  loglik_barma(
    y      = y_sim_barma11,
    ar     = 1,
    ma     = 1,
    alpha  = 0.0,
    varphi = 0.6,
    theta  = 0.3,
    phi    = 25.0,
    link   = "logit"
  )

  # Example 3: Log-likelihood for a BMA(1) model (no AR component)
  set.seed(2025)
  y_sim_bma1 <- simu_barma(
    n      = 250,
    alpha  = 0.0,
    theta  = 0.3,
    phi    = 20.0,
    link   = "logit",
    freq   = 12
  )

  loglik_barma(
    y      = y_sim_bma1,
    ma     = 1,
    alpha  = 0.0,
    varphi = numeric(0),
    theta  = 0.3,
    phi    = 20.0,
    link   = "logit"
  )

  # Example 4: Ridge-penalized log-likelihood for a BARMA(1, 1) model
  # Uses penalty = TRUE as recommended by Cribari-Neto, Costa and
  # Fonseca (2025, BJPS) to improve numerical stability.
  loglik_barma(
    y       = y_sim_barma11,
    ar      = 1,
    ma      = 1,
    alpha   = 0.0,
    varphi  = 0.6,
    theta   = 0.3,
    phi     = 25.0,
    link    = "logit",
    penalty = TRUE
  )
}

}
\references{
Rocha, A.V., & Cribari-Neto, F. (2009). Beta autoregressive moving
average models. \emph{TEST}, 18(3), 529-545.
\doi{10.1007/s11749-008-0112-z}

Rocha, A.V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
moving average models. \emph{TEST}, 26, 451-459.
\doi{10.1007/s11749-017-0528-4}

Cribari-Neto, F., Costa, E., & Fonseca, R.V. (2025). Numerical stability
enhancements in beta autoregressive moving average model estimation.
\emph{Brazilian Journal of Probability and Statistics}, 39(4), 410-437.
\doi{10.1214/25-BJPS645}
}
\seealso{
\code{\link{barma}} for model fitting,
\code{\link{score_vector_barma}} for gradient computation,
\code{\link{fim_barma}} for Fisher Information Matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.barma.R
\name{plot.barma}
\alias{plot.barma}
\title{Diagnostic Plots for a Fitted betaARMA Model}
\usage{
\method{plot}{barma}(
  x,
  which = c("default", "all", "fitted", "tsplot", "acf", "pacf", "ljungbox", "monti",
    "hist", "qq"),
  residual_type = c("pearson", "quantile", "link", "raw"),
  lag_max = 24,
  title = NULL,
  colour_observed = "#00BFC4",
  colour_fitted = "#F8766D",
  colour_residual = "#7CAE00",
  ...
)
}
\arguments{
\item{x}{An object of class \code{"barma"}, as returned by \code{\link{barma}}.}

\item{which}{A character string controlling which panel(s) to display. One of:
\describe{
  \item{\code{"default"} (default)}{Four-panel 2\eqn{\times}2 grid:
    observed vs. fitted (top left), residual ACF (top right),
    Ljung-Box p-values (bottom left), and residual PACF (bottom right).}
  \item{\code{"all"}}{Six-panel 3\eqn{\times}2 grid:
    observed vs. fitted (top left), residuals over time (top right),
    residual ACF (middle left), residual PACF (middle right),
    Ljung-Box p-values (bottom left), Monti p-values (bottom right).}
  \item{\code{"fitted"}}{Single panel: observed vs. fitted values.}
  \item{\code{"tsplot"}}{Single panel: residuals plotted as a time series,
    with a horizontal reference line at zero and dashed lines at
    \eqn{\pm 3} (ad hoc threshold; not shown for \code{"raw"} residuals).}
  \item{\code{"acf"}}{Single panel: residual ACF.}
  \item{\code{"pacf"}}{Single panel: residual PACF.}
  \item{\code{"ljungbox"}}{Single panel: Ljung-Box p-values.}
  \item{\code{"monti"}}{Single panel: Monti test p-values.}
  \item{\code{"hist"}}{Single panel: residual distribution histogram with
    kernel density overlay.}
  \item{\code{"qq"}}{Single panel: Normal Q-Q plot. Always uses quantile
    residuals regardless of \code{residual_type}; a message is issued if
    the type is overridden.}
}}

\item{residual_type}{Character string passed to \code{\link{residuals.barma}} controlling
which residuals are computed and displayed. One of \code{"pearson"}
(default), \code{"quantile"}, \code{"link"} or \code{"raw"}. Pearson
residuals are recommended for ACF/PACF analysis and portmanteau
tests (Scher, Cribari-Neto, Pumi, and Bayer, 2020); quantile residuals
are recommended for normality assessment (Dunn and Smyth, 1996). See also
Ferrari and Cribari-Neto (2004).}

\item{lag_max}{Maximum lag to display in the ACF, PACF, Ljung-Box, and Monti panels.
Defaults to \code{24}, appropriate for monthly series.}

\item{title}{An optional character string used as the overall title of the plot.
Defaults to \code{NULL} (no title).}

\item{colour_observed}{Colour for observed values in the fitted-values panel.
Defaults to \code{"#00BFC4"} (teal).}

\item{colour_fitted}{Colour for fitted values in the fitted-values panel.
Defaults to \code{"#F8766D"} (red).}

\item{colour_residual}{Colour for residual lines, ACF/PACF bars, portmanteau test points, and
histogram fill. Defaults to \code{"#7CAE00"} (green).}

\item{...}{Additional arguments (currently unused, included for S3 consistency).}
}
\value{
For grid requests (\code{"default"} or \code{"all"}), invisibly returns a
  \code{gtable} object produced by \code{gridExtra::arrangeGrob}, which can
  be saved with \code{ggplot2::ggsave}. For single-panel requests, invisibly
  returns the individual \code{ggplot} object.
}
\description{
Produces diagnostic plots for a \eqn{\beta}ARMA model fitted by
\code{\link{barma}}. By default (\code{which = "default"}), four panels are
displayed in a 2\eqn{\times}2 grid: observed vs. fitted values, residual
ACF, Ljung-Box p-values, and residual PACF.
}
\details{
**Residual type.** All panels (except \emph{Observed vs. Fitted}) use the
residuals selected by \code{residual_type}. The residual type is displayed
in each panel subtitle for transparency.

**Reference lines in the residuals-over-time panel.** The dashed horizontal 
lines are drawn at \eqn{\pm 3} as an ad hoc threshold for identifying 
atypical observations. No distributional assumption is made; observations 
outside these bounds may warrant individual inspection. Pearson and 
link-scale residuals do not follow a standard normal distribution, so 
normal-distribution-based quantiles (e.g., \eqn{\pm 1.96} or 
\eqn{\pm 2.576}) are not appropriate reference values for such residuals. 
Quantile residuals are approximately distributed as \eqn{N(0,1)} under a 
correctly specified model, so \eqn{\pm 3} is a conservative but reasonable 
threshold for them.

**Q-Q plot residuals.** The \code{"qq"} panel always uses quantile
residuals (Dunn and Smyth, 1996), regardless of \code{residual_type}.
Quantile residuals are the only type theoretically expected to follow
\eqn{N(0,1)} under a correctly specified model; using Pearson, raw, or
link-scale residuals in a normal Q-Q plot would produce misleading results.
A message is issued when \code{residual_type} is overridden.

**Effective sample size.** Both the Ljung-Box and Monti test statistics
rely on the effective sample size \eqn{N_{eff} = n - \max(p, q)} (where
\eqn{\max(p, q)} corresponds to the maximum lag in the model).

**Ljung-Box test.** The p-values are computed from the Ljung-Box test
statistic.
\eqn{Q_{LB}(k) = N_{eff}(N_{eff}+2)\sum_{j=1}^{k}\hat{\rho}_j^2/(N_{eff}-j)},
where \eqn{\hat{\rho}_j} is the \eqn{j}-th sample autocorrelation of the
residuals and \eqn{N_{eff} = n - \max(p, q)} is the effective sample size.
The degrees of freedom are adjusted by subtracting the total
number of estimated  AR and MA parameters,
\eqn{n_{ar} + n_{ma}} (Scher, Cribari-Neto, Pumi, and Bayer (2020)). The
 first \eqn{n_{ar} + n_{ma}} lags are set to \code{NA}
since the test statistic is not defined for \eqn{k \leq n_{ar} + n_{ma}}.

**Monti test.** The p-values are computed from the test statistic
\eqn{M(k) = N_{eff}(N_{eff}+2)\sum_{j=1}^{k}\hat{\rho}_{jj}^2/(N_{eff}-j)},
where \eqn{\hat{\rho}_{jj}} is the \eqn{j}-th sample partial autocorrelation
of the residuals, and \eqn{N_{eff} = n - \max\_lag} is the effective sample
size after stripping initial NAs. The degrees of freedom are adjusted to
\eqn{k - (n_{ar} + n_{ma})}
(Monti, 1994; Scher, Cribari-Neto, Pumi, and Bayer (2020)). The first
\eqn{n_{ar} + n_{ma}} lags are set to \code{NA} as the statistic is not
defined for \eqn{k \leq n_{ar} + n_{ma}}.
}
\examples{
data(brasilia_ts)
fit <- barma(brasilia_ts, ar = 1, ma = 1)

# Default four-panel diagnostic grid (Pearson residuals)
plot(fit)

# Six-panel diagnostic grid
plot(fit, which = "all")

# Single panel: residuals as time series
plot(fit, which = "tsplot")

# Single panel: Ljung-Box p-values only
plot(fit, which = "ljungbox")

# Single panel: Monti p-values only
plot(fit, which = "monti")

# Normal Q-Q plot (always uses quantile residuals)
plot(fit, which = "qq")

# Histogram with quantile residuals
plot(fit, which = "hist", residual_type = "quantile")

}
\references{
Dunn, P. K., & Smyth, G. K. (1996). Randomized quantile residuals.
  \emph{Journal of Computational and Graphical Statistics}, 5(3), 236--244.
  \doi{10.1080/10618600.1996.10474708}

  Ferrari, S. L. P., & Cribari-Neto, F. (2004). Beta regression for
  modelling rates and proportions. \emph{Journal of Applied Statistics},
  31(7), 799--815. \doi{10.1080/0266476042000214501}

  Monti, A. C. (1994). A proposal for a residual autocorrelation test in
  linear models. \emph{Biometrika}, 81(4), 776--780.
  \doi{10.1093/biomet/81.4.776}

  Rocha, A. V., & Cribari-Neto, F. (2009). Beta autoregressive moving
  average models. \emph{TEST}, 18(3), 529--545.
  \doi{10.1007/s11749-008-0112-z}

  Rocha, A. V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
  moving average models. \emph{TEST}, 26, 451--459.
  \doi{10.1007/s11749-017-0528-4}
}
\seealso{
\code{\link{barma}} for model fitting;
  \code{\link{residuals.barma}} for the residual extraction method;
  \code{\link{fitted.barma}} for the fitted-value extraction method;
  \code{\link{forecast.barma}} for out-of-sample forecasting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fim_barma.R
\name{fim_barma}
\alias{fim_barma}
\title{Compute Fisher Information Matrix for Beta Autoregressive
Moving Average Models}
\usage{
fim_barma(
  y,
  ar = integer(0),
  ma = integer(0),
  alpha,
  varphi = numeric(0),
  theta = numeric(0),
  phi,
  link = "logit",
  xreg = NULL,
  beta = NULL,
  penalty = FALSE
)
}
\arguments{
\item{y}{A numeric vector representing the time series data, in (0, 1).}

\item{ar}{A numeric vector specifying the autoregressive (AR) lags
(e.g., \code{c(1, 2)}). Defaults to \code{integer(0)}, which omits the
AR component entirely. Absence should be expressed by omitting this
argument or passing \code{integer(0)}.}

\item{ma}{A numeric vector specifying the moving average (MA) lags
(e.g., \code{1}). Defaults to \code{integer(0)}, which omits the
MA component entirely. Absence should be expressed by omitting this
argument or passing \code{integer(0)}.}

\item{alpha}{The intercept term (numeric scalar).}

\item{varphi}{A numeric vector of autoregressive (AR) parameters.
Absence should be expressed by omitting this argument or passing
\code{numeric(0)}.}

\item{theta}{A numeric vector of moving average (MA) parameters.
Absence should be expressed by omitting this argument or passing
\code{numeric(0)}.}

\item{phi}{The precision parameter of the BARMA model (must be positive and finite).
Larger values indicate less variance for a given mean.}

\item{link}{A character string specifying the link function:
\code{"logit"} (default), \code{"probit"}, or \code{"cloglog"}.}

\item{xreg}{A matrix or data frame of static regressors (optional).
Must have the same number of rows as length of \code{y}.}

\item{beta}{A numeric vector of regression coefficients for \code{xreg} (optional).
Length must match number of columns in \code{xreg}.}

\item{penalty}{Logical. If \code{TRUE}, the ridge penalty of Cribari-Neto, Costa and
Fonseca (2025) is added to the Fisher Information Matrix, yielding the
penalized FIM \eqn{K_{\text{pen}}(\boldsymbol{\nu}) = K(\boldsymbol{\nu})
+ 2(n-a)\Lambda}, where \eqn{\Lambda =
\text{diag}\{\lambda_n,\ldots,\lambda_n, 0,\ldots,0\}} with
\eqn{\lambda_n = 1/(n-a)^{0.9}} applied to \eqn{\alpha},
\eqn{\boldsymbol{\varphi}}, and \eqn{\boldsymbol{\theta}} only.
Defaults to \code{FALSE}.}
}
\value{
A list containing:
\item{fisher_info_mat}{The Fisher Information Matrix (numeric matrix).
  Dimensions: \code{(k+p+q+2) x (k+p+q+2)} where k is number of regressors,
  p is AR order, q is MA order. The matrix is symmetric and should be
  positive definite at the MLE.}
\item{fitted_ts}{The fitted values (conditional mean) as a \code{ts} object,
  with the same time index as the input \code{y}. Values for the first
  \eqn{m = \max(p,q)} observations are \code{NA}.}
\item{muhat_effective}{The fitted conditional means (numeric vector)
  excluding the first \eqn{m} observations.}
\item{etahat_full}{The estimated linear predictor values (numeric vector,
  length = length(y)), with \code{NA} for the first \eqn{m} observations.}
\item{errorhat_full}{The estimated errors on the predictor scale
  (numeric vector, length = length(y)), zero-padded for the first
  \eqn{m} observations.}
}
\description{
Computes the expected Fisher Information Matrix (FIM) of a Beta
Autoregressive Moving Average (BARMA) model. This function also
efficiently returns auxiliary values like fitted values and residuals.

This function is designed for users who:
\itemize{
  \item Compute standard errors of parameter estimates
  \item Construct confidence intervals
  \item Perform hypothesis tests
  \item Verify theoretical properties
  \item Conduct simulation studies
}
}
\details{
Fisher Information Matrix for BARMA Models


The expected Fisher Information Matrix is computed analytically using
the trigamma-based expressions from Rocha and Cribari-Neto (2009).
Standard errors are obtained via \code{sqrt(diag(solve(FIM)))}.

**Non-Diagonal Structure**: Unlike generalized linear models, the FIM
is not block-diagonal due to the coupling introduced by the ARMA
dynamics. This is documented in the Rocha & Cribari-Neto (2009) paper.

**Important**: This function implements the corrections from the 2017
Erratum (Rocha & Cribari-Neto, 2017) for moving average components.
See References section for details.

**Parameter Order**: Parameters should be supplied in the order:
alpha, varphi (AR), theta (MA), beta (regressors), phi.
This matches the parameter order used by \code{\link{barma}}.
}
\examples{
\donttest{
  # Example 1: Fisher Information Matrix for a BAR(1) model (no MA component)
  set.seed(2025)
  y_sim_bar1 <- simu_barma(
    n      = 250,
    alpha  = 0.0,
    varphi = 0.6,
    phi    = 25.0,
    link   = "logit",
    freq   = 12
  )

  result_bar1 <- fim_barma(
    y      = y_sim_bar1,
    ar     = 1,
    alpha  = 0.0,
    varphi = 0.6,
    theta  = numeric(0),
    phi    = 25.0,
    link   = "logit"
  )

  # Check positive definiteness
  fim <- result_bar1$fisher_info_mat
  all(eigen(fim)$values > 0)

  # Standard errors from inverse of FIM
  sqrt(diag(solve(fim)))

  # Example 2: Fisher Information Matrix for a BARMA(1, 1) model
  set.seed(2025)
  y_sim_barma11 <- simu_barma(
    n      = 250,
    alpha  = 0.0,
    varphi = 0.6,
    theta  = 0.3,
    phi    = 25.0,
    link   = "logit",
    freq   = 12
  )

  result_barma11 <- fim_barma(
    y      = y_sim_barma11,
    ar     = 1,
    ma     = 1,
    alpha  = 0.0,
    varphi = 0.6,
    theta  = 0.3,
    phi    = 25.0,
    link   = "logit"
  )

  sqrt(diag(solve(result_barma11$fisher_info_mat)))

  # Example 3: Fisher Information Matrix for a BMA(1) model (no AR component)
  set.seed(2025)
  y_sim_bma1 <- simu_barma(
    n      = 250,
    alpha  = 0.0,
    theta  = 0.3,
    phi    = 20.0,
    link   = "logit",
    freq   = 12
  )

  result_bma1 <- fim_barma(
    y      = y_sim_bma1,
    ma     = 1,
    alpha  = 0.0,
    varphi = numeric(0),
    theta  = 0.3,
    phi    = 20.0,
    link   = "logit"
  )

  sqrt(diag(solve(result_bma1$fisher_info_mat)))
}

}
\references{
Rocha, A.V., & Cribari-Neto, F. (2009). Beta autoregressive moving
average models. \emph{TEST}, 18(3), 529-545.
\doi{10.1007/s11749-008-0112-z}

Rocha, A.V., & Cribari-Neto, F. (2017). Erratum to: Beta autoregressive
moving average models. \emph{TEST}, 26, 451-459.
\doi{10.1007/s11749-017-0528-4}

Cribari-Neto, F., Costa, E., & Fonseca, R.V. (2025). Numerical stability
enhancements in beta autoregressive moving average model estimation.
\emph{Brazilian Journal of Probability and Statistics}, 39(4), 410-437.
\doi{10.1214/25-BJPS645}
}
\seealso{
\code{\link{barma}} for model fitting,
\code{\link{loglik_barma}} for log-likelihood computation,
\code{\link{score_vector_barma}} for score vector (gradient)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.barma.R
\name{summary.barma}
\alias{summary.barma}
\title{Summarize a barma Model Fit}
\usage{
\method{summary}{barma}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class `"barma"`.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A list object of class `"summary.barma"`...
}
\description{
(Full documentation block...)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.summary.barma.R
\name{print.summary.barma}
\alias{print.summary.barma}
\title{Print Method for a barma Summary}
\usage{
\method{print}{summary.barma}(x, ...)
}
\arguments{
\item{x}{A fitted model summary object of class `"summary.barma"`.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns the original object `x`.
}
\description{
S3 method for printing the detailed summary of a `"barma"` model object.
}
\details{
This function formats and displays the summary list created by
`summary.barma()`, including the call, coefficient table, and
information criteria.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitted.barma.R
\name{fitted.barma}
\alias{fitted.barma}
\title{Extract Fitted Values from a barma Model}
\usage{
\method{fitted}{barma}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class `"barma"`.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A `ts` object of the fitted mean values.
}
\description{
S3 method for extracting the fitted mean values (mu-hat) from a fitted 
model object of class `"barma"`.
}
\details{
The fitted values are returned as a time series (`ts`) object, 
matching the time properties of the original input `y`. The first
`max_lag` observations are `NA`, as they cannot be fitted.
}
